# coding: utf-8

"""
    Airtable API

    OpenAPI spec for Airtable's REST API  # noqa: E501

    OpenAPI spec version: 2024-12-27
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

from .api.bases_api import BasesApi  # noqa: E501
from airtable_swagger_client.rest import ApiException


class TestBasesApi(unittest.TestCase):
    """BasesApi unit test stubs"""

    def setUp(self):
        self.api = BasesApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_airtable_bases_create(self):
        """Test case for airtable_bases_create

        Create base  # noqa: E501
        """
        pass

    def test_airtable_bases_list(self):
        """Test case for airtable_bases_list

        List bases  # noqa: E501
        """
        pass

    def test_airtable_bases_schema(self):
        """Test case for airtable_bases_schema

        Get base schema  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
