from pydantic import BaseModel
from typing import ClassVar, Dict

class IdRegistry(BaseModel):
    # Properly annotate id_lookup as a ClassVar
    
    id_lookup: ClassVar[Dict[str, str]] = {
  "google_calendar_calendars_insert": "23c64fc1-661e-4c7b-a3aa-497f030ee364",
  "google_calendar_calendars_delete": "59bf3965-9aa5-4869-bac6-cd4dc452ced6",
  "google_calendar_calendars_get": "91531429-afc8-476e-a40d-8a78c084a4f2",
  "google_calendar_calendars_patch": "5feaf5d5-7948-4c33-b0b9-9b8ff24763ba",
  "google_calendar_calendars_update": "d13695fa-16ac-4cba-9b52-8082d8fe53ff",
  "google_calendar_acl_list": "2a1edd60-382b-427c-bcb1-a6a9e63cf77a",
  "google_calendar_acl_insert": "b5cbd079-c54e-4ca7-a811-d5e552c9f6b3",
  "google_calendar_acl_watch": "0e9ec2d4-29bd-44e6-a3ae-8d136ced15cf",
  "google_calendar_acl_delete": "40b0718d-16f3-41b7-8b3f-fb03d0604a48",
  "google_calendar_acl_get": "167f359f-1646-4fe3-b07f-80bc0364ee8a",
  "google_calendar_acl_patch": "8a8fdeaa-c381-4975-b069-7309302a8c31",
  "google_calendar_acl_update": "6d2b9818-e7f1-4b2a-b9c0-cec7dd0227fb",
  "google_calendar_calendars_clear": "233568f0-594b-4a34-bf7d-ce01d3fb546c",
  "google_calendar_events_list": "332a9eec-7dea-47eb-8ffa-38754b7b4b04",
  "google_calendar_events_insert": "2d28468d-5853-48f2-a8eb-d9b0d60af818",
  "google_calendar_events_import": "5ca1716b-34ca-4215-a945-632a4964fb16",
  "google_calendar_events_quick_add": "d32fcc93-d457-4820-be8e-f96551da8074",
  "google_calendar_events_watch": "63164df3-730d-4f29-9f50-4847cf825bb1",
  "google_calendar_events_delete": "52f1cc5b-9971-4e3d-af13-2cd03a3a9c4c",
  "google_calendar_events_get": "d4218736-bb85-4a37-b68c-5749d6d904a5",
  "google_calendar_events_patch": "aec1922b-f891-4a78-b8be-c6fc18984408",
  "google_calendar_events_update": "027e555c-fdcf-439a-992a-bdd710fb55fb",
  "google_calendar_events_instances": "3458b88a-80ec-464b-bb35-c270c2e8000c",
  "google_calendar_events_move": "a028dec8-bd26-4b13-86ee-9e9c6b655d5d",
  "google_calendar_channels_stop": "0e41881b-d4ac-4d3b-8b9b-12f83ee80f4f",
  "google_calendar_colors_get": "e265889e-900d-46b3-88b8-2833b5df8ef3",
  "google_calendar_freebusy_query": "a100cbeb-b44e-4317-8469-64b5e1d6c232",
  "google_calendar_calendar_list_list": "4dc25cdd-9345-4d14-81fa-2975aa46a718",
  "google_calendar_calendar_list_insert": "e55d6679-ff3c-4e1d-b4e1-9d0efb79326a",
  "google_calendar_calendar_list_watch": "86263c65-2334-4575-a86f-f71774f36fb9",
  "google_calendar_calendar_list_delete": "4ad3e931-68f9-439b-88d4-f702425296cb",
  "google_calendar_calendar_list_get": "02aeeafd-fbcb-4552-9adf-4c5e1d2698dd",
  "google_calendar_calendar_list_patch": "649bc943-1ade-4256-afdd-05c5b3a65d0b",
  "google_calendar_calendar_list_update": "643e2ba0-7bfe-4ce2-9a7b-c52d458625a4",
  "google_calendar_settings_list": "79936e55-8ae8-402f-a486-675607fab809",
  "google_calendar_settings_watch": "23808f3d-3364-47b5-bb7c-6b47cf6bb53d",
  "google_calendar_settings_get": "52bdc86f-f7be-4a4b-ad51-7e6a90a46511",
  "nyt_article_search_json_get": "268b0bdf-f685-4800-8c5a-6074f557581a",
  "youtube_data_abuse_reports_insert": "6e66674a-169b-49e3-b9ee-d44b51f2bde8",
  "youtube_data_activities_list": "cf917181-fddf-4b11-91c7-724e0b5d0a74",
  "youtube_data_captions_delete": "1b7a7709-fb43-4687-be59-a48d0e3af1a7",
  "youtube_data_captions_list": "0fb1f531-e88a-4826-9be6-4a087da63adf",
  "youtube_data_captions_insert": "f8a693d5-893a-486f-a4f1-4d816804b2a0",
  "youtube_data_captions_update": "fec161ab-0dc7-4759-89a9-63f9039f5529",
  "youtube_data_captions_download": "de832bb2-89ad-4ede-91d3-db5a57544597",
  "youtube_data_channel_banners_insert": "d8d2b732-f37d-4478-932e-092828d7b305",
  "youtube_data_channel_sections_delete": "49223c69-3765-40ba-a461-2cdeb522de82",
  "youtube_data_channel_sections_list": "ce62daac-ddbe-4cad-98b5-8e43a46a12ff",
  "youtube_data_channel_sections_insert": "0c936210-3451-4e90-aff2-1fa3b3193fdd",
  "youtube_data_channel_sections_update": "e8418a61-af14-44e3-9fcc-d8987f30e122",
  "youtube_data_channels_list": "3e8bcd91-ad55-4404-8f72-b457afe3a846",
  "youtube_data_channels_update": "bb0bd39d-ab9c-419d-98b8-fcfdabc3c3bd",
  "youtube_data_comment_threads_list": "ce270c94-bb54-4ff8-81fe-d4bc9f92bb65",
  "youtube_data_comment_threads_insert": "3989c2ab-d1a8-43c5-ab12-0f216443cd32",
  "youtube_data_youtube_v3_update_comment_threads": "732610f1-2a59-40e7-ab7f-bac251720f17",
  "youtube_data_comments_delete": "4e324040-9662-44d1-a047-08efa2eebeef",
  "youtube_data_comments_list": "844fa072-4854-47df-93d0-9359fad64de3",
  "youtube_data_comments_insert": "5d90225a-170d-4d8a-a442-c7f085ccabd8",
  "youtube_data_comments_update": "189e855e-e3d8-4351-9190-dcdc283c1389",
  "youtube_data_comments_mark_as_spam": "d4c3da7e-a53d-4e26-9ab1-abaaf8cd5900",
  "youtube_data_comments_set_moderation_status": "abba54ae-ce3d-4617-ad32-beefc110d85f",
  "youtube_data_i18n_languages_list": "ce19d68f-e1f3-482c-a590-7073d804a876",
  "youtube_data_i18n_regions_list": "c336b40f-6500-4614-ac95-665b5420de28",
  "youtube_data_live_broadcasts_delete": "9efd75ec-a522-49a8-b142-77e06513d086",
  "youtube_data_live_broadcasts_list": "7ddf3639-91d3-49cb-abd3-ab2ab36f44ef",
  "youtube_data_live_broadcasts_insert": "4649fae7-4aac-4ca8-a2de-37594f98559e",
  "youtube_data_live_broadcasts_update": "88cdd91a-6bef-497a-89c3-2a5c6e576b3c",
  "youtube_data_live_broadcasts_bind": "9e51588b-f117-4789-bf1f-9e60154ed95b",
  "youtube_data_live_broadcasts_insert_cuepoint": "85f0b075-c4cc-4af1-84d5-e517a4f554af",
  "youtube_data_live_broadcasts_transition": "a0e7ffde-588e-448a-b5de-89bba158f825",
  "youtube_data_live_chat_bans_delete": "65ed97fe-6259-4dd9-9ae4-6ce61e446bcc",
  "youtube_data_live_chat_bans_insert": "de21e842-49eb-49c6-b29f-3dcb89fe23aa",
  "youtube_data_live_chat_messages_delete": "638e438d-f9b6-4bd9-9ffd-57a46ad9c286",
  "youtube_data_live_chat_messages_list": "5383207d-9d01-4799-9891-e4e51fdd8b30",
  "youtube_data_live_chat_messages_insert": "0aa2cae3-4ac3-4ab9-8abb-4cc82d3b415f",
  "youtube_data_live_chat_moderators_delete": "188acbd2-d1f6-4a6e-b906-700d3657db4e",
  "youtube_data_live_chat_moderators_list": "5939ee25-449c-42c6-a83a-3ea29b81fec0",
  "youtube_data_live_chat_moderators_insert": "93aec2d3-b733-485b-88e6-a306b80c0009",
  "youtube_data_live_streams_delete": "583b3bbd-c57a-4e41-86e4-086046fdbe94",
  "youtube_data_live_streams_list": "11c36dc4-1b09-4541-a553-06758face74f",
  "youtube_data_live_streams_insert": "a95cd90f-773a-48cb-aef9-e394efcf9bec",
  "youtube_data_live_streams_update": "323bddac-e4f1-40cd-9ca1-a988e9c73225",
  "youtube_data_members_list": "6d1c640b-63cc-40c3-9ea9-5eb0f517067c",
  "youtube_data_memberships_levels_list": "c465557b-3c69-434f-bd2c-452c340f9d16",
  "youtube_data_playlist_items_delete": "08d37613-bacf-4f33-9fd3-9ad8c069e9de",
  "youtube_data_playlist_items_list": "3e3ef84c-6583-41f4-9231-fd423f86372e",
  "youtube_data_playlist_items_insert": "468bdb3d-8f8d-4f0e-9a48-700ae47ca3e2",
  "youtube_data_playlist_items_update": "d47b617c-1652-44d8-90d0-17a9802699cd",
  "youtube_data_playlists_delete": "ccb10df0-37e0-48d3-9347-9f07200966e5",
  "youtube_data_playlists_list": "8ec20190-f918-424d-984e-95f73bcf4085",
  "youtube_data_playlists_insert": "c62eabad-a03e-4842-9d5d-89199cfd6862",
  "youtube_data_playlists_update": "ae8fbdce-f600-489b-9da6-54a5f14e1658",
  "youtube_data_search_list": "242050bb-2d0e-4c67-b9ab-ecf180853e3b",
  "youtube_data_subscriptions_delete": "0aba906b-9b33-4e49-98bb-70111d10ff44",
  "youtube_data_subscriptions_list": "67165cca-44a7-421a-bf11-0285603e8015",
  "youtube_data_subscriptions_insert": "2618e971-6e8e-4005-b207-fbb473a9d9dc",
  "youtube_data_super_chat_events_list": "a195831a-bd74-40eb-a1dc-40c5730c8994",
  "youtube_data_tests_insert": "d7974383-b71a-48c9-9069-ea9d04c711aa",
  "youtube_data_third_party_links_delete": "e7fb0eb8-d8c2-48f5-a19d-9fcf8986c242",
  "youtube_data_third_party_links_list": "9f9a7e5a-cb72-4dd9-821a-74cd79fbdd45",
  "youtube_data_third_party_links_insert": "a12c9156-87d0-4c6f-90ba-acb83f31e2fc",
  "youtube_data_third_party_links_update": "2106d5fc-fe3c-4ee9-aeae-76b06f803216",
  "youtube_data_thumbnails_set": "b36f81d2-9cef-4195-aab3-00afaf382f73",
  "youtube_data_video_abuse_report_reasons_list": "3954e81f-f1ef-4418-ab19-b65bbd66e479",
  "youtube_data_video_categories_list": "4b2a0cbc-4e6e-4d47-b6fd-060b942ce8da",
  "youtube_data_videos_delete": "4cd10ed7-6910-4b6b-b48b-9e0491398179",
  "youtube_data_videos_list": "eb161474-5be4-4990-95e8-536f81e39dab",
  "youtube_data_videos_insert": "409d12eb-ae67-4958-a61b-91b679b8b9d7",
  "youtube_data_videos_update": "3347d8f0-f534-4c3c-bb5c-384e8fd76dc1",
  "youtube_data_videos_get_rating": "9cde234e-3430-4556-8673-704ba78fb2b5",
  "youtube_data_videos_rate": "8cb8958d-1248-4034-91ca-bcda4cf0aad5",
  "youtube_data_videos_report_abuse": "0aecd43f-20b6-4383-8a36-47255bd21533",
  "youtube_data_watermarks_set": "1345e324-7444-4ee4-8822-cf20eba6869b",
  "youtube_data_watermarks_unset": "e8ce0ba4-2c58-49ce-b12b-7c0f6d1a3e68",
  "notion_delete_ablock": "b44c3aa7-cda9-4599-a31b-74f8eb8806f2",
  "notion_retrieve_ablock": "8292cb92-90da-4928-8830-114bbd1517b7",
  "notion_update_ablock": "01193063-f704-4586-99dd-f1d1f5fb188e",
  "notion_retrieve_block_children": "8fd92edf-60ff-4670-892a-bcf70b776313",
  "notion_append_block_children": "5f73c88a-62d4-4b28-96e9-8dceec96feca",
  "notion_retrieve_comments": "3cab9ad5-9818-4b19-a343-1e7bb6f172c9",
  "notion_retrieve_adatabase": "70d0d0ca-da87-49a5-9aed-ef954c263805",
  "notion_update_adatabase": "c3b7496d-1f3b-4dd6-9b78-38a83d6ed292",
  "notion_query_adatabase": "049fb738-1b89-4ba6-9083-8639fc4bb5f5",
  "notion_retrieve_apage": "1b15030a-ba70-4717-b369-9cb672497220",
  "notion_update_page_properties": "3a31d285-56dd-44d5-9c6d-2f735c7ba3a1",
  "notion_retrieve_apage_property_item": "510fcd48-88cf-411c-88f0-fbfcbbcd9782",
  "notion_retrieve_auser": "6e9a79fb-4709-481b-9f28-c6e8e459c271",
  "airtable_get_user_info": "3d82af92-d727-49bc-acc9-5bece2e09898",
  "airtable_records_list": "f2a8b4d6-9c3e-4f7a-b5d1-8e2c6b9a3f7d",
  "airtable_records_update_multiple": "c7e5a9d2-4b8f-3e6d-9a1c-5f8b2d7e4c9a",
  "airtable_records_replace_multiple": "b6d4c8a3-7f2e-5b9d-8e4a-3c6f9b5d2a8c",
  "airtable_records_create": "a5c3b7d9-6e4f-2c8a-9b5d-7f3e8c4a2b6d",
  "airtable_records_delete_multiple": "d8f6e4c2-9b7a-1d3e-5c8f-4a2b6d9c7e5a",
  "airtable_records_get": "e9d7c5a3-8b6f-4d2e-7c9a-5b3f8d6e4c2a",
  "airtable_records_update": "c7d5e3b9-8a6f-2d4c-5b7e-9f3a8d6c4b2e",
  "airtable_records_replace": "f8e6d4b2-7c5a-3e9d-6b8f-4a2c9d7e5b3a",
  "airtable_records_delete": "b6c4a8d2-9e7f-5b3d-4c8e-7f2a9b5d6c3e",
  "airtable_records_sync_csv": "be4ed291-958e-49be-9f84-e458a1745a9f",
  "airtable_records_upload_attachment": "637fc379-3502-44fd-925b-e4d40248d827",
  "airtable_fields_update": "018b13ea-3a01-4608-9d30-3d06f5d35195",
  "airtable_fields_create": "ad675da4-a83c-4151-8431-5c170225ed37",
  "airtable_comments_list": "9398d0f4-0a97-4a89-93ef-fa5af5fd10b0",
  "airtable_comments_create": "d3cce98d-69bd-436a-aaa5-672a80edb0be",
  "airtable_comments_update": "870423ec-dc89-4fcb-ad42-3efd0f023c73",
  "airtable_comments_delete": "ca85acd7-d2ca-4b40-af30-2ade01b8ad46",
  "airtable_tables_update": "7a00cde3-383e-4995-a76a-0f99827d8ee6",
  "airtable_tables_create": "0fe8ca56-21b3-4359-bde8-f927e37b6f40",
  "airtable_bases_schema": "c8a19765-1b16-4df6-a65d-b2eb7e3cd1bd",
  "airtable_bases_list": "cda58275-c1a8-4fc5-96ab-17b26289ed68",
  "airtable_bases_create": "03a28d60-c9a7-418f-b67c-ecbf075cb962",
  "slack_web_api_admin_apps_approve": "2d988273-3c02-4dbb-80ce-eda9397186b0",
  "slack_web_api_admin_apps_approved_list": "ea49d928-212c-47d5-a95a-5bdb554d68ad",
  "slack_web_api_admin_apps_requests_list": "209e5d1f-de3b-4e62-8ccf-fffe630df927",
  "slack_web_api_admin_apps_restrict": "d40c1ba3-9f0e-4617-9096-6f12298c36d5",
  "slack_web_api_admin_apps_restricted_list": "4c43b355-520d-4063-a475-193f3e22e7b7",
  "slack_web_api_admin_conversations_archive": "897574c1-55d4-467b-8157-53ca58a6d240",
  "slack_web_api_admin_conversations_convert_to_private": "db3b8356-269b-4a8e-a4b4-ca07270d3137",
  "slack_web_api_admin_conversations_create": "455b1d34-18ce-45f7-af24-91ffcecc4b48",
  "slack_web_api_admin_conversations_delete": "9e909ead-4734-4060-afd4-1aafcc24ce5c",
  "slack_web_api_admin_conversations_disconnect_shared": "0e6bb466-5c0a-4401-bc0b-b3b5ac86aa57",
  "slack_web_api_admin_conversations_ekm_list_original_connected_channel_info": "41ded63c-7f88-47fd-8083-1d0b4c6e6888",
  "slack_web_api_admin_conversations_get_conversation_prefs": "8ca55dec-27a2-4ff8-af9e-fbd2dca6d3c4",
  "slack_web_api_admin_conversations_get_teams": "a963ecec-2927-4b4a-a77d-f8f05b488e3b",
  "slack_web_api_admin_conversations_invite": "66936658-6388-47c4-9c4a-fe922f0a6641",
  "slack_web_api_admin_conversations_rename": "4b0ce9cb-0386-40d8-aee2-cf9114693284",
  "slack_web_api_admin_conversations_restrict_access_add_group": "c1b2c5d0-e9d0-4dea-8703-57849098abc6",
  "slack_web_api_admin_conversations_restrict_access_list_groups": "7305201a-72c4-4df2-a3f1-f604ae94a896",
  "slack_web_api_admin_conversations_restrict_access_remove_group": "f54bc5a3-cb3b-4c66-a077-419f2c1be685",
  "slack_web_api_admin_conversations_search": "d62f3d11-da1e-4744-ba46-f970ad89fc29",
  "slack_web_api_admin_conversations_set_conversation_prefs": "f67c6bfc-cbac-4701-b892-47bd74bb3df1",
  "slack_web_api_admin_conversations_set_teams": "12f199e5-84f9-4963-8cde-6a6b9cd7e008",
  "slack_web_api_admin_conversations_unarchive": "15caaacb-a1e5-4500-a048-3c5e8f7dc6ee",
  "slack_web_api_admin_emoji_add": "1dff1f42-44f5-47ab-b985-b7f2deb28daf",
  "slack_web_api_admin_emoji_add_alias": "93dd10d8-74df-461b-933c-d5f594a86bc1",
  "slack_web_api_admin_emoji_list": "9868d68e-49e8-437a-8b97-eab44b639460",
  "slack_web_api_admin_emoji_remove": "9c46890f-4876-451d-b329-b967bdb2beaf",
  "slack_web_api_admin_emoji_rename": "97c9bf07-0910-4f23-a61d-10e0a38a7829",
  "slack_web_api_admin_invite_requests_approve": "47ab7560-7b1d-48e7-b05d-481830bf7f62",
  "slack_web_api_admin_invite_requests_approved_list": "8afa24bb-7c1f-41f6-9125-c80498fa73b6",
  "slack_web_api_admin_invite_requests_denied_list": "67593af7-9c0f-4de9-b170-0413cb51c547",
  "slack_web_api_admin_invite_requests_deny": "b5edc6c1-41e7-491d-a1cd-7fdb283af1d0",
  "slack_web_api_admin_invite_requests_list": "ad8fb2e9-957f-4810-842d-75a9c36e7009",
  "slack_web_api_admin_teams_admins_list": "a178015c-8023-4b03-a210-b24bf542303f",
  "slack_web_api_admin_teams_create": "3aa1f92e-4fa6-449a-ba10-47dceb1df8ec",
  "slack_web_api_admin_teams_list": "3c3370fc-9a09-4f9c-bfbb-b02d51e0c010",
  "slack_web_api_admin_teams_owners_list": "76aa4296-e834-48ca-a0d5-f3bbbadd86af",
  "slack_web_api_admin_teams_settings_info": "8ca8f1ec-7358-4e99-a0a7-3dbc8e57a8ec",
  "slack_web_api_admin_teams_settings_set_default_channels": "a59cf694-b587-44ef-8d8e-0bb0ed3ab4d4",
  "slack_web_api_admin_teams_settings_set_description": "c538ce06-0eb5-4666-b188-71c21da521d8",
  "slack_web_api_admin_teams_settings_set_discoverability": "ac9306a5-fd74-4d05-87e2-7cc59bc04ef9",
  "slack_web_api_admin_teams_settings_set_icon": "7d8fa47f-16d4-4aa3-b087-f16726040b47",
  "slack_web_api_admin_teams_settings_set_name": "8d17e673-1ff8-4fc5-80ec-583a800453f0",
  "slack_web_api_admin_usergroups_add_channels": "e98cc7e6-8805-4946-af9e-57c85e300bd2",
  "slack_web_api_admin_usergroups_add_teams": "b710819c-50b6-463f-97da-ec27f1e3464b",
  "slack_web_api_admin_usergroups_list_channels": "6ad0c86f-06ab-44dc-b619-55a98d029cec",
  "slack_web_api_admin_usergroups_remove_channels": "fdd693aa-8c03-43a8-93bd-1ab2d5c5eca9",
  "slack_web_api_admin_users_assign": "e308f6ce-22ab-40ce-8a72-9f6e4ff9e073",
  "slack_web_api_admin_users_invite": "d6538e33-6c1f-41db-87be-17f33399c108",
  "slack_web_api_admin_users_list": "d704e492-b26a-458d-816c-ddb3f716a2e4",
  "slack_web_api_admin_users_remove": "e8b18bd8-7aa2-46ea-af6f-80cd4103bc37",
  "slack_web_api_admin_users_session_invalidate": "c1d3bd32-0bd8-4430-91ca-31403782e4ac",
  "slack_web_api_admin_users_session_reset": "d28fd650-1745-4115-ad43-48b5590fe6e5",
  "slack_web_api_admin_users_set_admin": "7d7cf8ee-c1e0-4ee0-b18a-49bec22d2142",
  "slack_web_api_admin_users_set_expiration": "f6514f00-cda1-4a27-beef-e4426d18a527",
  "slack_web_api_admin_users_set_owner": "a6e69430-0f87-46f8-b673-597f9816d584",
  "slack_web_api_admin_users_set_regular": "1f311687-72e4-4b55-b258-574388984807",
  "slack_web_api_api_test": "f8f14d03-81ab-4ba0-8ac1-dce1ca2939f1",
  "slack_web_api_apps_event_authorizations_list": "8658bdba-da20-4b4b-98a1-77ca88d15830",
  "slack_web_api_apps_permissions_info": "8e9b3729-4c4d-4ae6-8306-2614f7d479e6",
  "slack_web_api_apps_permissions_request": "0086e09d-b934-47a4-9cde-ab07be638a39",
  "slack_web_api_apps_permissions_resources_list": "0a88214d-c35a-4a74-8ef3-3e422357d2f4",
  "slack_web_api_apps_permissions_scopes_list": "d4edc09d-dfe5-4671-a7e6-1f01645df57c",
  "slack_web_api_apps_permissions_users_list": "510a09c9-28ac-42eb-b5bc-39bcdd61a9f4",
  "slack_web_api_apps_permissions_users_request": "a9dbd01c-5b3b-40be-8b4c-0b95c03e2b9d",
  "slack_web_api_apps_uninstall": "6a40f668-8860-4855-b535-5c3fbb22d10b",
  "slack_web_api_auth_revoke": "7554134d-b3b0-47e1-9d17-fbb8ca979bb3",
  "slack_web_api_auth_test": "6eac46a5-3f13-4fc6-96c4-401c6271f43b",
  "slack_web_api_bots_info": "56724622-4a06-471f-9852-e5c5f9305b99",
  "slack_web_api_calls_add": "561130a3-abf3-4b7a-96dc-262c9f33fe6a",
  "slack_web_api_calls_end": "696d63f8-dcd4-41b7-b563-c87efa017f5c",
  "slack_web_api_calls_info": "205ec1ab-f828-41d2-88cc-436677608461",
  "slack_web_api_calls_participants_add": "091679a4-554d-47c0-ace5-ef80b136f4fc",
  "slack_web_api_calls_participants_remove": "9f8b5e14-0478-4e2b-bf36-b6673bb95bf7",
  "slack_web_api_calls_update": "3912525f-7bc2-4cc9-9efd-80af484dfc89",
  "slack_web_api_chat_delete": "a47a84b7-29fb-4b40-9f37-0dc396ce7d85",
  "slack_web_api_chat_delete_scheduled_message": "44e4e59d-501f-4dc6-a9e8-038777ba8c1c",
  "slack_web_api_chat_get_permalink": "80c6cace-de35-4091-b069-b26561302258",
  "slack_web_api_chat_me_message": "e1246f3a-d308-4cf0-b52f-aeaaff2546f8",
  "slack_web_api_chat_post_ephemeral": "60267f29-bccf-47e9-a2ce-7d66cdf3cf50",
  "slack_web_api_chat_post_message": "03745bdc-8751-40e7-88e0-6af6ca99e2b8",
  "slack_web_api_chat_schedule_message": "420a4fee-330c-4701-92fa-4792860fef8d",
  "slack_web_api_chat_scheduled_messages_list": "d3e036c5-68da-4963-9be2-bf46a61bb312",
  "slack_web_api_chat_unfurl": "050054da-36e6-4bba-ba80-d55cb1cc837c",
  "slack_web_api_chat_update": "904773eb-140d-4065-879d-670492a3a935",
  "slack_web_api_conversations_archive": "a7141f73-b5cf-47a7-ab03-4394626a1fbe",
  "slack_web_api_conversations_close": "823e5802-3f4a-42d4-a8f8-f7d9b33d4ca3",
  "slack_web_api_conversations_create": "a52b4ef6-4ae5-45e0-adab-57d072b034a0",
  "slack_web_api_conversations_history": "906e8d87-cef2-4e62-a367-cf791b84e06d",
  "slack_web_api_conversations_info": "daa6d78e-1a4c-4a07-b245-2b6ccddbf485",
  "slack_web_api_conversations_invite": "8242f6ee-b7a0-4708-9375-7ee27ae48a33",
  "slack_web_api_conversations_join": "bd1d7048-3166-43c7-8a07-c13d9f405630",
  "slack_web_api_conversations_kick": "43727777-792f-4572-bf8c-04e418bd12f8",
  "slack_web_api_conversations_leave": "09057030-607b-4452-b1fa-2c99b0299b55",
  "slack_web_api_conversations_list": "77c9f68b-d48e-4e41-adc6-9931c4e08e8f",
  "slack_web_api_conversations_mark": "18bb58b1-ad32-42c1-accd-4d8e660c9d0e",
  "slack_web_api_conversations_members": "17ed2dca-dd28-4be1-8d9a-a67d2ca1b5dc",
  "slack_web_api_conversations_open": "6f121113-9f08-4bb5-b8d1-883b19b6de3a",
  "slack_web_api_conversations_rename": "9c76a23c-9024-4e71-87cf-5349ddb1937e",
  "slack_web_api_conversations_replies": "9adf4e13-83b8-4b5b-ad0f-4152e14af376",
  "slack_web_api_conversations_set_purpose": "eac63de8-dd23-4450-9341-68f5f3c5deaf",
  "slack_web_api_conversations_set_topic": "19b38015-b7da-4e41-bd20-79bca9bfbcdf",
  "slack_web_api_conversations_unarchive": "8d0299f6-a43b-430b-8b7b-2fe27c0ded55",
  "slack_web_api_dialog_open": "2380b7eb-9bae-4a06-aaf6-3c4e551f0732",
  "slack_web_api_dnd_end_dnd": "d78c0e98-fac3-4114-bdf2-b4be24dc4cde",
  "slack_web_api_dnd_end_snooze": "d26d9c91-1c1c-4791-baf6-38ac35e71b3c",
  "slack_web_api_dnd_info": "6712e8af-1b72-4b25-b68b-a65041f6f479",
  "slack_web_api_dnd_set_snooze": "5f5f119d-7c68-49bd-8e7f-df60764be6d5",
  "slack_web_api_dnd_team_info": "d61a2f39-6959-4820-be19-0274c42646f1",
  "slack_web_api_emoji_list": "aed76ce1-3ff5-4817-ad1d-1ebb6e1ef9b7",
  "slack_web_api_files_comments_delete": "b77bb3c0-9543-4fc8-865b-f3f8bedffae5",
  "slack_web_api_files_delete": "c58b80be-d759-4793-85dd-5bf8bad6d4c8",
  "slack_web_api_files_info": "905ff0a1-17b3-413e-b0cc-bf07f20979b2",
  "slack_web_api_files_list": "35bdb847-51b9-4512-ab59-726eb58c03d4",
  "slack_web_api_files_remote_add": "4a4b11db-7e21-4112-9dcc-c83179e79d3c",
  "slack_web_api_files_remote_info": "eae2c2d5-dc14-4373-9302-3d8cce4473f1",
  "slack_web_api_files_remote_list": "109c4330-c134-4ae7-987e-15ccc03b59b8",
  "slack_web_api_files_remote_remove": "6e3cbcc3-503d-46ce-bf82-8a2e1139fbb6",
  "slack_web_api_files_remote_share": "676cf9c7-b500-4a72-86c0-0efa1dc47ad0",
  "slack_web_api_files_remote_update": "2d2f60c2-9674-46ff-8321-ce2d64208378",
  "slack_web_api_files_revoke_public_url": "fca2ff0b-3462-424d-9fd3-0cda740228bb",
  "slack_web_api_files_shared_public_url": "e4b3427b-e17f-4522-859a-f5edfaaf3b92",
  "slack_web_api_files_upload": "08d48527-aaff-411b-9a3e-b358a7a2ad83",
  "slack_web_api_migration_exchange": "a7f37606-f56f-43da-914c-b948d5c06e28",
  "slack_web_api_oauth_access": "29098431-9024-4419-b6b9-098b05d4c54f",
  "slack_web_api_oauth_token": "6d82add8-52aa-4065-ae4e-3756a7bc0347",
  "slack_web_api_oauth_v2_access": "67e469ae-2679-45c0-977f-0210f95a6653",
  "slack_web_api_pins_add": "b4d20f5d-1796-417d-be0d-68e1f681d70e",
  "slack_web_api_pins_list": "784b0fc9-79c3-4767-8720-f01262309813",
  "slack_web_api_pins_remove": "bcd76f70-486f-4314-8b84-a01bd018570b",
  "slack_web_api_reactions_add": "e01bd93a-b4d1-40d9-b3f4-a5f72b02b9b6",
  "slack_web_api_reactions_get": "697be0f9-c1b7-4220-b82b-f664051f8d01",
  "slack_web_api_reactions_list": "4a4cd900-bd75-44ed-9189-2eb3d24215fa",
  "slack_web_api_reactions_remove": "7fd6c325-5444-404c-894a-9a6e13507719",
  "slack_web_api_reminders_add": "b6921860-fc34-44ac-928c-d85c5cc9fce3",
  "slack_web_api_reminders_complete": "502a2872-3d92-4fb0-a27d-3f972fe501d9",
  "slack_web_api_reminders_delete": "ef31e971-04ae-4302-a31a-4b3e146730bb",
  "slack_web_api_reminders_info": "d816eb0d-7829-4ebc-9d65-db5dc0244fe5",
  "slack_web_api_reminders_list": "69100e6e-06da-4416-9001-9d3b3c44fcfa",
  "slack_web_api_rtm_connect": "da3dc2d1-d49e-4a94-a965-668094fe44a5",
  "slack_web_api_search_messages": "dbb35b5e-45db-4773-944e-a30dfb44b2f2",
  "slack_web_api_stars_add": "5dac944e-babb-4815-b983-43222649f648",
  "slack_web_api_stars_list": "1f0db219-7a49-47e3-8963-b30cf39ce9d4",
  "slack_web_api_stars_remove": "add237f7-27f1-4ce3-918d-2900324848e8",
  "slack_web_api_team_access_logs": "fadf63e2-646c-42a3-893b-81282f259f99",
  "slack_web_api_team_billable_info": "b410797c-f153-48ed-9ef6-753466d6a3dc",
  "slack_web_api_team_info": "450d5fe5-214a-4ec5-b7a3-5c1ce019210a",
  "slack_web_api_team_integration_logs": "af19f9ab-ea8d-4ced-91ac-6d8c9e8ebbce",
  "slack_web_api_team_profile_get": "23e3b115-5ac8-4820-9aaa-792886b79f23",
  "slack_web_api_usergroups_create": "54f0f0d3-5bc2-4a43-ba13-a55e941a5223",
  "slack_web_api_usergroups_disable": "0c1d737d-df91-4dc5-ada1-18c7e2c45544",
  "slack_web_api_usergroups_enable": "46de3d09-12c6-4e38-9d08-84b05af8f1db",
  "slack_web_api_usergroups_list": "674bd393-57df-4a7c-a2b7-63ad9e10e848",
  "slack_web_api_usergroups_update": "39da7cf2-4053-4ee6-8ce8-abb921e572b2",
  "slack_web_api_usergroups_users_list": "e065f8ee-83d4-4428-9fdc-dfaeba83f78a",
  "slack_web_api_usergroups_users_update": "e99a0d6e-1657-4bf0-836b-0b2a4085dbad",
  "slack_web_api_users_conversations": "2247afd4-7abc-4937-9d41-bae66e25d926",
  "slack_web_api_users_delete_photo": "6d47a470-ba8d-492f-9018-fcfdf54d4f4c",
  "slack_web_api_users_get_presence": "1f24f954-0176-4d1b-8e94-bdda5755aab7",
  "slack_web_api_users_identity": "4dab75cb-a5f8-4f8a-b403-c6cceabd74d2",
  "slack_web_api_users_info": "1561a995-bbbd-49f8-a3f0-4d08f99222d3",
  "slack_web_api_users_list": "84cef0df-6686-4fdf-bf6f-1aa39af282b9",
  "slack_web_api_users_lookup_by_email": "1f27abf6-57a7-4550-a10a-589e10a74bdb",
  "slack_web_api_users_profile_get": "807847b7-38a1-4952-8a73-ad4a6aa757ad",
  "slack_web_api_users_profile_set": "078659eb-de74-46a9-b083-8c2c42c4f503",
  "slack_web_api_users_set_active": "89ea6143-0b5c-4f47-b4d3-e0d1f3bdd8bf",
  "slack_web_api_users_set_photo": "3597578b-2ef1-4ece-bea5-c5f341fe3b45",
  "slack_web_api_users_set_presence": "979bf218-0c29-4abe-a182-c55dd13570cf",
  "slack_web_api_views_open": "83768320-21af-4320-b526-f4369b12ea99",
  "slack_web_api_views_publish": "5890b5f0-916a-41c4-bec5-667bd83be8c0",
  "slack_web_api_views_push": "565bcf4f-55e1-4437-bc29-9e9d85df9bd1",
  "slack_web_api_views_update": "354048d2-4135-4fec-994f-d82171cfd698",
  "slack_web_api_workflows_step_completed": "ca34131e-d991-4eea-b90d-454389a08f79",
  "slack_web_api_workflows_step_failed": "6315e24e-f4b1-4afe-970f-9608b82225f7",
  "slack_web_api_workflows_update_step": "dd3bc4dd-4bd4-4d94-998d-6469c93bbe3c",
  "zoom_accounts": "ecb22f58-80d7-462c-abca-7aae7f6b895c",
  "zoom_account_create": "da2c2bac-ca71-4ed6-8a29-44600ba3606d",
  "zoom_account_disassociate": "badca3d8-51b5-4d71-a168-10dbd29a3da8",
  "zoom_account": "94ce0ddb-3d8a-4e34-a42c-1101620b83ce",
  "zoom_account_billing": "d6e4c46e-dc11-4692-ba6f-40c05ccdd1a5",
  "zoom_account_billing_update": "29d787b3-c4bb-4ec5-8f13-c7af349693ae",
  "zoom_account_billing_invoices": "0416ecf6-043b-412b-b0f8-2f4ab7148f97",
  "zoom_get_account_billing_invoice": "311dbf21-4c71-4e81-9989-f5c135d48e40",
  "zoom_get_account_lock_settings": "7a2a6762-de12-495e-b790-238f0d6cb8a6",
  "zoom_update_account_lock_settings": "dd862e4a-380f-4bc2-b6d0-85fd2c4ce185",
  "zoom_account_managed_domain": "89a4f542-f5db-4247-8333-e2d395ecceb4",
  "zoom_account_options_update": "fc684d91-2d66-4b70-bcf1-0b8f42af39dc",
  "zoom_update_account_owner": "563dcb09-af86-4076-8285-4f741eea7b0a",
  "zoom_update_phone_settings": "3f340237-81a1-4b8f-8587-7ce8ab7a7185",
  "zoom_set_up_account": "7d9b8335-bf83-4134-86c5-a341416cc5a0",
  "zoom_post_phone_siptrunk": "00bf57b5-bb9d-487d-895c-da7754d93ac4",
  "zoom_update_phone_siptrunk": "67382817-1cf8-476d-b72b-f70026d26e59",
  "zoom_account_plans": "63a3f311-6055-4fe6-bbe8-90b10c10c06b",
  "zoom_account_plan_create": "9da81c16-5a7c-4f36-8856-8385cc191483",
  "zoom_account_plan_addon_create": "d331dd44-b702-4197-a46a-7bef71bdd5a3",
  "zoom_account_plan_addon_update": "225c331f-935d-43c0-936f-562d70a03a30",
  "zoom_account_plan_addon_cancel": "0d4d0204-a790-4c40-b9c8-1ec3c45954a8",
  "zoom_account_plan_base_update": "9e14a6e7-05b1-4e73-b709-f3aba65fda4d",
  "zoom_account_plan_base_delete": "b6539ff0-0103-4ccb-a8c2-aa3d9149dd66",
  "zoom_get_plan_usage": "026716e5-ffa9-4c04-8339-426a02f977bc",
  "zoom_get_account_cloud_recording": "43841fe4-dbe1-4f3f-a9dc-5fc2b02c153a",
  "zoom_account_settings": "1f88ce76-b944-457b-b18a-887f5e36627f",
  "zoom_account_settings_update": "de8f85e4-c655-48c8-90e5-310777258284",
  "zoom_del_vb": "799f4496-c489-41cb-a394-725a04e89792",
  "zoom_upload_vb": "550a918d-d2dd-464a-9bff-1e0d52f3129c",
  "zoom_list_internal_callout_countries": "d7dfdc35-f315-4c13-a741-767cd07c85c3",
  "zoom_add_callout_countries": "e030f1de-afd6-472c-bc33-66f683bda62d",
  "zoom_delete_internal_call_out_country": "6870109a-83ae-4c34-be18-9904c0623119",
  "zoom_list_internal_numbers": "ab2b53a7-f2de-40dc-bd27-96ce62711b99",
  "zoom_add_internal_numbers": "e8bea3ea-e31c-4c4b-9da5-47e1942b6fc3",
  "zoom_delete_internal_number": "73383dcc-89ee-45ae-924d-fec7d576d403",
  "zoom_delete_all_sip_numbers": "362ae8ce-9434-40f3-850d-82ff664bd06f",
  "zoom_assign_sip_trunk_numbers": "218e7694-4e5b-4c8e-a790-53a532f70400",
  "zoom_assign_sipconfig": "67d62721-44ff-42ce-8fd2-77df0aa3ce39",
  "zoom_list_siptrunks": "6dd42ccb-3153-45be-aad1-5a2ceb69ba5d",
  "zoom_assign_siptrunks": "1937dbbf-7978-4de6-b5fc-dbffb5c8278a",
  "zoom_delete_siptrunk": "e37b8551-d0cf-4af1-8b2a-b1c7a47a06f0",
  "zoom_account_trusted_domain": "039ba639-ddc5-4557-a531-efb7dcc645c5",
  "zoom_switch_user_account": "fdf10e07-9353-4520-860b-ed3c987961ca",
  "zoom_download_invoice_pdf": "12504257-32cc-4add-baf4-48db9c91a41c",
  "zoom_list_archived_files": "dda35518-8167-4b53-8678-61dfe850864c",
  "zoom_delete_user_level_channel": "58e69feb-e1f0-46a6-b817-7cc22db3bf4b",
  "zoom_get_user_level_channel": "7f799d5f-75ac-49b4-aa93-e5904459e672",
  "zoom_update_user_level_channel": "4d93017d-dfdf-4fea-bacd-42e20c774980",
  "zoom_leave_channel": "30e4765e-991e-429b-8c69-96ee939bdf82",
  "zoom_join_channel": "920f86cb-6c9b-4805-9683-d0a3e9ee04c1",
  "zoom_remove_auser_level_channel_member": "0f67c9f9-4ff6-41b0-beb4-2789eb7ab93c",
  "zoom_get_user_contacts": "4e92a1c0-11d0-470b-ad3e-c5a7852652b3",
  "zoom_get_user_contact": "7091bc54-140c-4091-8700-52bf17317035",
  "zoom_get_channels": "0408cd34-20ea-4581-b133-aead61b5c087",
  "zoom_create_channel": "a8c0ff33-4d66-4db2-97d1-b42456f02c5b",
  "zoom_delete_channel": "ac0153fb-a7e3-4fe4-9149-4b1606f76fe5",
  "zoom_get_channel": "30660431-17ae-42b0-8b39-ce9c813735e6",
  "zoom_update_channel": "883de544-19dc-4656-af1f-0e5c9ec3f917",
  "zoom_list_channel_members": "7ac8483c-781f-4689-8819-d2684faab838",
  "zoom_invite_channel_members": "17e6d05f-c96a-4490-a417-9831ba14f12e",
  "zoom_remove_achannel_member": "943e82ca-9dd8-4914-8497-b7c0f579e236",
  "zoom_get_chat_messages": "5f653cc7-42bc-41fe-b34e-524ae103a5f9",
  "zoom_senda_chat_message": "e38253fc-83ea-44ae-bbb5-16cd569fdd59",
  "zoom_delete_chat_message": "d76bf65a-3c4c-456d-95f8-fd1a5df83ca8",
  "zoom_edit_message": "f54e6a44-9cee-49b8-9179-7a52cb43ccb7",
  "zoom_search_company_contacts": "60091a4a-83ce-49cc-ad91-e26a6bacf14e",
  "zoom_groups": "2fe7c6dc-1d9e-4212-98fa-5f3fb7b4779e",
  "zoom_group_create": "733128c1-50c4-4a5d-b74e-43a49929532d",
  "zoom_group_delete": "dc72a8a0-6fd2-4c50-82cb-e2a5763b0af8",
  "zoom_group": "c45397c7-1cb3-4642-aa99-ad8d6760353d",
  "zoom_group_update": "8c54bebe-4490-4c62-a5ca-2745e65e4b43",
  "zoom_get_group_lock_settings": "8f13b49b-0c9e-4531-8cda-540e92f46fbc",
  "zoom_group_locked_settings": "6c33f3f1-82ac-42f9-8948-34542c7992ef",
  "zoom_group_members": "ec492127-2917-4972-a6c7-3ec785ff7c69",
  "zoom_group_members_create": "101c84ad-507c-4115-9ac2-7ccad3aa42fe",
  "zoom_group_members_delete": "47f08b6e-636d-412c-b460-27b3e7ce1a10",
  "zoom_update_agroup_member": "1132d490-7bb3-498c-96df-c925edc370b8",
  "zoom_get_group_settings": "d067a443-b4b0-4ae5-9a46-67e257a08a1a",
  "zoom_update_group_settings": "a64c872a-8865-4c99-b7c7-84a40a91901c",
  "zoom_del_group_vb": "ec9086d7-8457-4677-9df5-6e959a58f4eb",
  "zoom_upload_group_vb": "e99ff3ef-d7e5-4ebf-a06e-7c530bcadd41",
  "zoom_device_list": "9ec78ff0-5c1d-482d-b2cc-fcebef74849a",
  "zoom_device_create": "03e23ff5-7981-4ef2-a501-65d26fe26094",
  "zoom_device_delete": "0acc3257-bddd-468f-84a8-a53617982df8",
  "zoom_device_update": "9a993e24-e70c-463e-b8e3-a01ab9f92cf0",
  "zoom_sendchatbot": "92d3837e-3082-4251-8e5b-856e1c26d7ca",
  "zoom_delete_achatbot_message": "eaf1971c-2dc3-42c3-a5c3-d724125d6703",
  "zoom_edit_chatbot_message": "98c5f9a4-e743-48a2-bffc-50554318d17e",
  "zoom_im_chat_sessions": "7347eed8-ddac-4123-8ddf-4a877271145f",
  "zoom_im_chat_messages": "93e31ebf-70c6-42c0-80bc-0457c04c6670",
  "zoom_im_groups": "059f4e77-7334-4a04-96ea-b7c7989fe421",
  "zoom_im_group_create": "9aa05e2a-1877-4999-8309-2e1a609a5348",
  "zoom_im_group_delete": "711e0cb7-bd75-4401-b7f3-d49f6cf75c7b",
  "zoom_im_group": "024f30a3-a47c-4542-b165-02f908369a25",
  "zoom_im_group_update": "4eded50c-0539-4d00-96cb-4eaa9509092c",
  "zoom_im_group_members": "f90766cb-6a93-4be2-9e49-035b0939530e",
  "zoom_im_group_members_create": "7fc4ee3c-a3e1-4c9b-a4ca-e1cb7cab6ef2",
  "zoom_im_group_members_delete": "ccb1ec47-f17d-4e8f-8a9c-fefc3bce309b",
  "zoom_sendimmessages": "5d832853-d0f1-4268-a072-84f1bafa1aff",
  "zoom_listimmessages": "34d431b2-0f66-4cf2-b864-fc41ac733a43",
  "zoom_in_meeting_recording_control": "6e989114-34e9-42aa-819f-3a21afd0e31b",
  "zoom_meeting_delete": "3f1e3964-f537-4304-ac68-e90ce3638083",
  "zoom_meeting": "c0db67f3-5532-42f8-b667-565ed65d7852",
  "zoom_meeting_update": "5f71ffb7-bdb0-47dd-b87b-6bd8204b7e77",
  "zoom_create_batch_polls": "efbeb2dc-45c9-4358-8ce6-19d045a4638d",
  "zoom_meeting_invitation": "7ad97667-2e1d-48f8-89b4-d23b81200bf5",
  "zoom_get_live_stream_details": "f42364b3-e8dd-4743-8c58-285722fe2060",
  "zoom_meeting_live_stream_update": "f2bcfcef-7041-444f-8388-46b79f92e71a",
  "zoom_meeting_live_stream_status_update": "8fac005f-8388-4a51-bead-d80d68204f0b",
  "zoom_meeting_polls": "dab65e5f-a84e-4314-bc38-b9691d1a117b",
  "zoom_meeting_poll_create": "b6b446dc-91bc-474d-8787-b77ce67157b1",
  "zoom_meeting_poll_delete": "7786ad04-1873-40d3-9612-d35f515a77aa",
  "zoom_meeting_poll_get": "fbc37865-13b8-493b-b089-d5f2b0ee7653",
  "zoom_meeting_poll_update": "047e8d5f-62bf-4a38-a0ef-fb22dd3639e8",
  "zoom_recording_delete": "f17feb1f-5efc-473e-849e-863bf31fcc83",
  "zoom_recording_get": "d819aceb-5008-4c87-939a-29008746a670",
  "zoom_meeting_recording_registrants": "5ffdbb5c-4055-4b82-ade5-0a5f54b250bf",
  "zoom_meeting_recording_registrant_create": "72e82a4e-ecc7-4384-bc29-a386bbdc03eb",
  "zoom_recording_registrants_questions_get": "61f07669-bca1-4a5f-945b-306ce760e8cf",
  "zoom_recording_registrant_question_update": "6959bc32-376a-446c-91dc-ab171f9ae435",
  "zoom_meeting_recording_registrant_status": "99f154c5-0b89-4a4a-81a4-6cfc096dd100",
  "zoom_recording_setting_update": "1dc1e61d-8116-401a-9c3a-da2f582c06dd",
  "zoom_recording_settings_update": "981e7684-175c-486f-a785-6f9790569645",
  "zoom_recording_status_update": "ea4a28e3-daf4-4516-9542-650df3348130",
  "zoom_recording_delete_one": "98951cc8-db1f-492a-933a-e626f3753aca",
  "zoom_recording_status_update_one": "b593b6d4-6189-4514-b8c6-460521d5514b",
  "zoom_meeting_registrants": "12e27afa-204c-4c7a-9dd2-7426dd6abda2",
  "zoom_meeting_registrant_create": "dd62f8a5-484f-408f-92aa-747215cb28ad",
  "zoom_meeting_registrants_questions_get": "046a56ca-b2fd-410b-b4a8-1dadf7442d18",
  "zoom_meeting_registrant_question_update": "a678412c-be86-4fbb-9f4b-1790457311df",
  "zoom_meeting_registrant_status": "21b98f93-5560-497a-9ebb-77960146b43c",
  "zoom_meetingregistrantdelete": "46b198ef-3377-408f-98f9-7d04e8ffe6cb",
  "zoom_meeting_status": "5bbb9b3d-9e88-47e3-a26a-6c5fba571ac8",
  "zoom_dashboard_client_feedback": "d4ed5447-47a0-4635-af03-619e2ed0a120",
  "zoom_dashboard_client_feedback_detail": "e6792cbd-8a21-4120-9d73-6c5ab0a60bf8",
  "zoom_list_meeting_satisfaction": "07efd72d-a52e-4882-b172-f78effa48e99",
  "zoom_dashboard_crc": "56d059f9-e68b-40d5-82f4-7e7705f86686",
  "zoom_dashboard_im": "790fa2d4-5099-4496-8ab4-3df85a85eeb6",
  "zoom_dashboard_issue_zoom_room": "4d168f2f-039f-4426-b9b8-691f01d16359",
  "zoom_dashboard_issue_detail_zoom_room": "43b55fc6-e3a3-4dd8-8ccc-e79542c0b3f6",
  "zoom_dashboard_meetings": "677350e3-0bad-4244-8a2b-d3d53c958318",
  "zoom_dashboard_meeting_detail": "c471c966-489d-4194-9bb2-074bc552dc0c",
  "zoom_dashboard_meeting_participants": "0320800b-67e3-4d04-9eae-e4c3d2b57a13",
  "zoom_dashboard_meeting_participants_qos": "61d2d8ee-88a8-4c5a-839f-dd3574459c54",
  "zoom_participant_feedback": "c05f2685-4e83-421c-a964-362099ffa570",
  "zoom_dashboard_meeting_participant_share": "ea6dfde7-ce76-41b1-bf06-b554fcfd4452",
  "zoom_dashboard_meeting_participant_qos": "ef613773-a8b9-4b38-b85f-869890cd5390",
  "zoom_dashboard_webinars": "6c47e024-5400-48a6-8053-f7adc1ce02c7",
  "zoom_dashboard_webinar_detail": "d85fb978-835e-4db1-ae07-9a2397aeffa9",
  "zoom_dashboard_webinar_participants": "6b81e8f3-28a1-437a-b620-598627bf5b7d",
  "zoom_dashboard_webinar_participants_qos": "f0ef5f24-982a-4299-87cd-2a054e0e09db",
  "zoom_participant_webinar_feedback": "8cc3e117-2976-4e47-9252-4c0f10f3c5f2",
  "zoom_dashboard_webinar_participant_share": "5dfc8579-899a-44ec-847c-53fca2b107c0",
  "zoom_dashboard_webinar_participant_qos": "1833c35a-374f-4698-851f-f4cd7caf308d",
  "zoom_dashboard_zoom_rooms": "cc8f15d2-b8a1-4af6-8cce-5b94ba79673f",
  "zoom_dashboard_zoom_room_issue": "cad719aa-9006-4a9d-abef-e2a52f1eef25",
  "zoom_dashboard_zoom_room": "d61c9048-b682-460a-9a35-0e211e99a1d1",
  "zoom_list_past_meeting_files": "0d37c73f-de2c-4a08-afb8-1624c83c4d4d",
  "zoom_past_meetings": "24660866-16e6-4bd4-b837-6b5f3cf945c8",
  "zoom_list_past_meeting_polls": "744d4a8f-a4b4-4fb3-ab58-d667f9b4a28c",
  "zoom_past_meeting_details": "0ba98a83-ca1b-4adc-88e3-b907a5bc2522",
  "zoom_past_meeting_participants": "298c2da0-e287-4d5e-a68f-44b661960226",
  "zoom_webinar_absentees": "92e6f89d-6567-4c5d-b843-1c08a27686b3",
  "zoom_list_past_webinar_files": "a2166075-3098-453e-8281-df26010c471b",
  "zoom_past_webinars": "2e1db055-31b0-4631-b96a-fc2d17288049",
  "zoom_list_webinar_participants": "987c85f5-219d-4393-a963-187f6bed427d",
  "zoom_list_past_webinar_poll_results": "747b1f59-aeaa-4dd5-aad0-67c5bd81d2e2",
  "zoom_list_past_webinar_qa": "f6926aac-c5a7-4dab-b97b-88a311058f60",
  "zoom_add_auto_receptionist": "27407441-f7fc-4b98-800e-29ba012c709b",
  "zoom_update_auto_receptionist": "74e87381-7d85-47c6-8f30-adbc50e18643",
  "zoom_unassign_all_phone_nums_auto_receptionist": "92093889-c272-4421-b9bc-a4492c39b8bc",
  "zoom_assign_phone_numbers_auto_receptionist": "30a1513e-3a96-49e0-8c83-ef9f859c5c24",
  "zoom_unassign_aphone_num_auto_receptionist": "57bed949-6a0b-40ac-a49f-11a7cf6db057",
  "zoom_list_blocked_list": "18396e42-55c6-445e-b8ef-29602294ec26",
  "zoom_add_anumber_to_blocked_list": "61cb0001-8242-4881-939b-6bcbb11d503f",
  "zoom_delete_ablocked_list": "b9b080b3-a266-40e2-b353-2cb6123cc95f",
  "zoom_get_ablocked_list": "c6c146f4-7b2f-47ce-b860-5c2d752297c9",
  "zoom_update_blocked_list": "50fdd1d8-fc89-439f-a0b3-aaf551eee1e1",
  "zoom_add_byocnumber": "859c12ce-bae8-44f0-9b16-a20c0783fa29",
  "zoom_account_call_logs": "54417a4c-df9a-4da3-83f9-18ce2ecaf37a",
  "zoom_list_call_queues": "0eae8114-4fde-4124-a655-bc7838e0b6ef",
  "zoom_create_call_queue": "8718802f-da0b-4ec6-921c-8f6cebf8f1a5",
  "zoom_delete_acall_queue": "441c966b-8763-4dc9-8bca-8c82ef837f6d",
  "zoom_get_acall_queue": "a98aadf3-07e1-4cc3-82c5-ccf3c9749d91",
  "zoom_update_call_queue": "64ea0ac0-8bb5-4146-abaa-a530f33ed838",
  "zoom_change_call_queue_manager": "66baa2b5-c21e-43a7-8afe-d04057ea539b",
  "zoom_unassign_all_members": "84d209c6-3dec-40a0-9266-dda90cebf2e3",
  "zoom_add_members_to_call_queue": "7fd09976-885f-4b2d-9775-a3639de45e74",
  "zoom_unassign_member_from_call_queue": "e4a7ecd5-88de-4de5-8994-b69d725dcf20",
  "zoom_unassign_aphone_num_call_queue": "80d59db8-03f2-45ab-bc9f-8e5ed3d401e8",
  "zoom_assign_phone_to_call_queue": "1e809099-586a-42b1-b388-95f49acbfe36",
  "zoom_un_assign_phone_num_call_queue": "d2bce754-37c5-4c1a-8c0b-20525ceadd4d",
  "zoom_get_call_queue_recordings": "5ef8b09e-3f62-4744-bcf6-20c8004ad401",
  "zoom_list_calling_plans": "c889659b-882a-4ba8-b857-5b2752c27909",
  "zoom_list_common_area_phones": "1d5bf878-fd35-4233-a92e-618fe0836b7c",
  "zoom_add_common_area_phone": "73acc3ac-b3b7-4f0f-8619-63ce4592ebf9",
  "zoom_delete_common_area_phone": "2396345d-389e-45ec-85f1-e4bfb6406584",
  "zoom_get_acommon_area_phone": "6f71e1af-eb80-481b-a783-e6fb76211726",
  "zoom_update_common_area_phone": "e9728179-3bd6-46f1-b410-b838c7b10603",
  "zoom_change_main_company_number": "5dbb1019-04a3-47df-86fb-c8a52338dabe",
  "zoom_list_phone_devices": "bee0cff8-3c1c-478a-b7c6-59a6fe7e821a",
  "zoom_add_phone_device": "5cd19496-f8a4-4a21-bffb-269a3f6cc840",
  "zoom_delete_adevice": "13916b5e-ceb5-46e7-9464-27169ab59e1d",
  "zoom_get_adevice": "cdda10a4-f9b3-4781-bcb4-234ba3a354c6",
  "zoom_update_adevice": "8210dbb9-3ee2-4411-adb1-f4ab0c12d6b4",
  "zoom_list_call_logs_metrics": "cd759185-6561-4439-ae0a-d5a0051879de",
  "zoom_get_call_qo_s": "7b81809c-c83e-41d9-87cc-58ffb407da6d",
  "zoom_get_call_log_metrics_details": "eb107d46-6fc8-4720-9ba5-42e1a5dfe7e1",
  "zoom_list_account_phone_numbers": "d2975798-767d-402e-838b-8ee261250977",
  "zoom_get_phone_number_details": "6d0a947f-a6d8-4c37-96e1-4c8e7a2c98a8",
  "zoom_update_phone_number_details": "58d0b9bb-ea1c-417e-af7f-708205721ffd",
  "zoom_get_phone_recordings": "32014f6e-2e38-4fcc-80b8-277012a21a40",
  "zoom_get_psoperation_logs": "a595db7d-610a-4307-bbe4-678116a79968",
  "zoom_list_setting_templates": "0defa4fb-930b-480c-a6cc-d8c23c71d22e",
  "zoom_add_setting_template": "4a691978-b399-4070-b58f-b3052ecadde2",
  "zoom_get_setting_template": "6a84c054-4df6-4715-84d0-3e28717edded",
  "zoom_update_setting_template": "32c66331-5f12-4158-ba3a-e3581513f384",
  "zoom_list_shared_line_groups": "e8e1b92c-53f8-44e6-b90c-fc8d8de2fc8d",
  "zoom_create_ashared_line_group": "709dfd16-950b-441e-ad78-23132cc8d6b7",
  "zoom_delete_ashared_line_group": "eead0eed-9160-47b9-917a-62e26114e82a",
  "zoom_get_ashared_line_group": "6384a284-3d8c-45bc-851e-c5cfcace6b27",
  "zoom_update_ashared_line_group": "1ac1fb39-e9a7-4fa7-a106-6b8cc129ffe8",
  "zoom_delete_members_of_slg": "2b382ea0-ae5f-49c2-b979-e4c5cc3f7bde",
  "zoom_add_members_to_shared_line_group": "58273a39-1983-4dcb-b9bd-2bd373cd8662",
  "zoom_delete_amember_slg": "fb7ca455-67fb-4057-91ae-6f5a36cef2e2",
  "zoom_assign_phone_numbers_slg": "44f3755e-7680-4874-8a29-078bc8db4f8b",
  "zoom_delete_aphone_number_slg": "b4d9607e-3056-4122-84e0-3cf5f63b3d43",
  "zoom_list_byocsiptrunk": "6e46a726-af4d-484f-b1c1-d9a3cc1d253d",
  "zoom_list_phone_sites": "d05474fb-05c0-4551-bebf-a909f0e4c50e",
  "zoom_create_phone_site": "3ee11d90-ba6d-4640-b569-ede35d983cf5",
  "zoom_delete_phone_site": "181bce04-4471-4861-b560-8988bb22b81a",
  "zoom_get_asite": "f19e9c8f-b59e-4781-bfe6-582603ce91d7",
  "zoom_update_site_details": "e1a8e406-b3a9-43fd-a48e-8be4d4760d37",
  "zoom_list_phone_users": "6cd83650-12fe-40b5-b238-f14e85d316aa",
  "zoom_phone_user": "0ec5674d-0bd9-4a49-b220-3466110fa554",
  "zoom_update_user_profile": "7a22e02d-c9e2-4be6-a746-43eb4c32e657",
  "zoom_phone_user_call_logs": "0381d878-d9ac-49bf-bbfa-403f52b25fb2",
  "zoom_delete_call_log": "c6b4aeb0-e10e-4f04-8a36-8dd7f77a2b36",
  "zoom_assign_calling_plan": "db32ab5e-b87a-407e-ba3f-c9b77eba87c4",
  "zoom_unassign_calling_plan": "8a1ca86c-53ca-4603-a0bd-7431314a0f12",
  "zoom_assign_phone_number": "7da95968-a24d-46ff-a706-6242dea2b6e2",
  "zoom_unassign_phone_number": "8949092c-cd86-4ccf-982d-b2607907c066",
  "zoom_phone_user_recordings": "9d6d2b89-b472-4859-a0d8-b5a2e504ef3c",
  "zoom_phone_user_settings": "1d543183-3b2e-483c-8a8a-ccb2eeb6ef6f",
  "zoom_delete_user_setting": "d6166815-30f5-437c-82f6-0e8557c209ba",
  "zoom_update_user_setting": "ffa49027-0aeb-45d2-9f0f-1f43ef287cdd",
  "zoom_add_user_setting": "90d396fc-ebfb-47a9-af42-1807f725cf58",
  "zoom_phone_user_voice_mails": "9aae8bb0-7f4c-4d76-879a-4e912d5c10c9",
  "zoom_delete_voicemail": "3d69c029-19fd-4dd3-b633-362298d9ae6e",
  "zoom_report_sign_in_sign_out_activities": "e84c90a5-eb11-47e1-aa09-358757c1b604",
  "zoom_report_cloud_recording": "ed52787b-e7b9-41ff-8b92-264aff751772",
  "zoom_report_daily": "9eb1069b-fd30-4b34-b3e4-86ddf39e2947",
  "zoom_report_meeting_details": "2932bc4c-eb16-4429-bd8a-b0e5ec9210cd",
  "zoom_report_meeting_participants": "b5e21b0b-0de1-4854-b466-581ee5e69aa9",
  "zoom_report_meeting_polls": "55003b75-d61b-4564-8b47-b4464c8f5cf5",
  "zoom_report_operation_logs": "8edeb43c-ceb3-42a3-bc19-ed8b1a902e2c",
  "zoom_report_telephone": "aa785c23-5f21-44ba-b4f1-e05c1ca98356",
  "zoom_report_users": "231fb5e9-440d-4793-80e8-0ab14bc33c72",
  "zoom_report_meetings": "d7a8c150-5ed0-4847-abf5-90ef3e96cdfe",
  "zoom_report_webinar_details": "7a8c73f5-b175-4609-8ff4-bfeb27fd4856",
  "zoom_report_webinar_participants": "ea45080b-1e76-4b86-ab6c-6635a4999f05",
  "zoom_report_webinar_polls": "89005a01-41b3-440c-ae9c-3ec24066c13d",
  "zoom_report_webinar_qa": "4d41101c-7d99-4e4e-bd19-6d5d5603410c",
  "zoom_roles": "27eef70d-9545-4440-9a6f-3ed9bec277e1",
  "zoom_create_role": "49c96920-c659-429c-8730-1be6f487cc52",
  "zoom_delete_role": "eabdd02e-5866-4d53-a0ff-3ef64850ad41",
  "zoom_get_role_information": "58e0923b-d88b-48f2-a091-eeecee9ece2d",
  "zoom_update_role": "147cadd1-f695-4dad-9fd9-cac4b33689b1",
  "zoom_role_members": "5050551a-c50f-44f4-bf26-ff8880814abf",
  "zoom_add_role_members": "fdb0eba4-98c0-4606-ae02-e519ef0679a7",
  "zoom_role_member_delete": "392698c1-77f8-40f6-9ffb-4ecfb39a78e9",
  "zoom_list_zoom_rooms": "eca00daa-e4c8-4ef4-b7db-e37540441fed",
  "zoom_add_aroom": "43458ec7-da52-4b05-bb04-e7b3002503e1",
  "zoom_get_zraccount_profile": "332f43e3-ec57-4b99-a9dd-6a5c2d001072",
  "zoom_update_zracc_profile": "3d49f21c-2652-4b74-a4e4-e3ae6f93c5f1",
  "zoom_get_zraccount_settings": "e5dad23e-fac7-4d04-a72b-0d7660e0fbb3",
  "zoom_update_zoom_room_acc_settings": "86dee441-abbb-4798-8c5d-3f3be5869e25",
  "zoom_list_digital_signage_content": "ea75bf21-aeef-414b-9a82-637e2abf1dbe",
  "zoom_manage_e911signage": "4edb3330-0d2e-44ff-903b-741e606fe3c2",
  "zoom_list_zrlocations": "13b296e1-f998-4396-96dc-25c139e83953",
  "zoom_add_azrlocation": "8faf4e5c-8dd2-4577-a6f1-1843d5e528de",
  "zoom_get_zrlocation_structure": "2767b866-91cd-444e-94cf-e3419f21fc2c",
  "zoom_update_zoom_rooms_location_structure": "92d7638c-6518-4f79-af64-15344fe74060",
  "zoom_get_zrlocation_profile": "537849c1-5686-4a13-87e5-8ecbb1c9f95d",
  "zoom_update_zrlocation_profile": "47f022b7-04f4-478b-b413-bc7bebf451b5",
  "zoom_change_parent_location": "6280ae98-de65-491c-a3b1-46bcbce8d575",
  "zoom_get_zrlocation_settings": "07168367-7b0f-41b4-8d1e-62c691bacae3",
  "zoom_update_zrlocation_settings": "3f4d7305-007f-4eae-947d-8f26ef854bca",
  "zoom_check_in_rooms": "223fb05a-21ed-4620-8606-5322c74970dd",
  "zoom_delete_azoom_room": "1c46701a-a6a9-4452-8fd4-2e511a8e5110",
  "zoom_get_zrprofile": "4d3b7cb1-a6d1-451f-8d91-e51a89f4e249",
  "zoom_update_room_profile": "93dda470-8cef-4b2b-ac21-507b4ab30279",
  "zoom_list_zrdevices": "bc38cd79-9a1d-4805-a711-ed5a72d59eb7",
  "zoom_change_zoom_rooms_app_version": "9ed60cb2-11ff-4627-9a2d-7a3ed9a33cd8",
  "zoom_change_zrlocation": "5885d52e-7426-409a-9b61-bb236df58f98",
  "zoom_get_zrsettings": "df9e3772-2b31-4580-bdd2-e6bc638cb5ad",
  "zoom_update_zrsettings": "82e2a22a-75aa-4c74-96ce-f14964a33d34",
  "zoom_list_sip_phones": "81b8f862-bdbd-47f2-9b9a-8b69b6cad1c5",
  "zoom_create_sipphone": "31869977-099f-44c3-9dc2-8eedf07aa4bf",
  "zoom_delete_sipphone": "94538b31-eea0-4cf4-b56f-6b859e159983",
  "zoom_update_sipphone": "0e6e73e6-1894-44d0-98fb-47d5c2fe890e",
  "zoom_list_sip_trunk_numbers": "2493fd76-9ec4-41da-8ac3-82e1e6b55ab3",
  "zoom_trackingfield_list": "d11c19fa-ccb7-437e-be25-a04c17be65d3",
  "zoom_trackingfield_create": "ba6686cf-414d-43fe-a68c-5ad659621308",
  "zoom_trackingfield_delete": "a80f473f-3b99-4e67-8fe8-ecc9e050bac3",
  "zoom_trackingfield_get": "d49e6080-81c6-484e-a5d5-2adf7eb5a2f3",
  "zoom_trackingfield_update": "f11df85d-c0d9-4727-b2ac-a91479633e03",
  "zoom_tsp": "79925ca8-a17f-4099-a549-91172dd4bdd4",
  "zoom_tsp_update": "a5587f8c-e111-45e0-a824-a2278cccc134",
  "zoom_users": "567e5419-d0dd-4d7d-846d-23e5353c711e",
  "zoom_user_create": "d411fb68-90bf-454c-a0a8-6dbb0c58d3bb",
  "zoom_user_email": "fd2ec2c9-8fe8-44c2-9b9a-f900faea1bfe",
  "zoom_user_zak": "fcc23030-ba01-49ff-a2c5-28f263bc2c86",
  "zoom_user_vanity_name": "27ad4fd7-334f-4c0d-bd72-102b6f6dad5e",
  "zoom_user_delete": "dd93f077-4a90-4d0c-acf6-21c4eab05b52",
  "zoom_user": "6fe97037-b202-4aeb-8000-1fb64beff955",
  "zoom_user_update": "b90a4091-62dd-461d-b2e8-dd4822a2f7b8",
  "zoom_user_assistants_delete": "95a71177-d5f3-414f-9881-d3d06ccc46d0",
  "zoom_user_assistants": "d37b732e-c5e3-4b0c-8c64-08e941440e62",
  "zoom_user_assistant_create": "156f43dc-1bf4-4f96-9fae-cac65df8162a",
  "zoom_user_assistant_delete": "41884502-f6f1-4bd9-aa29-a52dcb352fe9",
  "zoom_user_email_update": "a23fd82f-d98b-4b32-8da8-dca56d92696e",
  "zoom_list_meeting_templates": "19561718-dae2-4d87-9979-8686611d316e",
  "zoom_meetings": "3c78c1f8-7b89-4d71-bda0-9a2c268a0750",
  "zoom_meeting_create": "9e9c4957-99cc-496b-864f-072b0a41e5eb",
  "zoom_user_pacs": "db360a46-279d-4a0f-8082-6ce76dc114e3",
  "zoom_user_password": "9c02ab22-b533-41f9-908b-6e5e948d3a8e",
  "zoom_user_permission": "722b2741-eb32-4f5d-9503-3889cb7ebcc7",
  "zoom_user_picture": "8f4c4e97-8b8c-453a-b6ff-6401e913ee81",
  "zoom_update_presence_status": "2a6a5ecf-b47b-4dc3-a03d-725dd11eaf12",
  "zoom_recordings_list": "e1cdfc94-e370-45b6-8113-d4d9d9b5b90e",
  "zoom_user_schedulers_delete": "c6988496-0661-4311-8544-ea979ece0d96",
  "zoom_user_schedulers": "7680025b-36f7-4ff5-bef0-b42c98ac7f8a",
  "zoom_user_scheduler_delete": "a2ccbfd1-7a81-41ef-b71c-2b0c95b3037e",
  "zoom_user_settings": "400564e3-8447-41b4-b87c-8df0afd5beaa",
  "zoom_user_settings_update": "5e551a1a-5fcc-4ce5-aecc-4126e5f6674f",
  "zoom_del_user_vb": "2969a972-a082-407c-adfb-79314145d75d",
  "zoom_upload_vbuser": "97a7e9fd-df09-4ae0-9cf7-d1123b26fff6",
  "zoom_user_status": "5db1ffdf-a6e2-4a4b-a8d0-3eea299f899e",
  "zoom_user_ssotoken_delete": "e64e8cea-c47a-4939-8698-0303521ca5a2",
  "zoom_user_token": "f8032cc8-742c-4865-8f9a-22b7ea7634f3",
  "zoom_user_tsps": "3a1221fd-1315-4b8a-8223-11744d41128c",
  "zoom_user_tspcreate": "4d078036-274c-4849-86a2-d284436bc6cd",
  "zoom_tsp_url_update": "416147b3-f6a7-4428-b4e6-7f996f0eab03",
  "zoom_user_tspdelete": "d06ea9ea-8698-4c67-b5f8-b38ed51372b7",
  "zoom_user_tsp": "b42dd5f1-6704-4c42-ad2b-15bc6c17d7d0",
  "zoom_user_tspupdate": "5fca1b20-40d4-4d3a-a73f-53bdb105e394",
  "zoom_list_webinar_templates": "ac906faa-bd67-4de1-8aa6-ee0cba980ce5",
  "zoom_webinars": "55471697-ac59-4611-8306-01162dacb10b",
  "zoom_webinar_create": "cbfe9e07-0b1a-4999-954d-a46c880a9355",
  "zoom_webinar_delete": "b809546a-1388-42d3-bb5c-fab6b4c72847",
  "zoom_webinar": "b8f37f80-1ce0-435d-b41c-1aaf2024cd1e",
  "zoom_webinar_update": "32fe971a-34df-44ea-bc42-b5ae72d5f447",
  "zoom_add_batch_webinar_registrants": "75c55e73-d10a-4836-acba-9e45b3951309",
  "zoom_webinar_panelists_delete": "dd4ce7c1-6b5f-4409-8208-a504e725a87a",
  "zoom_webinar_panelists": "3ea5f2cb-bdc7-4d25-92c5-9bfaf4898f80",
  "zoom_webinar_panelist_create": "179e1f18-698e-4d2d-871b-a02498b5bc4e",
  "zoom_webinar_panelist_delete": "7985e934-56c7-44a1-befc-35215e878203",
  "zoom_webinar_polls": "d84e932a-08f0-4282-8c0e-18521739ded2",
  "zoom_webinar_poll_create": "d4462c5d-dad1-48d0-83ff-3a4004bce024",
  "zoom_webinar_poll_delete": "0894cb55-86ab-4b9b-9fba-0ca08ce7da07",
  "zoom_webinar_poll_get": "e5d69349-f504-4d55-984b-60a20c321768",
  "zoom_webinar_poll_update": "46b3d17c-f8b0-4366-a342-ddf85e112422",
  "zoom_webinar_registrants": "8e3239b1-a792-4449-9734-e9fa69f2c967",
  "zoom_webinar_registrant_create": "779e7e49-fad5-48d2-87a1-eb4bd563bc19",
  "zoom_webinar_registrants_questions_get": "f624805b-bae6-41c5-8765-c7ff4cd9ee7b",
  "zoom_webinar_registrant_question_update": "66107c0e-5fc4-4dc4-8d9f-7d121d2ae1db",
  "zoom_webinar_registrant_status": "8113dcfb-4202-4858-8207-0dd5bef87863",
  "zoom_delete_webinar_registrant": "93bb2288-4bfb-4247-97cb-1f4593aeb69c",
  "zoom_webinar_registrant_get": "8244dc34-d210-412f-8e84-cd577d43e752",
  "zoom_webinar_status": "acca6897-d5b6-4d27-8ed9-d27a0e4cc798",
  "zoom_get_tracking_sources": "f414ae85-5369-40d8-89cf-7f02459dd4ba",
  
  "brave_search_get_web_search_results": "664ffa48-80ba-4eaf-8798-8953aeec2708",
  
  "gmail_users_drafts_list": "713c5cd5-edef-42be-b2d1-e70b6c2d056c",
  "gmail_users_drafts_create": "d87694b5-fa5a-4c51-95f9-de371e522b8b",
  "gmail_users_drafts_send": "1b9b9323-bb98-4838-97aa-5591818b1566",
  "gmail_users_drafts_delete": "039d47a4-2246-42d1-ac9f-e0c81be66289",
  "gmail_users_drafts_get": "e9896feb-0037-4bac-9bbd-b92d7afc8e95",
  "gmail_users_drafts_update": "bc5b022e-5c23-4fdc-8853-06f5947372a7",
  "gmail_users_history_list": "76a81f4d-8de7-4d92-a798-ad293e8daeee",
  "gmail_users_labels_list": "d74b5bd0-d674-4f9b-a091-eabf44a3bc8c",
  "gmail_users_labels_create": "4e823c42-2b89-41b8-ba9a-e9d4920b9544",
  "gmail_users_labels_delete": "18352eb9-a826-499d-b0c2-891198e0ad88",
  "gmail_users_labels_get": "ff22ca9b-624c-4d1c-9980-47e5eb5c2fce",
  "gmail_users_labels_patch": "eb15b5b7-2b75-4cc8-9591-d216f56c5f43",
  "gmail_users_labels_update": "7348de2c-bf2f-45b9-84fe-902a82f7e96e",
  "gmail_users_messages_list": "cddc2eeb-5c6f-459b-a550-18f0bd432068",
  "gmail_users_messages_insert": "11505bcb-2f0a-40c7-b0be-46619ed9f7ef",
  "gmail_users_messages_batch_delete": "bad5232b-43ee-470d-9730-2216d563c36d",
  "gmail_users_messages_batch_modify": "3519c37a-4fed-43d8-83b9-1095fa2dfb18",
  "gmail_users_messages_import": "ae6b9ac0-b93c-401a-9191-429b002a0b41",
  "gmail_users_messages_send": "b6f57ab4-1f59-47d8-b0e6-e42c2451d96f",
  "gmail_users_messages_delete": "e6fd6614-55d5-4ee3-bcb4-5bf57694a5b5",
  "gmail_users_messages_get": "8d044926-45bc-4c2b-bb9a-26a580290824",
  "gmail_users_messages_modify": "287a55a6-9f59-40c6-8b6b-733e3ccb184e",
  "gmail_users_messages_trash": "a12fbd23-ba53-4550-b6f0-4fb191717e5f",
  "gmail_users_messages_untrash": "bf186bff-e4a6-41b4-a89c-541055d44b64",
  "gmail_users_messages_attachments_get": "08b872de-4bd0-4da8-bc11-e594567cd960",
  "gmail_users_get_profile": "b006fb08-a1ad-4bf3-89cb-25df5579fad2",
  "gmail_users_settings_get_auto_forwarding": "ad9d54b8-1617-4d2a-9eea-c9b54c1d0939",
  "gmail_users_settings_update_auto_forwarding": "d39bbf90-a79e-42ae-88d4-9f0d9ef6cc13",
  "gmail_users_settings_cse_identities_list": "7774aa89-c638-4efc-a5e9-ce6172337d43",
  "gmail_users_settings_cse_identities_create": "9772c847-bc72-4114-a4e0-3b0db02bdfcb",
  "gmail_users_settings_cse_identities_delete": "da4862a9-6422-4ea3-a11e-3aa9ca92f1eb",
  "gmail_users_settings_cse_identities_get": "893191f6-657e-47b5-840e-d3c3d1125b79",
  "gmail_users_settings_cse_identities_patch": "5fc5ac2f-1ede-4ea9-affe-3b690dcb1923",
  "gmail_users_settings_cse_keypairs_list": "eff5e249-58cb-4dc1-8413-f6ea872e9d66",
  "gmail_users_settings_cse_keypairs_create": "d99db2d5-e55e-482e-b42e-9a7d8584c56a",
  "gmail_users_settings_cse_keypairs_get": "30b4254e-b823-49a4-addc-1e590899440c",
  "gmail_users_settings_cse_keypairs_disable": "b0d6fa80-9f25-460a-9100-cea4cd1cf02b",
  "gmail_users_settings_cse_keypairs_enable": "c96007ce-a37a-4067-8d49-108f897469d5",
  "gmail_users_settings_cse_keypairs_obliterate": "97490061-bdb1-4f21-acb5-612f2f0a6290",
  "gmail_users_settings_delegates_list": "3e311642-fea9-4252-9270-17cd44004759",
  "gmail_users_settings_delegates_create": "d86edd9d-5102-49ff-8b8b-262a12ef85d3",
  "gmail_users_settings_delegates_delete": "7e0e024f-4aa6-4450-8f89-045653e5b181",
  "gmail_users_settings_delegates_get": "3170ed96-ec7b-43ca-806c-4716d9c43d94",
  "gmail_users_settings_filters_list": "9f2bace1-e0aa-46c3-b9ac-badef9ba981f",
  "gmail_users_settings_filters_create": "ca28f74e-30fa-4fd5-af70-9be808580ddb",
  "gmail_users_settings_filters_delete": "cda9bba8-a1e7-4d51-9d2d-c5b2b63ca436",
  "gmail_users_settings_filters_get": "9d112070-3a34-4073-9b63-f93ae61bbc59",
  "gmail_users_settings_forwarding_addresses_list": "71678aad-c1c4-4262-97e4-dacf016ee38f",
  "gmail_users_settings_forwarding_addresses_create": "077dd080-2c45-485c-9031-206a4f545c77",
  "gmail_users_settings_forwarding_addresses_delete": "4d0ce1cf-1513-44d4-b03f-83848552b62b",
  "gmail_users_settings_forwarding_addresses_get": "7f5363de-5def-472a-bc98-b01b89736976",
  "gmail_users_settings_get_imap": "bcccc292-e663-491f-a0c6-73902a88359e",
  "gmail_users_settings_update_imap": "9eab50ec-167d-4977-8428-1e6cf2549512",
  "gmail_users_settings_get_language": "79a4f40a-8c2a-4407-9f0f-04f7a58a1e49",
  "gmail_users_settings_update_language": "3056b3e5-a89a-490d-b122-5c9abf0177a0",
  "gmail_users_settings_get_pop": "1de11f18-fb2f-4612-abca-92172406a261",
  "gmail_users_settings_update_pop": "a94e04f8-7ad5-42e5-8a19-ce5bcfbefbb2",
  "gmail_users_settings_send_as_list": "700e9ae2-6d82-4662-8aca-85684099bf74",
  "gmail_users_settings_send_as_create": "955832e3-ab21-4d6e-8e9d-18415c631b7a",
  "gmail_users_settings_send_as_delete": "177dddb6-3d20-43a8-adff-d1343f085130",
  "gmail_users_settings_send_as_get": "95ee71d9-f699-4963-bc3b-6dc2a3863a61",
  "gmail_users_settings_send_as_patch": "5cd76e75-f2b2-4790-8e2a-ad64eeb98bee",
  "gmail_users_settings_send_as_update": "e27c6a8c-624b-4051-9798-9642896c3de5",
  "gmail_users_settings_send_as_smime_info_list": "562de6ea-dcdb-4a3d-b5ce-f8782cda8145",
  "gmail_users_settings_send_as_smime_info_insert": "3fe81f34-ec29-4ee1-9ea9-1990887ab92d",
  "gmail_users_settings_send_as_smime_info_delete": "b80c11c2-b5bb-430a-9577-79194931ec70",
  "gmail_users_settings_send_as_smime_info_get": "4a2d2c5f-c36b-46cb-a029-2ea98c821b1f",
  "gmail_users_settings_send_as_smime_info_set_default": "da5f494c-5de5-432c-a6fa-5061d355218c",
  "gmail_users_settings_send_as_verify": "c0a40a52-89a9-44e3-bc69-d5be74df7ef6",
  "gmail_users_settings_get_vacation": "196ba81c-97bb-4b72-9499-0cf6953b8e12",
  "gmail_users_settings_update_vacation": "4a7304d5-59e0-496d-ae6a-77bda6c0f691",
  "gmail_users_stop": "5a6e0240-4dc1-48fc-8f89-87401edb6d4c",
  "gmail_users_threads_list": "511214f8-0897-464c-803e-2566f4d15197",
  "gmail_users_threads_delete": "764ac696-304b-44bc-9b4f-0c8b29029807",
  "gmail_users_threads_get": "7fc7f6b7-8d35-403e-8b6f-82c026d24e31",
  "gmail_users_threads_modify": "713974c5-580e-4afb-9eea-dda4539a0875",
  "gmail_users_threads_trash": "75c575df-76bb-4804-80a0-4455a0182b77",
  "gmail_users_threads_untrash": "b6016052-02d8-4a19-b6e0-9187529bf690",
  "gmail_users_watch": "8b7dc0ea-d946-441f-8b8c-3f33755ae4bf",
  "stripe_get_account": "bc81d3d4-647d-4eb8-aa00-7b544a62ccc4",
  "stripe_post_account_links": "54b236a4-8959-4c44-b397-7ef28f0affae",
  "stripe_post_account_sessions": "9d2c0e32-4e05-424f-b455-560ac1a8126a",
  "stripe_get_accounts": "75da9b18-f2a0-46c8-bb8b-f31ae39818cd",
  "stripe_post_accounts": "3fe4a10d-1827-410f-8df8-ca6af12f96f8",
  "stripe_delete_accounts_account": "e489f402-210d-477f-813d-903f496b591a",
  "stripe_get_accounts_account": "f7a2515f-ed95-4ad6-a4b0-2c692e2827d8",
  "stripe_post_accounts_account": "1c25db01-3a1e-47b4-b6ce-c4aa5a4095b3",
  "stripe_post_accounts_account_bank_accounts": "19b05605-cd67-4ec1-a257-a55c52e94c06",
  "stripe_delete_accounts_account_bank_accounts_id": "1feb4981-3070-4c54-8475-81a0c4fbd2a5",
  "stripe_get_accounts_account_bank_accounts_id": "5582b61b-3c22-44de-aabd-7837fcae2672",
  "stripe_post_accounts_account_bank_accounts_id": "92b5fe26-3f00-455d-a6c0-7fcee8330a14",
  "stripe_get_accounts_account_capabilities": "136e4902-941c-4f91-976b-292ed6145493",
  "stripe_get_accounts_account_capabilities_capability": "1c7b5690-57ad-4284-970c-3a2ff8c35b98",
  "stripe_post_accounts_account_capabilities_capability": "d32d46c9-4751-4c48-b279-5042df5bd288",
  "stripe_get_accounts_account_external_accounts": "6e1dc5fa-07c6-4051-a3a3-7570c4e8788a",
  "stripe_post_accounts_account_external_accounts": "7f151fb4-ca97-46ad-851a-dcca49676d86",
  "stripe_delete_accounts_account_external_accounts_id": "13f40634-4cf5-4180-9f1d-2e4a8b0d1ac3",
  "stripe_get_accounts_account_external_accounts_id": "84b45838-d178-4a1a-9fb6-97a4b11a0445",
  "stripe_post_accounts_account_external_accounts_id": "8050b200-3381-4853-900f-80641804877f",
  "stripe_post_accounts_account_login_links": "b870b03d-97f6-46e0-9c68-814f38e366e2",
  "stripe_get_accounts_account_people": "897d1d7d-05ad-4315-8a73-2967a74fffc7",
  "stripe_post_accounts_account_people": "3deb84be-73a2-4b64-b2c9-6566d8cdf425",
  "stripe_delete_accounts_account_people_person": "5f8ec176-0554-4408-bc32-b7223028eed6",
  "stripe_get_accounts_account_people_person": "15b6b0f4-2e5d-4a5a-9dbd-6a55f5f7596a",
  "stripe_post_accounts_account_people_person": "93ce643e-9169-4944-9279-7bef6f7430c6",
  "stripe_get_accounts_account_persons": "137934c2-3052-42fd-80ad-492af31843bc",
  "stripe_post_accounts_account_persons": "53fc68a0-7327-43b7-95cb-fdffad528243",
  "stripe_delete_accounts_account_persons_person": "5b7f2bac-46f6-47eb-be07-03ee61a0363d",
  "stripe_get_accounts_account_persons_person": "0ce11d95-4c44-4514-9947-1279e613a093",
  "stripe_post_accounts_account_persons_person": "2571dabc-ac12-41de-864f-3b442a052a89",
  "stripe_post_accounts_account_reject": "3e56659d-274a-4422-b379-5e9099894166",
  "stripe_get_apple_pay_domains": "4caf917d-b89c-4408-ac84-984081673bad",
  "stripe_post_apple_pay_domains": "19016a18-4539-46d2-81d0-03ed007049b0",
  "stripe_delete_apple_pay_domains_domain": "17995f4b-79af-43a1-aed8-cc4b49b38326",
  "stripe_get_apple_pay_domains_domain": "176f2530-9d84-403a-b7ee-f812d2d9dc59",
  "stripe_get_application_fees": "bd087299-29af-4d7c-9ead-b182ef9decea",
  "stripe_get_application_fees_fee_refunds_id": "2a757567-efc4-4c8c-b9f7-f0b104f334b1",
  "stripe_post_application_fees_fee_refunds_id": "625bd4b8-ee15-4fc4-b5c8-c958c52219b7",
  "stripe_get_application_fees_id": "cf594e8c-655a-4323-aac3-a32327b52e95",
  "stripe_post_application_fees_id_refund": "c23bb53d-5f27-40cd-8b7a-a6c0545e1045",
  "stripe_get_application_fees_id_refunds": "85403007-f851-4c91-91d6-487380f5bc92",
  "stripe_post_application_fees_id_refunds": "c9bb72b1-e55d-4d36-80f9-e0d8c3c6076d",
  "stripe_get_apps_secrets": "9344d33f-763b-4d14-9b19-374fe21cc173",
  "stripe_post_apps_secrets": "9924e02e-3b86-42e4-96ff-d34962df87e0",
  "stripe_post_apps_secrets_delete": "d3a1c2b8-dc06-4142-b998-1df8b91a6c33",
  "stripe_get_apps_secrets_find": "028db81d-0840-4d70-8e95-e71ccd270ec6",
  "stripe_get_balance": "dd1b8eb1-fb80-4c19-b1c2-a8523a24553e",
  "stripe_get_balance_history": "bb1e4216-5865-45b7-aa44-24fc70662d8c",
  "stripe_get_balance_history_id": "468a1a31-9096-4daf-a08c-370791cb9ba5",
  "stripe_get_balance_transactions": "02516369-5b57-485c-b5a7-bb1a653e2296",
  "stripe_get_balance_transactions_id": "dd065d73-6843-4f47-8f1a-512a40f0d3a3",
  "stripe_get_billing_alerts": "2a9e0c91-6e34-44f3-8902-b8ef9d634946",
  "stripe_post_billing_alerts": "a035dbae-3443-42f6-8b91-a141765790a6",
  "stripe_get_billing_alerts_id": "767f8189-43c9-4dd4-b838-2478e961917f",
  "stripe_post_billing_alerts_id_activate": "08a3f248-03e2-4181-a8a7-a18c0a584bc9",
  "stripe_post_billing_alerts_id_archive": "cbab23cd-5815-4883-a164-f37c689ff76e",
  "stripe_post_billing_alerts_id_deactivate": "d653dda0-050e-4c88-8f5f-bc12312b71fa",
  "stripe_get_billing_credit_balance_summary": "8a9123b0-7c22-4f63-a1b5-eb23de1c2bbb",
  "stripe_get_billing_credit_balance_transactions": "5a162963-2218-43a4-8c84-a7335cd3188d",
  "stripe_get_billing_credit_balance_transactions_id": "81b23992-146a-48c9-8491-451f2b8ee906",
  "stripe_get_billing_credit_grants": "60c07bda-b2d4-41b0-8989-28521ad760a8",
  "stripe_post_billing_credit_grants": "b083f527-b14b-4dde-bfdb-ec1c8bdc4c38",
  "stripe_get_billing_credit_grants_id": "a154ad1f-8cab-45cb-9e92-a126e28126cd",
  "stripe_post_billing_credit_grants_id": "d5a2b707-b303-46c4-94b4-7e6581a4f275",
  "stripe_post_billing_credit_grants_id_expire": "0cded6ae-c978-49e3-8e80-b94b8f54d9d0",
  "stripe_post_billing_credit_grants_id_void": "c0c13db7-3bd6-4a8a-bff2-8e572ac14d95",
  "stripe_post_billing_meter_event_adjustments": "9acae2f8-1fd1-4968-ae26-139e467fdf9a",
  "stripe_post_billing_meter_events": "ac4ca6eb-df05-41bf-8a6b-1376b6cb03bf",
  "stripe_get_billing_meters": "2b9da966-49a7-4027-a70a-e4af94f0806c",
  "stripe_post_billing_meters": "0bfcc07c-2939-4f5f-8871-dd1cdf3f1361",
  "stripe_get_billing_meters_id": "254e3828-3e07-40b8-b63a-fb36df478128",
  "stripe_post_billing_meters_id": "aa82007e-1adf-4408-ab31-7306f6afee75",
  "stripe_post_billing_meters_id_deactivate": "1b18c326-f952-43f4-899b-146226fc49e7",
  "stripe_get_billing_meters_id_event_summaries": "c3359d25-8141-48d7-bd85-cc2777256c51",
  "stripe_post_billing_meters_id_reactivate": "88864ded-03fb-446d-a277-26beab5e7cc5",
  "stripe_get_billing_portal_configurations": "62620530-9d72-49af-a808-fbec768765be",
  "stripe_post_billing_portal_configurations": "128a3d61-ab4f-4914-8553-0c72a933d517",
  "stripe_get_billing_portal_configurations_configuration": "3966c382-7ade-408e-b22b-b530e861bf08",
  "stripe_post_billing_portal_configurations_configuration": "935cae83-e0af-4912-9519-cbbff56f5259",
  "stripe_post_billing_portal_sessions": "eba3ffff-05a6-453d-bb1d-4319fda9ae89",
  "stripe_get_charges": "9f8889cb-8f04-4c21-9031-178d6d02620c",
  "stripe_post_charges": "cdbfe58d-08fa-4a44-8a28-a05b96b99048",
  "stripe_get_charges_search": "a0a1d37c-d88d-4c1a-a108-d9cb5b306224",
  "stripe_get_charges_charge": "88ade440-0784-4843-9cf1-1fa63f8fe015",
  "stripe_post_charges_charge": "196651db-fe8b-4d33-8cdf-002d794f432b",
  "stripe_post_charges_charge_capture": "92fb2e65-ae61-4ae4-a5b6-63ac3ecdcf4e",
  "stripe_get_charges_charge_dispute": "34e53402-d7ea-47a9-b06e-9523db2b413e",
  "stripe_post_charges_charge_dispute": "cb9a0420-0a27-4c59-93f0-ccd857886747",
  "stripe_post_charges_charge_dispute_close": "81523c88-6ca3-4d01-9fed-2f1cad9e2278",
  "stripe_post_charges_charge_refund": "9c4da163-443f-4e60-af72-3c70b383f0f0",
  "stripe_get_charges_charge_refunds": "9fd49eb8-f04e-41bc-84b9-02b5ec185f10",
  "stripe_post_charges_charge_refunds": "c2890595-8fab-431c-b9d4-1c666fee8e2e",
  "stripe_get_charges_charge_refunds_refund": "bcabf068-3351-4a68-8625-c27c64ac9695",
  "stripe_post_charges_charge_refunds_refund": "fd1a9ee8-2b11-413b-a5a0-3015ed74884b",
  "stripe_get_checkout_sessions": "4b396ec4-6a81-4d83-881e-8afcffaf51a2",
  "stripe_post_checkout_sessions": "41ca3345-fb1a-4d83-9420-6e257bd241b0",
  "stripe_get_checkout_sessions_session": "e5d4e74d-5309-48b1-89d2-758d31c8f5b5",
  "stripe_post_checkout_sessions_session": "bf173f8f-fe9b-4039-b5af-aa2b8dcb30cc",
  "stripe_post_checkout_sessions_session_expire": "f28d39fc-c89e-4c94-a462-07d8eddc0dcd",
  "stripe_get_checkout_sessions_session_line_items": "32fe5f50-917b-446f-a16d-3085203bbbc0",
  "stripe_get_climate_orders": "5ebccc40-b9fb-455f-80bc-a6c974df63b9",
  "stripe_post_climate_orders": "af175763-2962-441b-be90-46ab4f5b1c84",
  "stripe_get_climate_orders_order": "c573c762-b7f0-4f9f-ac11-6f141575408a",
  "stripe_post_climate_orders_order": "ee5e844b-cf49-4735-9eca-38c508899111",
  "stripe_post_climate_orders_order_cancel": "c06a6a3c-65b7-48f7-b850-5a4254e81e70",
  "stripe_get_climate_products": "a5e52e36-3600-4671-b508-fc54002712c4",
  "stripe_get_climate_products_product": "efd6236c-b6f5-4364-8c95-c79894565fee",
  "stripe_get_climate_suppliers": "b5bb82a1-739f-42fa-9200-44da68f7d3c8",
  "stripe_get_climate_suppliers_supplier": "fefcdb45-5e7f-4610-ae10-5bef1b9447a9",
  "stripe_get_confirmation_tokens_confirmation_token": "62143f56-324e-42e2-a07e-383d085f8eb0",
  "stripe_get_country_specs": "eff02ddc-72fb-4e43-a995-ab03087c8783",
  "stripe_get_country_specs_country": "aed24e6f-997b-454a-88e8-4085f5d2aab9",
  "stripe_get_coupons": "0dc98702-0df6-43df-a11d-8530c9eb6641",
  "stripe_post_coupons": "13af113a-0257-4214-b2d4-65b13c610518",
  "stripe_delete_coupons_coupon": "913878ce-0602-4b0f-aaeb-e863df1110ee",
  "stripe_get_coupons_coupon": "956b631d-90b5-45b4-895d-fa28ceeecd5b",
  "stripe_post_coupons_coupon": "0b9df57c-1b1d-455c-9f74-2e0ab5e71e89",
  "stripe_get_credit_notes": "51c2348a-5771-43a0-9945-2fbbe715ce01",
  "stripe_post_credit_notes": "2bf8b266-da3a-460d-8b20-fb8d8c63475a",
  "stripe_get_credit_notes_preview": "5131077d-fd54-487d-9736-8806995f3d69",
  "stripe_get_credit_notes_preview_lines": "cbfd461f-9538-48a6-9b03-77e1442b3c80",
  "stripe_get_credit_notes_credit_note_lines": "247573b7-7a3d-4eef-95b9-8ae7d05738de",
  "stripe_get_credit_notes_id": "190e8a1f-4d93-4f34-a23e-d42c70b63f91",
  "stripe_post_credit_notes_id": "bf255c9a-894d-45cf-ac2d-d9e676b696e1",
  "stripe_post_credit_notes_id_void": "9cd9cf2b-e0fd-445b-9e8a-f516ba376d6c",
  "stripe_post_customer_sessions": "faa0bde2-0f18-4f0f-8b7e-c830accc464a",
  "stripe_get_customers": "bad2719a-31cf-4b88-972f-93d16ed56eb5",
  "stripe_post_customers": "a4212956-efe4-4dcd-a25b-7c33069e96f7",
  "stripe_get_customers_search": "ae7868df-1243-4265-b58c-884ab19d06ea",
  "stripe_delete_customers_customer": "1fd696ec-3dfc-4a06-b0fd-d420d09be51a",
  "stripe_get_customers_customer": "1dc718dc-d8b4-42a1-b480-f7ef4956a2bf",
  "stripe_post_customers_customer": "ff920b71-2e9f-424d-bb23-e64a545961f6",
  "stripe_get_customers_customer_balance_transactions": "a7f996a0-79cc-483c-9ad5-b6d0b1cce9d0",
  "stripe_post_customers_customer_balance_transactions": "3d931aa2-02f3-4fda-9f39-24b47b7c6411",
  "stripe_get_customers_customer_balance_transactions_transaction": "73fb898f-8f42-49d6-8eab-62dc6213f80a",
  "stripe_post_customers_customer_balance_transactions_transaction": "37a9c1d1-2315-4d1b-9a2f-6bec3439f4d7",
  "stripe_get_customers_customer_bank_accounts": "6180df96-6fd5-4040-8e32-f8ba15e96b8f",
  "stripe_post_customers_customer_bank_accounts": "9425288f-c22b-4683-be88-0874e3aa861c",
  "stripe_delete_customers_customer_bank_accounts_id": "512a5956-a61a-46d9-b92e-8e8a0fe928d1",
  "stripe_get_customers_customer_bank_accounts_id": "54ccd0ba-a2fa-4f83-b7ac-0b649e16b7fb",
  "stripe_post_customers_customer_bank_accounts_id": "e8a58aa5-980e-41f0-bff2-cd2124872685",
  "stripe_post_customers_customer_bank_accounts_id_verify": "d5e847c8-97db-447e-acbf-731bf10c6309",
  "stripe_get_customers_customer_cards": "dcb18107-36e6-412e-9a32-44211e0c336c",
  "stripe_post_customers_customer_cards": "8ccfce9a-bbcf-468e-8d42-e20c8c6e37e1",
  "stripe_delete_customers_customer_cards_id": "dafc70aa-69fb-4e3d-bfd4-219c4fa4b887",
  "stripe_get_customers_customer_cards_id": "06fecc09-e823-45ac-a348-254637b4450d",
  "stripe_post_customers_customer_cards_id": "3db28d63-10b3-4a73-af89-d672678d848f",
  "stripe_get_customers_customer_cash_balance": "3212798d-53b5-4a4c-b869-d3da236e5d94",
  "stripe_post_customers_customer_cash_balance": "89e2edad-a823-4ee5-af9e-fa6c5b59d3cc",
  "stripe_get_customers_customer_cash_balance_transactions": "e2b87a30-1cb6-4035-8720-bd21177ddd9c",
  "stripe_get_customers_customer_cash_balance_transactions_transaction": "a75f0333-d0f0-44d0-ad10-4ad18047246c",
  "stripe_delete_customers_customer_discount": "69aecdf5-b0d6-436c-9f61-582a9db5643e",
  "stripe_get_customers_customer_discount": "311d049f-681c-4b97-893d-8e93f74bf6fa",
  "stripe_post_customers_customer_funding_instructions": "6a9c9696-22dd-4554-8a13-8fa3d95c53c2",
  "stripe_get_customers_customer_payment_methods": "f7e04b9e-3a83-492c-9d31-eab3749dcfba",
  "stripe_get_customers_customer_payment_methods_payment_method": "971cb62d-6bab-49df-85aa-3891d921736a",
  "stripe_get_customers_customer_sources": "46787a18-34da-499a-b4f2-b0f7de354e63",
  "stripe_post_customers_customer_sources": "be62a12e-229b-4624-83d3-6223949acaef",
  "stripe_delete_customers_customer_sources_id": "1e3baed9-9222-4b9b-916b-82284602c02c",
  "stripe_get_customers_customer_sources_id": "eb2b4a51-8075-4ea5-b7a2-48cd996c4ed0",
  "stripe_post_customers_customer_sources_id": "60b33cc6-291f-4ce7-8df5-8dd1036723dc",
  "stripe_post_customers_customer_sources_id_verify": "24e1ea8d-058c-4402-aba6-1b830f451785",
  "stripe_get_customers_customer_subscriptions": "f3486069-1173-4bc2-b965-35c3fbbd8e5c",
  "stripe_post_customers_customer_subscriptions": "a94ec134-2e6b-4cd8-a774-622fe8b346fd",
  "stripe_delete_customers_customer_subscriptions_subscription_exposed_id": "6b0f1439-00d2-40b3-975f-cef3ebd3dd5c",
  "stripe_get_customers_customer_subscriptions_subscription_exposed_id": "29e4caad-dda4-4fbb-8bbc-c99d059f3e22",
  "stripe_post_customers_customer_subscriptions_subscription_exposed_id": "7ca2f9c9-c0ca-4d83-8744-9f372ca9caab",
  "stripe_delete_customers_customer_subscriptions_subscription_exposed_id_discount": "f26cda75-7266-4a5c-8e38-1061809e6bc0",
  "stripe_get_customers_customer_subscriptions_subscription_exposed_id_discount": "7e79706d-784c-4908-88f8-91ccb9b751e2",
  "stripe_get_customers_customer_tax_ids": "5e6b4f3c-7504-47b8-8c99-5245cf7e0929",
  "stripe_post_customers_customer_tax_ids": "d130a819-f98a-4347-ac48-279952da278a",
  "stripe_delete_customers_customer_tax_ids_id": "9d9977be-a01a-4857-88c0-5cd63b8f1834",
  "stripe_get_customers_customer_tax_ids_id": "fb0ce2b1-ead8-427d-877b-f4dbd481eb6e",
  "stripe_get_disputes": "647eb24d-3c97-474a-befa-101bfbf7dbc3",
  "stripe_get_disputes_dispute": "862edf62-6832-41bc-be6d-f82adfaa5c93",
  "stripe_post_disputes_dispute": "6fa87df5-c0a2-4fc9-b8a0-8cba006c5abe",
  "stripe_post_disputes_dispute_close": "09b17cc5-5eb3-4f91-920f-c0527f37eb73",
  "stripe_get_entitlements_active_entitlements": "8969c07d-09b7-4d3e-a732-2f6e8fc16304",
  "stripe_get_entitlements_active_entitlements_id": "3c24a14a-da14-45e6-8f13-17949557323a",
  "stripe_get_entitlements_features": "45d0f3b0-7ec4-4307-869e-aecdd9ab9caf",
  "stripe_post_entitlements_features": "7bea13d2-6a49-4a63-9079-0587dfc7d20b",
  "stripe_get_entitlements_features_id": "0845a2aa-ee66-4e41-86cd-b3f2fc0747a5",
  "stripe_post_entitlements_features_id": "080fa4b5-0386-4499-9f6b-21f8ae49a456",
  "stripe_post_ephemeral_keys": "a160960e-eaaa-4151-a9a3-20b844db791c",
  "stripe_delete_ephemeral_keys_key": "ac8efd59-8b73-4708-ae3b-0adfbe6ab668",
  "stripe_get_events": "73e9de23-0c98-4df2-b4fc-88005abed759",
  "stripe_get_events_id": "e22c5f79-eb4f-4a2c-9506-0b4c545e3f5e",
  "stripe_get_exchange_rates": "e2395110-b774-4c2c-9054-6444558c4554",
  "stripe_get_exchange_rates_rate_id": "48d8664a-db71-4469-bbcf-b9355216320a",
  "stripe_get_file_links": "eb7700fc-eb23-4a75-9ece-d1527cacd80f",
  "stripe_post_file_links": "98d0173f-c851-470d-900a-1c4ce3be5c87",
  "stripe_get_file_links_link": "baade64c-e663-4b41-ae33-2ba245576f48",
  "stripe_post_file_links_link": "be9da96e-2ed3-4b51-9c4d-adfe143292e5",
  "stripe_get_files": "311be310-2e74-4d7f-b20d-c77a571e846c",
  "stripe_post_files": "e61bf64a-a457-4db7-b365-22b646da833f",
  "stripe_get_files_file": "f59e4653-4956-4ec5-9e61-914c1a829db6",
  "stripe_get_financial_connections_accounts": "9cf80237-8c92-4d03-a0f0-1ddd47d9cec9",
  "stripe_get_financial_connections_accounts_account": "e89b01a1-cad1-439a-8299-829180f1374d",
  "stripe_post_financial_connections_accounts_account_disconnect": "b443b3d6-3e51-4ffb-9e72-ede8388edea3",
  "stripe_get_financial_connections_accounts_account_owners": "1e041af7-25c6-4caa-9134-0293a0ab3305",
  "stripe_post_financial_connections_accounts_account_refresh": "cc4938cb-8385-4b9b-a13e-c3adc6870e76",
  "stripe_post_financial_connections_accounts_account_subscribe": "72c01af8-0537-4bc5-bc90-6b034904b049",
  "stripe_post_financial_connections_accounts_account_unsubscribe": "0482e244-5ca2-40f8-b061-86c6a3be3bd9",
  "stripe_post_financial_connections_sessions": "6add7574-a94c-4270-b259-4e45333f4c5a",
  "stripe_get_financial_connections_sessions_session": "c7fa917c-1ebd-46a8-b83c-0b96303e05ef",
  "stripe_get_financial_connections_transactions": "3a5f1492-46f7-46d5-94a1-8758b87b1624",
  "stripe_get_financial_connections_transactions_transaction": "3b14af28-0367-49c5-8d7f-d6ea4e6c6724",
  "stripe_get_forwarding_requests": "8d4efebc-36bd-4a05-89ce-e91da2945cbe",
  "stripe_post_forwarding_requests": "e21df603-918c-4085-bdee-88ee7d41bdc3",
  "stripe_get_forwarding_requests_id": "66f596d0-ad72-49ba-89c7-88b5464e0db8",
  "stripe_get_identity_verification_reports": "f15cfb78-09cc-4822-a093-c4e371f10df3",
  "stripe_get_identity_verification_reports_report": "6f02f1ac-2207-4760-b05b-e8b7dad67d54",
  "stripe_get_identity_verification_sessions": "e30bd071-9ba2-4ba3-a3fb-4ed0d29acef5",
  "stripe_post_identity_verification_sessions": "8e6592e4-28ae-4f83-9041-f0dd41fe6721",
  "stripe_get_identity_verification_sessions_session": "9e03bad9-5cb1-435d-9b27-aba3810c1ac8",
  "stripe_post_identity_verification_sessions_session": "2b920f92-708c-47f7-b9dd-805503e04cc7",
  "stripe_post_identity_verification_sessions_session_cancel": "b132330c-3fa9-42f1-adf6-87f5b2826c17",
  "stripe_post_identity_verification_sessions_session_redact": "59f4bd11-1e5c-4a2a-bae6-e397aaa9b982",
  "stripe_get_invoice_rendering_templates": "a9c84adc-49a3-43c6-8623-0c031d88ee75",
  "stripe_get_invoice_rendering_templates_template": "ddb87535-be91-4db1-8eb8-892a5976fe09",
  "stripe_post_invoice_rendering_templates_template_archive": "ac157c68-40a8-4132-8cb5-69abc849c810",
  "stripe_post_invoice_rendering_templates_template_unarchive": "cb7e4a19-1b71-44f0-84c1-750796425718",
  "stripe_get_invoiceitems": "3119f89a-59bc-4c29-a634-cfdcf3196cbc",
  "stripe_post_invoiceitems": "6afdbf53-b00e-47df-b72b-7ff73ff078d8",
  "stripe_delete_invoiceitems_invoiceitem": "f8cb7b92-1132-4ed0-ad69-21157f375fee",
  "stripe_get_invoiceitems_invoiceitem": "5e29c83a-4e67-4259-be26-2210ec0c950c",
  "stripe_post_invoiceitems_invoiceitem": "5f6129da-7b26-4bdf-8554-5f4e6f9991a4",
  "stripe_get_invoices": "e215b485-3aff-4dfa-83b2-f31e0eb271af",
  "stripe_post_invoices": "6ae139e3-9ae3-481e-94b7-7a99ac6732aa",
  "stripe_post_invoices_create_preview": "3b268e61-fc21-455d-a278-32605f350696",
  "stripe_get_invoices_search": "69a583d3-24cb-47fb-af61-d7ef7971bd7a",
  "stripe_get_invoices_upcoming": "75d2f1b2-3c49-4d81-ab0d-b7fc049a957c",
  "stripe_get_invoices_upcoming_lines": "f9c817dd-076c-4a38-882d-4747cfa31145",
  "stripe_delete_invoices_invoice": "e7250a2c-8ae9-4227-90d4-d46226a4374d",
  "stripe_get_invoices_invoice": "366354fe-867b-4e0b-b38f-3bebc266ba6d",
  "stripe_post_invoices_invoice": "a0c52558-f0ce-47af-99d9-530d575e0b5e",
  "stripe_post_invoices_invoice_add_lines": "9ecf5add-82a5-4a4d-95eb-101cd77dcf83",
  "stripe_post_invoices_invoice_finalize": "23c30f37-7c17-4b97-bb8f-fe61e1fa8fa4",
  "stripe_get_invoices_invoice_lines": "34cfd7a5-0602-4b79-b8a9-11cf12723549",
  "stripe_post_invoices_invoice_lines_line_item_id": "91e031ef-cbc9-4cb1-8121-5398b28c348f",
  "stripe_post_invoices_invoice_mark_uncollectible": "21a8ccfe-37d2-48e3-8d45-9cd51de5e693",
  "stripe_post_invoices_invoice_pay": "c5e4f47a-2dce-4b67-8bb3-598640986c78",
  "stripe_post_invoices_invoice_remove_lines": "e40a93bc-ae74-4c01-87ee-a0007f6c7565",
  "stripe_post_invoices_invoice_send": "572e59a4-c95c-4e14-8ddb-89571409bdad",
  "stripe_post_invoices_invoice_update_lines": "c006a2a1-ff43-48b7-807d-e0f953e3253e",
  "stripe_post_invoices_invoice_void": "fe7a5ddb-b65f-4307-bbbe-8b36b7a25ebe",
  "stripe_get_issuing_authorizations": "fccca908-cad0-4b0c-aed2-2645070ddc1b",
  "stripe_get_issuing_authorizations_authorization": "e6f25324-ef50-4890-b592-5d41ea76d8b6",
  "stripe_post_issuing_authorizations_authorization": "c6eb7679-4acc-4816-9e95-0a8329d97a9d",
  "stripe_post_issuing_authorizations_authorization_approve": "dc3c5b63-ab89-498c-8269-71ce8ae72c17",
  "stripe_post_issuing_authorizations_authorization_decline": "16731797-adab-42b2-9e25-1a2e18810536",
  "stripe_get_issuing_cardholders": "903ad034-119b-4f23-96e5-328d2682f866",
  "stripe_post_issuing_cardholders": "1c559a0a-4763-488b-92b2-9ed51032208c",
  "stripe_get_issuing_cardholders_cardholder": "304ebed7-2438-49ab-af32-3d04816fa63f",
  "stripe_post_issuing_cardholders_cardholder": "48d2271c-d866-4488-8b88-d93d8a48e683",
  "stripe_get_issuing_cards": "b82e9e78-f478-427f-9e3f-db9e6331badc",
  "stripe_post_issuing_cards": "ff89a8fb-fb34-41dc-aa8e-de5503d16b9b",
  "stripe_get_issuing_cards_card": "d12d4015-d1e6-40bc-9825-7af3cf7ce0e3",
  "stripe_post_issuing_cards_card": "2789cfc0-e8be-446a-9d53-ad5b0754346f",
  "stripe_get_issuing_disputes": "ec7b5cc6-b40f-4aae-b948-ff5d8e3936e6",
  "stripe_post_issuing_disputes": "1311b263-de65-4f00-b81c-c2102f3a2b82",
  "stripe_get_issuing_disputes_dispute": "cb0a99e3-aa29-4e1d-a2f6-911621fe8c04",
  "stripe_post_issuing_disputes_dispute": "27178d44-b691-4f07-bfc6-a82c1e59f868",
  "stripe_post_issuing_disputes_dispute_submit": "5f442daa-b1b9-4568-add9-8a4fd3cd304f",
  "stripe_get_issuing_personalization_designs": "7e8f0240-a6cb-4a05-b5b8-a9ad5de3e970",
  "stripe_post_issuing_personalization_designs": "0213d185-6b18-4cc5-9526-d83a3ce3219d",
  "stripe_get_issuing_personalization_designs_personalization_design": "bb276c56-86d6-4383-8908-89ff5b997925",
  "stripe_post_issuing_personalization_designs_personalization_design": "627d85bb-33a6-46c5-8121-ffbec124fea5",
  "stripe_get_issuing_physical_bundles": "f8abbf81-df31-464d-be48-bb6f4fc4c6ca",
  "stripe_get_issuing_physical_bundles_physical_bundle": "a8aae5de-0747-482a-be59-388fa99ce9f2",
  "stripe_get_issuing_settlements_settlement": "2314aef5-9aa9-4a6f-a107-597759363c69",
  "stripe_post_issuing_settlements_settlement": "210de1d8-255f-4424-ba42-cd4b1449c660",
  "stripe_get_issuing_tokens": "24aba854-ba82-4e13-b0d2-c71cd49f84eb",
  "stripe_get_issuing_tokens_token": "8ed8d8f2-14a6-414c-b5f0-6b8d131eb9d8",
  "stripe_post_issuing_tokens_token": "80721f47-7d76-417a-bdde-de34e23b2708",
  "stripe_get_issuing_transactions": "7a055242-d511-4ca6-847f-87045def2ab3",
  "stripe_get_issuing_transactions_transaction": "f7b1f583-eb66-4635-8349-7ea078b72e96",
  "stripe_post_issuing_transactions_transaction": "7dbfe742-88c7-4a62-bbe8-b54319a25b32",
  "stripe_post_link_account_sessions": "b713f8dc-b739-4842-9978-e029bb211018",
  "stripe_get_link_account_sessions_session": "077b7155-b235-4e0b-b0b7-4a3a9f215204",
  "stripe_get_linked_accounts": "04c9ae8d-6005-4af9-b64f-92c213e64aae",
  "stripe_get_linked_accounts_account": "adbc59ce-b4d4-4685-87af-1341023311df",
  "stripe_post_linked_accounts_account_disconnect": "df32f1fb-7da3-4117-a14b-6a6962212a3f",
  "stripe_get_linked_accounts_account_owners": "dc150350-d43a-4e37-acc5-e9b85420136b",
  "stripe_post_linked_accounts_account_refresh": "e1f4dd61-d44b-4db5-8b9c-f2f0a31a0b5b",
  "stripe_get_mandates_mandate": "2f755f2e-71c6-49e4-9f41-9988a1026135",
  "stripe_get_payment_intents": "2c0e19e3-ab7d-4cbb-a027-ed1e0307ea4f",
  "stripe_post_payment_intents": "58277f02-9222-4733-9397-a000058d2bd8",
  "stripe_get_payment_intents_search": "405b89ef-ed4e-4a7c-8752-8b3b84d87cbf",
  "stripe_get_payment_intents_intent": "539c6608-b4bb-4211-bf20-888c66850753",
  "stripe_post_payment_intents_intent": "bcee356a-85bb-4904-9d1f-b2d758e182cc",
  "stripe_post_payment_intents_intent_apply_customer_balance": "a4435416-dad1-4321-8b16-a3ad5ef11a3a",
  "stripe_post_payment_intents_intent_cancel": "d39e034d-2dd2-4110-805c-f18136357baa",
  "stripe_post_payment_intents_intent_capture": "d46a1490-a0a5-45bc-a06c-42f74988997d",
  "stripe_post_payment_intents_intent_confirm": "41f2d5d1-f3f6-4e0f-ab05-1208d7d1cd19",
  "stripe_post_payment_intents_intent_increment_authorization": "52494655-16b7-4a7a-b0f0-70e6f0d907a2",
  "stripe_post_payment_intents_intent_verify_microdeposits": "d2f78a24-e8db-4fb7-b6de-16dddee7fe40",
  "stripe_get_payment_links": "f18d313b-cab0-4a2a-8507-1100cdd7f1fe",
  "stripe_post_payment_links": "7cf28fc7-9dc2-4129-9a54-c6d73f3cbb1a",
  "stripe_get_payment_links_payment_link": "cfa0df5b-082d-4157-9a86-6deeaa1829d1",
  "stripe_post_payment_links_payment_link": "f2250e95-884d-41bc-a0b1-4cbb14b6edb7",
  "stripe_get_payment_links_payment_link_line_items": "f95ae275-7989-4510-b37f-5df3b117e1bf",
  "stripe_get_payment_method_configurations": "8b92260c-b7de-4220-809c-32354fa52137",
  "stripe_post_payment_method_configurations": "1c408976-48e9-4433-91e4-0cc577b01510",
  "stripe_get_payment_method_configurations_configuration": "19840630-0376-492a-9660-e14c4c8112e3",
  "stripe_post_payment_method_configurations_configuration": "35b8655a-96e0-4fd5-b02a-03cf9d1d5ada",
  "stripe_get_payment_method_domains": "2e57362b-0886-41d2-9224-b95f7eb10bad",
  "stripe_post_payment_method_domains": "a400495c-9e87-42ef-874e-4742be3eeae3",
  "stripe_get_payment_method_domains_payment_method_domain": "72d915fc-5105-42c0-961a-85e1bb70b8e1",
  "stripe_post_payment_method_domains_payment_method_domain": "bcf31138-600a-492f-92b7-b6eda0c6e4cf",
  "stripe_post_payment_method_domains_payment_method_domain_validate": "09f1c702-83a1-4fc1-8c7c-5aef206c167c",
  "stripe_get_payment_methods": "eb872270-12f9-4c40-a4de-350b2dca4a3d",
  "stripe_post_payment_methods": "9b28a3c3-0263-4731-a89f-78e407ff9abb",
  "stripe_get_payment_methods_payment_method": "9d187635-817a-4e98-9ee9-54784059346a",
  "stripe_post_payment_methods_payment_method": "4e2893eb-84f4-4c60-ab5f-073a9aa0e9b3",
  "stripe_post_payment_methods_payment_method_attach": "412ea829-287b-41f4-a4b9-2460183a428d",
  "stripe_post_payment_methods_payment_method_detach": "0135755f-7e23-4382-9068-5ee867b39c3a",
  "stripe_get_payouts": "59321deb-464a-4954-babc-390008f81e42",
  "stripe_post_payouts": "5caf873c-43e7-4277-97cf-838f48cdba4d",
  "stripe_get_payouts_payout": "bda7b1b2-e54f-4188-b358-e1580bf1e26c",
  "stripe_post_payouts_payout": "99e7d36d-4f16-4264-b8fd-12e4e3488579",
  "stripe_post_payouts_payout_cancel": "42ea5cd0-0209-451e-8738-029a82336c8f",
  "stripe_post_payouts_payout_reverse": "0da06d3b-5e60-490b-ad03-92bf0644c2b4",
  "stripe_get_plans": "72ae32a6-b43a-4537-8934-a0bde8a81a8c",
  "stripe_post_plans": "fb2b4c94-9184-4d99-a0e1-031b90f5698a",
  "stripe_delete_plans_plan": "0e372f05-0d31-428b-ba0b-25b089624772",
  "stripe_get_plans_plan": "72c83042-1a52-482e-8af1-0f7e2d64ad59",
  "stripe_post_plans_plan": "a9b80922-450c-4186-9378-b9362d27aeaa",
  "stripe_get_prices": "b7ae87a6-a898-43e6-aafb-323a7bcb9302",
  "stripe_post_prices": "2aca5943-3bf3-416e-aca4-a3524c821983",
  "stripe_get_prices_search": "204729a0-6e3b-42e1-9beb-2fa2dab5cee4",
  "stripe_get_prices_price": "b9495b34-4ae8-472e-8790-ef345df44cef",
  "stripe_post_prices_price": "b8dd76f1-f3e9-48e1-838e-cdb262e497af",
  "stripe_get_products": "02727a3a-91df-48e3-ae06-28aef5f7237e",
  "stripe_post_products": "3057d709-b0ee-4f4b-93e7-d218613c165f",
  "stripe_get_products_search": "265a7442-d03f-41f1-b78c-9fb5ddbe7c4f",
  "stripe_delete_products_id": "9e54cce4-1f6a-4b21-b636-170376a8e53e",
  "stripe_get_products_id": "f3633052-1c13-4c8a-ae00-77160ec47a03",
  "stripe_post_products_id": "2bb0d830-4e44-48f0-969c-e758d7b586a5",
  "stripe_get_products_product_features": "a024aaaa-d249-4d1f-a6a2-69b9d5a99cd8",
  "stripe_post_products_product_features": "88d0e151-bb01-4ff9-8328-6a6c42df1623",
  "stripe_delete_products_product_features_id": "43cca5d5-c67e-4201-a804-bda6ea89bf4d",
  "stripe_get_products_product_features_id": "d034e866-ef60-472c-b8c0-27676f096815",
  "stripe_get_promotion_codes": "5e9f44ca-a58c-4f2b-875e-dffd4345b7f6",
  "stripe_post_promotion_codes": "160bfd24-a900-4e6f-be84-26afc667d664",
  "stripe_get_promotion_codes_promotion_code": "b2453edc-c6e8-4793-b8a9-2ed8e8c929a6",
  "stripe_post_promotion_codes_promotion_code": "6b869a0c-df34-447e-9922-438fdc7570cf",
  "stripe_get_quotes": "3594d5a7-eb9a-45cf-aa2d-4d5549b00092",
  "stripe_post_quotes": "007e14d8-8910-495e-93fa-4c5ec8b14043",
  "stripe_get_quotes_quote": "127a0cff-a058-414b-8875-b6fb0151c565",
  "stripe_post_quotes_quote": "7251df73-8f05-492c-b0d7-0d90dac1820c",
  "stripe_post_quotes_quote_accept": "7bc44696-f194-4a05-9bbf-63825a44d152",
  "stripe_post_quotes_quote_cancel": "64d03868-427a-41ac-94f5-5ebcfdf99e51",
  "stripe_get_quotes_quote_computed_upfront_line_items": "6987c3d7-cdb8-4e57-8feb-b79e4c7eea08",
  "stripe_post_quotes_quote_finalize": "b37b50d1-6c27-42ab-a640-bc92df99530a",
  "stripe_get_quotes_quote_line_items": "495f8595-83a1-4b9e-930e-b1c1368ce8c6",
  "stripe_get_quotes_quote_pdf": "18e87d2d-eb37-4662-bb76-cc91f4f83a5d",
  "stripe_get_radar_early_fraud_warnings": "48ebf479-3190-4c1e-a70d-3c0b26ccb330",
  "stripe_get_radar_early_fraud_warnings_early_fraud_warning": "f98c997c-6738-4289-9bde-3c0fa6dc633a",
  "stripe_get_radar_value_list_items": "3ea8ae52-af3e-48e1-af0e-feae44878f3a",
  "stripe_post_radar_value_list_items": "74853aee-5425-4a25-ab92-b436497c5181",
  "stripe_delete_radar_value_list_items_item": "f868fce7-fecd-4224-b7c4-5a077ea4ddaf",
  "stripe_get_radar_value_list_items_item": "a001c986-03fc-4dc4-a850-e1cdc64c1cd5",
  "stripe_get_radar_value_lists": "2f748c61-5a86-4887-8e35-621065baa1a8",
  "stripe_post_radar_value_lists": "119ded16-d747-44b9-85ac-b6ec3ea9f89c",
  "stripe_delete_radar_value_lists_value_list": "c55309f1-3b88-4e99-b62c-b714ffc46f3b",
  "stripe_get_radar_value_lists_value_list": "6ac7e6d0-0acd-46a3-9c5f-4fc3c3e36005",
  "stripe_post_radar_value_lists_value_list": "38c0f059-2860-4c4c-ad2d-a2231eac5e44",
  "stripe_get_refunds": "3a1d6028-f750-4786-a677-325cd505293f",
  "stripe_post_refunds": "1073a4cf-5c24-49bb-9831-31676a6d9b99",
  "stripe_get_refunds_refund": "b693f908-b4e7-4a1c-ba27-89fa87ea2589",
  "stripe_post_refunds_refund": "1d589234-6f86-46ba-916d-86398d2dcbeb",
  "stripe_post_refunds_refund_cancel": "bb5e314e-929e-4b0f-a96a-475734ac8822",
  "stripe_get_reporting_report_runs": "53b6762c-bd9f-4c2e-ba64-325ca38d9faa",
  "stripe_post_reporting_report_runs": "6e06ee74-f260-4358-bc56-c4db11a98055",
  "stripe_get_reporting_report_runs_report_run": "299383f6-93a1-44fd-9e20-b8993d2798d8",
  "stripe_get_reporting_report_types": "facc50fe-265a-4269-b4c9-be127c3d5594",
  "stripe_get_reporting_report_types_report_type": "d0103b5c-dd96-44c1-89a6-ce7e2824f91a",
  "stripe_get_reviews": "1636ef28-6885-4275-9b41-d8edc2c598cf",
  "stripe_get_reviews_review": "8ea9e268-4f7a-449c-930c-7e543809c3ee",
  "stripe_post_reviews_review_approve": "a71c4689-8024-47ab-8161-75985c12a577",
  "stripe_get_setup_attempts": "85df838e-a22f-43f3-8af2-c68054a30fca",
  "stripe_get_setup_intents": "1f482776-9473-44dd-bf34-d89439083f17",
  "stripe_post_setup_intents": "eb1ff11a-89e1-4884-9af2-6411ed9e099d",
  "stripe_get_setup_intents_intent": "6824ac43-1cb3-4b97-b078-b68a3368bd54",
  "stripe_post_setup_intents_intent": "1248416f-e6bf-4c79-9704-5db341f756c3",
  "stripe_post_setup_intents_intent_cancel": "6029cff3-14fb-40da-b9c2-056c5aca52f3",
  "stripe_post_setup_intents_intent_confirm": "60a88ac8-9066-4a44-bcd4-a83e43fc3070",
  "stripe_post_setup_intents_intent_verify_microdeposits": "6b4c71c8-045d-4456-9ebf-00263f575a4a",
  "stripe_get_shipping_rates": "3a4f2f1a-fc75-44d5-870e-de891f1b0930",
  "stripe_post_shipping_rates": "4e8f52d2-287f-4acb-8c83-21d294223c7c",
  "stripe_get_shipping_rates_shipping_rate_token": "82b0ad48-b872-4087-ba39-17bac0a545c6",
  "stripe_post_shipping_rates_shipping_rate_token": "af44d0bd-c655-4ee8-998c-bb72d0662fee",
  "stripe_get_sigma_scheduled_query_runs": "db462ac5-1be1-4f4a-bc9b-9c5600acb4cb",
  "stripe_get_sigma_scheduled_query_runs_scheduled_query_run": "578715bd-30f5-4c88-a216-c866d87451dd",
  "stripe_post_sources": "e1ecec44-6cc7-4baf-886b-9492475b5f63",
  "stripe_get_sources_source": "3e1c6190-22ea-46e9-8252-7fff0468cddb",
  "stripe_post_sources_source": "bf5f96d9-62c1-4a0b-ad9c-e7f5d735dfc8",
  "stripe_get_sources_source_mandate_notifications_mandate_notification": "52f80228-d658-403b-85d4-bfdedfaf1e0d",
  "stripe_get_sources_source_source_transactions": "a66a04b4-d98d-47b4-8fe4-ac86694c31af",
  "stripe_get_sources_source_source_transactions_source_transaction": "ddc35f41-c7e3-4777-b0b9-eb17188a2b6f",
  "stripe_post_sources_source_verify": "706a59a3-4fc9-4bc8-87d9-65ecd7d7d491",
  "stripe_get_subscription_items": "82a9490e-fc8b-4482-b04e-b5df95244d3b",
  "stripe_post_subscription_items": "6e25f021-8b28-4f91-8556-5cabaf62bcb5",
  "stripe_delete_subscription_items_item": "ac597e3d-9319-4ee0-bbc6-94b8e550f071",
  "stripe_get_subscription_items_item": "6207ce04-237e-4567-abf3-99a0d945799a",
  "stripe_post_subscription_items_item": "165d0728-c7b3-4511-bd38-3a3ed06bf71e",
  "stripe_get_subscription_items_subscription_item_usage_record_summaries": "0c0af609-d8c4-45a4-9d65-e2051cec325f",
  "stripe_post_subscription_items_subscription_item_usage_records": "ad6a417e-8089-4d23-be12-36b39487c08a",
  "stripe_get_subscription_schedules": "795df04d-bce3-48c9-bea3-bacd97eee8f4",
  "stripe_post_subscription_schedules": "e5ec4876-24a3-4214-bbf9-724332ec45d8",
  "stripe_get_subscription_schedules_schedule": "671b421d-70ef-4b98-826d-4f442a16a61e",
  "stripe_post_subscription_schedules_schedule": "58030ef8-b80d-42b2-8059-a4511d795b4c",
  "stripe_post_subscription_schedules_schedule_cancel": "331ca4e8-6f2a-4106-83f1-a2718362ec8b",
  "stripe_post_subscription_schedules_schedule_release": "76daf682-41d1-4b4d-92e8-cc14630c9e44",
  "stripe_get_subscriptions": "037a7ce2-bd7a-44e3-b30a-6dc282c4a326",
  "stripe_post_subscriptions": "840d5bb9-3270-46e0-bdb1-f1fe697d0d6c",
  "stripe_get_subscriptions_search": "25457282-8fd7-4d8a-99d5-eabe95ed3f47",
  "stripe_delete_subscriptions_subscription_exposed_id": "748b45d5-0fa0-4e03-9989-7544bc7804eb",
  "stripe_get_subscriptions_subscription_exposed_id": "5f0d8eac-9ecd-44f3-9d2e-d559326f9655",
  "stripe_post_subscriptions_subscription_exposed_id": "a8ce2619-a222-4c5c-a434-cedd439798fe",
  "stripe_delete_subscriptions_subscription_exposed_id_discount": "3dac2bfd-2015-49de-85b1-202c3715997c",
  "stripe_post_subscriptions_subscription_resume": "c88b9a04-aec8-4a75-b888-79ecc38f4505",
  "stripe_post_tax_calculations": "14869cd8-b6c0-4ee6-a799-cad2a04446f5",
  "stripe_get_tax_calculations_calculation": "490623ca-8c50-4e52-bacf-1e1da7baf57b",
  "stripe_get_tax_calculations_calculation_line_items": "c61a0cbd-9932-4e32-b5d2-2f7ae92e0d1c",
  "stripe_get_tax_registrations": "e63dca49-763a-4441-8655-0edc55f06e2b",
  "stripe_post_tax_registrations": "37d0caa9-f3cb-4a02-8310-1a2db5782ed6",
  "stripe_get_tax_registrations_id": "3968463f-796a-4908-b079-68619557e2b5",
  "stripe_post_tax_registrations_id": "1198ec3a-bb9a-4c3f-abc1-85ae17389f89",
  "stripe_get_tax_settings": "168d1035-2c8d-4a9b-93d3-d82b65eb3657",
  "stripe_post_tax_settings": "6c1cdd2c-5ddb-4fee-b6cf-1e540250b609",
  "stripe_post_tax_transactions_create_from_calculation": "26cbfe0b-05c9-4d24-ac7f-dece5ac237a6",
  "stripe_post_tax_transactions_create_reversal": "e4b0fd05-78f8-452e-93fe-01936aa5b011",
  "stripe_get_tax_transactions_transaction": "62cdee4f-0f26-4e4b-9d0e-1e3d79b9ffad",
  "stripe_get_tax_transactions_transaction_line_items": "2be9a841-903b-4b95-9185-1c869d25411a",
  "stripe_get_tax_codes": "fa5bb00c-ca7b-48c2-8c41-9bae857f0940",
  "stripe_get_tax_codes_id": "6157a93c-11b4-4b9a-a1a9-35fc3b2932e1",
  "stripe_get_tax_ids": "0a311ba4-b28f-47e7-9aac-9a9c6e236a58",
  "stripe_post_tax_ids": "fa5839ff-ac37-43ae-86af-7c8d1383cbb3",
  "stripe_delete_tax_ids_id": "ac900d56-2c30-40c0-b2ac-f7287a96800d",
  "stripe_get_tax_ids_id": "b7121732-97b3-4af6-83c3-68197cdb2294",
  "stripe_get_tax_rates": "07d6353a-7875-4208-8ee3-79fdcdff6f7a",
  "stripe_post_tax_rates": "e918655c-8b20-4159-a154-3b6b78262900",
  "stripe_get_tax_rates_tax_rate": "e2808058-86fe-4b6c-94fe-ec6ab0908991",
  "stripe_post_tax_rates_tax_rate": "a580f08d-d244-4e8a-8385-a497e29743e0",
  "stripe_get_terminal_configurations": "a742d733-520d-4fa6-a8c1-e770aacaffc9",
  "stripe_post_terminal_configurations": "37472a3d-f602-4b8f-8f43-1925b803ce5d",
  "stripe_delete_terminal_configurations_configuration": "9356808b-390a-4f56-b377-863212cfb255",
  "stripe_get_terminal_configurations_configuration": "c8ae70e3-7c08-4287-ad6e-79ef4c2a51a4",
  "stripe_post_terminal_configurations_configuration": "3f4c80e1-c132-453b-abc4-8fdf1ec77b3d",
  "stripe_post_terminal_connection_tokens": "c13c18cd-2681-4d28-b8b2-aea73bedf29f",
  "stripe_get_terminal_locations": "367eca90-b44c-4623-97f6-b70f6f7dc557",
  "stripe_post_terminal_locations": "ae2e2a7a-4b3c-4296-9932-d2549ac14e19",
  "stripe_delete_terminal_locations_location": "741870ae-7678-49b4-94a2-5b2eac070c03",
  "stripe_get_terminal_locations_location": "26d37ce6-059e-4f2e-905b-cbc9bb9a2748",
  "stripe_post_terminal_locations_location": "86f525be-9d4f-4bf0-9275-45a3e92c2cd2",
  "stripe_get_terminal_readers": "15483d4d-e37d-4423-b324-9b1342951d3b",
  "stripe_post_terminal_readers": "de75d452-1e8d-4aeb-a332-320ad3613951",
  "stripe_delete_terminal_readers_reader": "25bbf73e-8d92-4f1e-8001-df903a10f8e0",
  "stripe_get_terminal_readers_reader": "7f864e5f-c388-4a01-ace2-80f3b1fc069e",
  "stripe_post_terminal_readers_reader": "bd3208aa-b897-456a-b9f0-699e8d3aef6b",
  "stripe_post_terminal_readers_reader_cancel_action": "efb2ae96-05e0-4133-9885-11f06943ebe3",
  "stripe_post_terminal_readers_reader_process_payment_intent": "823147fe-86c4-4935-ae2d-e0a4cd1c3c54",
  "stripe_post_terminal_readers_reader_process_setup_intent": "f9f3d101-6103-4d29-b1aa-b135a466354d",
  "stripe_post_terminal_readers_reader_refund_payment": "5d1b888a-b4ee-496e-8199-28e6e4724285",
  "stripe_post_terminal_readers_reader_set_reader_display": "bcc66ef3-ee81-479d-98e5-c1e7b1c862cc",
  "stripe_post_test_helpers_confirmation_tokens": "2a2e4651-6e5c-4c2f-815c-95e7802cc0ae",
  "stripe_post_test_helpers_customers_customer_fund_cash_balance": "e07caac8-5887-446c-b6be-ac020510d84a",
  "stripe_post_test_helpers_issuing_authorizations": "dab6bb11-07bc-4735-8b9f-930aca83c809",
  "stripe_post_test_helpers_issuing_authorizations_authorization_capture": "6dc212de-fbaa-4d9e-92f7-da9def795dbc",
  "stripe_post_test_helpers_issuing_authorizations_authorization_expire": "dc4a2bff-7852-41d7-9c4c-c52a47254de5",
  "stripe_post_test_helpers_issuing_authorizations_authorization_finalize_amount": "13f2e121-9e85-4a6f-8cc5-46adad66f836",
  "stripe_post_test_helpers_issuing_authorizations_authorization_increment": "cbcceaf6-e0ee-4ae9-883a-50b428a86ccc",
  "stripe_post_test_helpers_issuing_authorizations_authorization_reverse": "c9dd1581-3989-402e-994c-814aa67a5ff2",
  "stripe_post_test_helpers_issuing_cards_card_shipping_deliver": "56705e8d-1c75-4f6b-ac3e-95dac5330e00",
  "stripe_post_test_helpers_issuing_cards_card_shipping_fail": "295d157d-cddf-4483-bb50-8315ce2c41a4",
  "stripe_post_test_helpers_issuing_cards_card_shipping_return": "d77cc101-395b-429b-913b-5dc93de6b3ff",
  "stripe_post_test_helpers_issuing_cards_card_shipping_ship": "a6b4ea1d-c34d-4856-96bf-ba0770e7ad7a",
  "stripe_post_test_helpers_issuing_cards_card_shipping_submit": "94e20719-8864-4d0e-a8ae-b837b6e1779f",
  "stripe_post_test_helpers_issuing_personalization_designs_personalization_design_activate": "6cbe4d45-7bd3-4788-ae96-b96e6098814b",
  "stripe_post_test_helpers_issuing_personalization_designs_personalization_design_deactivate": "fe365e8a-7e72-4a47-accf-317fa18e4517",
  "stripe_post_test_helpers_issuing_personalization_designs_personalization_design_reject": "4d5b43c1-0f94-4fff-b237-943a197cd8ec",
  "stripe_post_test_helpers_issuing_settlements": "ec2fd26e-6572-4ed6-8f89-d0fb9e7633f0",
  "stripe_post_test_helpers_issuing_transactions_create_force_capture": "dc0caa9c-4307-4ce9-8db0-724193b8d2da",
  "stripe_post_test_helpers_issuing_transactions_create_unlinked_refund": "0227dbab-27d7-4aa2-a49a-5b422f39e0c0",
  "stripe_post_test_helpers_issuing_transactions_transaction_refund": "69500374-0859-4ea7-af19-685330c3f14c",
  "stripe_post_test_helpers_refunds_refund_expire": "97a9d93b-75c6-427d-891c-3ffe0ab0f274",
  "stripe_post_test_helpers_terminal_readers_reader_present_payment_method": "3ae297dc-1175-4559-b060-9bb9ec28ec67",
  "stripe_get_test_helpers_test_clocks": "5bd04241-3b3a-4bb5-87be-8637d27e05e8",
  "stripe_post_test_helpers_test_clocks": "51ef600d-ae98-4669-879e-f0fb3bcad436",
  "stripe_delete_test_helpers_test_clocks_test_clock": "e2de4cda-11ba-45a6-9d7e-787d8b6cbeb6",
  "stripe_get_test_helpers_test_clocks_test_clock": "8183904e-df6b-4905-a7bf-4e764275a806",
  "stripe_post_test_helpers_test_clocks_test_clock_advance": "fb3148b1-55b9-4b59-9e6b-6c00bcb59574",
  "stripe_post_test_helpers_treasury_inbound_transfers_id_fail": "dc14e954-0a34-4e14-850e-ab0485362676",
  "stripe_post_test_helpers_treasury_inbound_transfers_id_return": "8085d72d-06f3-4477-be95-92b684f5e10a",
  "stripe_post_test_helpers_treasury_inbound_transfers_id_succeed": "d830374e-fa1d-48c2-a648-8b1e91e6fce9",
  "stripe_post_test_helpers_treasury_outbound_payments_id": "a23cfb72-07c3-4ac1-929e-b95065cd3031",
  "stripe_post_test_helpers_treasury_outbound_payments_id_fail": "e3d24672-e705-41c9-8392-e9be26d91d23",
  "stripe_post_test_helpers_treasury_outbound_payments_id_post": "7ee7b99d-c0b5-49a5-8ccd-7f83b274d0d6",
  "stripe_post_test_helpers_treasury_outbound_payments_id_return": "5b2ea304-c73f-4027-8ad2-88539d8c4fd8",
  "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer": "c8bd71da-9756-47de-99ed-8534b27160cf",
  "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer_fail": "f31e6478-40f6-45de-8d76-3f0889285f26",
  "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer_post": "ccdad07e-0ce7-4030-a12a-c2428362f403",
  "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer_return": "2f881ec5-1b23-41b0-be56-3cba68f6cbab",
  "stripe_post_test_helpers_treasury_received_credits": "17d31778-3425-4309-899d-e2f5f7917d4a",
  "stripe_post_test_helpers_treasury_received_debits": "0f22a740-0f4e-4592-ab05-25c416ddaa8f",
  "stripe_post_tokens": "5d09c036-a130-4e3b-b114-e219cf3e80fd",
  "stripe_get_tokens_token": "39a9f227-4706-4fc2-a329-36ee73b2f695",
  "stripe_get_topups": "83cc0e03-4e77-4bfe-bb5c-0f72aa8daa4a",
  "stripe_post_topups": "f517b124-4999-43ef-8eb4-94c077961d35",
  "stripe_get_topups_topup": "620b2c1f-6e24-4591-a2e8-e5c0403a0bc5",
  "stripe_post_topups_topup": "93bc9420-013b-4334-96d5-f67053487d98",
  "stripe_post_topups_topup_cancel": "47da25f4-07f1-4da8-9f0d-7816cc63a8fc",
  "stripe_get_transfers": "15a489e7-92cc-4835-8257-e11c9232f58b",
  "stripe_post_transfers": "d624a2ea-64cb-4614-b951-3915d79fe2e0",
  "stripe_get_transfers_id_reversals": "ef70b4fd-32ef-4027-8cf9-a6dc359800dc",
  "stripe_post_transfers_id_reversals": "0926b81f-8303-44c6-b6fe-5b36b7d33d00",
  "stripe_get_transfers_transfer": "adc4e103-5bb9-4b26-877c-b4c431c62c0d",
  "stripe_post_transfers_transfer": "202eba4a-7d0e-4dc2-b8b2-4ee3a3d3a953",
  "stripe_get_transfers_transfer_reversals_id": "69f364ee-bce9-4927-9981-ba8e3b06945b",
  "stripe_post_transfers_transfer_reversals_id": "d56bea50-c2d6-4f01-83ea-9235e03d325c",
  "stripe_get_treasury_credit_reversals": "93706590-cb42-4040-8187-0de15080143e",
  "stripe_post_treasury_credit_reversals": "b5975996-c9fd-4f40-aa04-b6ff24058f7d",
  "stripe_get_treasury_credit_reversals_credit_reversal": "94901627-cae9-4618-abd2-2aaa1298c677",
  "stripe_get_treasury_debit_reversals": "9301d4b0-5ae8-45d4-9844-6c9e9a1b4e7e",
  "stripe_post_treasury_debit_reversals": "9718f8f1-b5be-41ed-80d1-9a5f14559a96",
  "stripe_get_treasury_debit_reversals_debit_reversal": "8db75ac2-dbec-42ff-b196-d412614d6f2f",
  "stripe_get_treasury_financial_accounts": "a235e12f-b871-4595-9342-11328d65142f",
  "stripe_post_treasury_financial_accounts": "4a2e7200-5c03-4c2b-9191-dd00ae61039a",
  "stripe_get_treasury_financial_accounts_financial_account": "b7d7cb89-bc21-4fbc-b226-b2b6523796b1",
  "stripe_post_treasury_financial_accounts_financial_account": "136377b8-7376-4a9c-acf6-f8f12c4e718c",
  "stripe_get_treasury_financial_accounts_financial_account_features": "d3310119-a6f4-4afd-8e11-bb8169e09533",
  "stripe_post_treasury_financial_accounts_financial_account_features": "9ae0736b-4aca-478f-bc9b-f0c7a00c431f",
  "stripe_get_treasury_inbound_transfers": "8945f13a-bc0b-4853-86b6-c8cb09b82c7a",
  "stripe_post_treasury_inbound_transfers": "db742d53-5db4-485e-8e41-81efbc4886f8",
  "stripe_get_treasury_inbound_transfers_id": "12ca9fa1-58b8-4347-a18d-705462c659d6",
  "stripe_post_treasury_inbound_transfers_inbound_transfer_cancel": "940cc541-c159-42a5-aa52-8c7e357d605d",
  "stripe_get_treasury_outbound_payments": "1681d5aa-cc76-4781-858b-c39dd23898bd",
  "stripe_post_treasury_outbound_payments": "857bda62-c199-4911-b6b4-f3cd402ef277",
  "stripe_get_treasury_outbound_payments_id": "7b7337f0-3404-4402-90de-6ef2081da6f6",
  "stripe_post_treasury_outbound_payments_id_cancel": "48243142-34b9-486e-9f9d-e7e606a198ce",
  "stripe_get_treasury_outbound_transfers": "e9dddd0e-f191-45f1-b8e2-caa8bf74b533",
  "stripe_post_treasury_outbound_transfers": "e6769687-6f0e-4482-8ddf-b2cfd6783bfa",
  "stripe_get_treasury_outbound_transfers_outbound_transfer": "e361b930-4df2-4e24-b4a5-dc95c3814aad",
  "stripe_post_treasury_outbound_transfers_outbound_transfer_cancel": "12ef23b6-2819-4aaf-b9e2-687617da9949",
  "stripe_get_treasury_received_credits": "e4be3857-63e7-47a2-ad26-3ddaf3403646",
  "stripe_get_treasury_received_credits_id": "308ca308-3bee-4575-9dfd-4913bc9852c5",
  "stripe_get_treasury_received_debits": "0da95007-d1b9-4dd8-884f-a4c1016a3335",
  "stripe_get_treasury_received_debits_id": "e08026e6-1dc3-44a9-aca2-72dbdfabc372",
  "stripe_get_treasury_transaction_entries": "6f7fc993-2bdc-47ff-8db0-51638960a6ca",
  "stripe_get_treasury_transaction_entries_id": "5b485e78-095d-4aef-bf03-5c15c88130d2",
  "stripe_get_treasury_transactions": "00063776-2837-4e9d-98e9-f0376f8ee8e8",
  "stripe_get_treasury_transactions_id": "38685169-aa90-4388-ad31-67d3c2f7d42e",
  "stripe_get_webhook_endpoints": "d5c747db-b220-4017-985c-14f80826f7db",
  "stripe_post_webhook_endpoints": "3c76f048-0cc9-46a7-8b16-a1d98cab0eef",
  "stripe_delete_webhook_endpoints_webhook_endpoint": "a4cd6d3e-d229-4638-be7b-896505610efa",
  "stripe_get_webhook_endpoints_webhook_endpoint": "a0fa3319-2365-4182-821d-eabdd46d1c05",
  "stripe_post_webhook_endpoints_webhook_endpoint": "0deb580d-ca2e-4a8a-aa38-22ef0fb5fbf3"
}
    
    tool_lookup: ClassVar[Dict[str, str]] = {
  "23c64fc1-661e-4c7b-a3aa-497f030ee364": "google_calendar_calendars_insert",
  "59bf3965-9aa5-4869-bac6-cd4dc452ced6": "google_calendar_calendars_delete",
  "91531429-afc8-476e-a40d-8a78c084a4f2": "google_calendar_calendars_get",
  "5feaf5d5-7948-4c33-b0b9-9b8ff24763ba": "google_calendar_calendars_patch",
  "d13695fa-16ac-4cba-9b52-8082d8fe53ff": "google_calendar_calendars_update",
  "2a1edd60-382b-427c-bcb1-a6a9e63cf77a": "google_calendar_acl_list",
  "b5cbd079-c54e-4ca7-a811-d5e552c9f6b3": "google_calendar_acl_insert",
  "0e9ec2d4-29bd-44e6-a3ae-8d136ced15cf": "google_calendar_acl_watch",
  "40b0718d-16f3-41b7-8b3f-fb03d0604a48": "google_calendar_acl_delete",
  "167f359f-1646-4fe3-b07f-80bc0364ee8a": "google_calendar_acl_get",
  "8a8fdeaa-c381-4975-b069-7309302a8c31": "google_calendar_acl_patch",
  "6d2b9818-e7f1-4b2a-b9c0-cec7dd0227fb": "google_calendar_acl_update",
  "233568f0-594b-4a34-bf7d-ce01d3fb546c": "google_calendar_calendars_clear",
  "332a9eec-7dea-47eb-8ffa-38754b7b4b04": "google_calendar_events_list",
  "2d28468d-5853-48f2-a8eb-d9b0d60af818": "google_calendar_events_insert",
  "5ca1716b-34ca-4215-a945-632a4964fb16": "google_calendar_events_import",
  "d32fcc93-d457-4820-be8e-f96551da8074": "google_calendar_events_quick_add",
  "63164df3-730d-4f29-9f50-4847cf825bb1": "google_calendar_events_watch",
  "52f1cc5b-9971-4e3d-af13-2cd03a3a9c4c": "google_calendar_events_delete",
  "d4218736-bb85-4a37-b68c-5749d6d904a5": "google_calendar_events_get",
  "aec1922b-f891-4a78-b8be-c6fc18984408": "google_calendar_events_patch",
  "027e555c-fdcf-439a-992a-bdd710fb55fb": "google_calendar_events_update",
  "3458b88a-80ec-464b-bb35-c270c2e8000c": "google_calendar_events_instances",
  "a028dec8-bd26-4b13-86ee-9e9c6b655d5d": "google_calendar_events_move",
  "0e41881b-d4ac-4d3b-8b9b-12f83ee80f4f": "google_calendar_channels_stop",
  "e265889e-900d-46b3-88b8-2833b5df8ef3": "google_calendar_colors_get",
  "a100cbeb-b44e-4317-8469-64b5e1d6c232": "google_calendar_freebusy_query",
  "4dc25cdd-9345-4d14-81fa-2975aa46a718": "google_calendar_calendar_list_list",
  "e55d6679-ff3c-4e1d-b4e1-9d0efb79326a": "google_calendar_calendar_list_insert",
  "86263c65-2334-4575-a86f-f71774f36fb9": "google_calendar_calendar_list_watch",
  "4ad3e931-68f9-439b-88d4-f702425296cb": "google_calendar_calendar_list_delete",
  "02aeeafd-fbcb-4552-9adf-4c5e1d2698dd": "google_calendar_calendar_list_get",
  "649bc943-1ade-4256-afdd-05c5b3a65d0b": "google_calendar_calendar_list_patch",
  "643e2ba0-7bfe-4ce2-9a7b-c52d458625a4": "google_calendar_calendar_list_update",
  "79936e55-8ae8-402f-a486-675607fab809": "google_calendar_settings_list",
  "23808f3d-3364-47b5-bb7c-6b47cf6bb53d": "google_calendar_settings_watch",
  "52bdc86f-f7be-4a4b-ad51-7e6a90a46511": "google_calendar_settings_get",
  "268b0bdf-f685-4800-8c5a-6074f557581a": "nyt_article_search_json_get",
  "6e66674a-169b-49e3-b9ee-d44b51f2bde8": "youtube_data_abuse_reports_insert",
  "cf917181-fddf-4b11-91c7-724e0b5d0a74": "youtube_data_activities_list",
  "1b7a7709-fb43-4687-be59-a48d0e3af1a7": "youtube_data_captions_delete",
  "0fb1f531-e88a-4826-9be6-4a087da63adf": "youtube_data_captions_list",
  "f8a693d5-893a-486f-a4f1-4d816804b2a0": "youtube_data_captions_insert",
  "fec161ab-0dc7-4759-89a9-63f9039f5529": "youtube_data_captions_update",
  "de832bb2-89ad-4ede-91d3-db5a57544597": "youtube_data_captions_download",
  "d8d2b732-f37d-4478-932e-092828d7b305": "youtube_data_channel_banners_insert",
  "49223c69-3765-40ba-a461-2cdeb522de82": "youtube_data_channel_sections_delete",
  "ce62daac-ddbe-4cad-98b5-8e43a46a12ff": "youtube_data_channel_sections_list",
  "0c936210-3451-4e90-aff2-1fa3b3193fdd": "youtube_data_channel_sections_insert",
  "e8418a61-af14-44e3-9fcc-d8987f30e122": "youtube_data_channel_sections_update",
  "3e8bcd91-ad55-4404-8f72-b457afe3a846": "youtube_data_channels_list",
  "bb0bd39d-ab9c-419d-98b8-fcfdabc3c3bd": "youtube_data_channels_update",
  "ce270c94-bb54-4ff8-81fe-d4bc9f92bb65": "youtube_data_comment_threads_list",
  "3989c2ab-d1a8-43c5-ab12-0f216443cd32": "youtube_data_comment_threads_insert",
  "732610f1-2a59-40e7-ab7f-bac251720f17": "youtube_data_youtube_v3_update_comment_threads",
  "4e324040-9662-44d1-a047-08efa2eebeef": "youtube_data_comments_delete",
  "844fa072-4854-47df-93d0-9359fad64de3": "youtube_data_comments_list",
  "5d90225a-170d-4d8a-a442-c7f085ccabd8": "youtube_data_comments_insert",
  "189e855e-e3d8-4351-9190-dcdc283c1389": "youtube_data_comments_update",
  "d4c3da7e-a53d-4e26-9ab1-abaaf8cd5900": "youtube_data_comments_mark_as_spam",
  "abba54ae-ce3d-4617-ad32-beefc110d85f": "youtube_data_comments_set_moderation_status",
  "ce19d68f-e1f3-482c-a590-7073d804a876": "youtube_data_i18n_languages_list",
  "c336b40f-6500-4614-ac95-665b5420de28": "youtube_data_i18n_regions_list",
  "9efd75ec-a522-49a8-b142-77e06513d086": "youtube_data_live_broadcasts_delete",
  "7ddf3639-91d3-49cb-abd3-ab2ab36f44ef": "youtube_data_live_broadcasts_list",
  "4649fae7-4aac-4ca8-a2de-37594f98559e": "youtube_data_live_broadcasts_insert",
  "88cdd91a-6bef-497a-89c3-2a5c6e576b3c": "youtube_data_live_broadcasts_update",
  "9e51588b-f117-4789-bf1f-9e60154ed95b": "youtube_data_live_broadcasts_bind",
  "85f0b075-c4cc-4af1-84d5-e517a4f554af": "youtube_data_live_broadcasts_insert_cuepoint",
  "a0e7ffde-588e-448a-b5de-89bba158f825": "youtube_data_live_broadcasts_transition",
  "65ed97fe-6259-4dd9-9ae4-6ce61e446bcc": "youtube_data_live_chat_bans_delete",
  "de21e842-49eb-49c6-b29f-3dcb89fe23aa": "youtube_data_live_chat_bans_insert",
  "638e438d-f9b6-4bd9-9ffd-57a46ad9c286": "youtube_data_live_chat_messages_delete",
  "5383207d-9d01-4799-9891-e4e51fdd8b30": "youtube_data_live_chat_messages_list",
  "0aa2cae3-4ac3-4ab9-8abb-4cc82d3b415f": "youtube_data_live_chat_messages_insert",
  "188acbd2-d1f6-4a6e-b906-700d3657db4e": "youtube_data_live_chat_moderators_delete",
  "5939ee25-449c-42c6-a83a-3ea29b81fec0": "youtube_data_live_chat_moderators_list",
  "93aec2d3-b733-485b-88e6-a306b80c0009": "youtube_data_live_chat_moderators_insert",
  "583b3bbd-c57a-4e41-86e4-086046fdbe94": "youtube_data_live_streams_delete",
  "11c36dc4-1b09-4541-a553-06758face74f": "youtube_data_live_streams_list",
  "a95cd90f-773a-48cb-aef9-e394efcf9bec": "youtube_data_live_streams_insert",
  "323bddac-e4f1-40cd-9ca1-a988e9c73225": "youtube_data_live_streams_update",
  "6d1c640b-63cc-40c3-9ea9-5eb0f517067c": "youtube_data_members_list",
  "c465557b-3c69-434f-bd2c-452c340f9d16": "youtube_data_memberships_levels_list",
  "08d37613-bacf-4f33-9fd3-9ad8c069e9de": "youtube_data_playlist_items_delete",
  "3e3ef84c-6583-41f4-9231-fd423f86372e": "youtube_data_playlist_items_list",
  "468bdb3d-8f8d-4f0e-9a48-700ae47ca3e2": "youtube_data_playlist_items_insert",
  "d47b617c-1652-44d8-90d0-17a9802699cd": "youtube_data_playlist_items_update",
  "ccb10df0-37e0-48d3-9347-9f07200966e5": "youtube_data_playlists_delete",
  "8ec20190-f918-424d-984e-95f73bcf4085": "youtube_data_playlists_list",
  "c62eabad-a03e-4842-9d5d-89199cfd6862": "youtube_data_playlists_insert",
  "ae8fbdce-f600-489b-9da6-54a5f14e1658": "youtube_data_playlists_update",
  "242050bb-2d0e-4c67-b9ab-ecf180853e3b": "youtube_data_search_list",
  "0aba906b-9b33-4e49-98bb-70111d10ff44": "youtube_data_subscriptions_delete",
  "67165cca-44a7-421a-bf11-0285603e8015": "youtube_data_subscriptions_list",
  "2618e971-6e8e-4005-b207-fbb473a9d9dc": "youtube_data_subscriptions_insert",
  "a195831a-bd74-40eb-a1dc-40c5730c8994": "youtube_data_super_chat_events_list",
  "d7974383-b71a-48c9-9069-ea9d04c711aa": "youtube_data_tests_insert",
  "e7fb0eb8-d8c2-48f5-a19d-9fcf8986c242": "youtube_data_third_party_links_delete",
  "9f9a7e5a-cb72-4dd9-821a-74cd79fbdd45": "youtube_data_third_party_links_list",
  "a12c9156-87d0-4c6f-90ba-acb83f31e2fc": "youtube_data_third_party_links_insert",
  "2106d5fc-fe3c-4ee9-aeae-76b06f803216": "youtube_data_third_party_links_update",
  "b36f81d2-9cef-4195-aab3-00afaf382f73": "youtube_data_thumbnails_set",
  "3954e81f-f1ef-4418-ab19-b65bbd66e479": "youtube_data_video_abuse_report_reasons_list",
  "4b2a0cbc-4e6e-4d47-b6fd-060b942ce8da": "youtube_data_video_categories_list",
  "4cd10ed7-6910-4b6b-b48b-9e0491398179": "youtube_data_videos_delete",
  "eb161474-5be4-4990-95e8-536f81e39dab": "youtube_data_videos_list",
  "409d12eb-ae67-4958-a61b-91b679b8b9d7": "youtube_data_videos_insert",
  "3347d8f0-f534-4c3c-bb5c-384e8fd76dc1": "youtube_data_videos_update",
  "9cde234e-3430-4556-8673-704ba78fb2b5": "youtube_data_videos_get_rating",
  "8cb8958d-1248-4034-91ca-bcda4cf0aad5": "youtube_data_videos_rate",
  "0aecd43f-20b6-4383-8a36-47255bd21533": "youtube_data_videos_report_abuse",
  "1345e324-7444-4ee4-8822-cf20eba6869b": "youtube_data_watermarks_set",
  "e8ce0ba4-2c58-49ce-b12b-7c0f6d1a3e68": "youtube_data_watermarks_unset",
  "b44c3aa7-cda9-4599-a31b-74f8eb8806f2": "notion_delete_ablock",
  "8292cb92-90da-4928-8830-114bbd1517b7": "notion_retrieve_ablock",
  "01193063-f704-4586-99dd-f1d1f5fb188e": "notion_update_ablock",
  "8fd92edf-60ff-4670-892a-bcf70b776313": "notion_retrieve_block_children",
  "5f73c88a-62d4-4b28-96e9-8dceec96feca": "notion_append_block_children",
  "3cab9ad5-9818-4b19-a343-1e7bb6f172c9": "notion_retrieve_comments",
  "70d0d0ca-da87-49a5-9aed-ef954c263805": "notion_retrieve_adatabase",
  "c3b7496d-1f3b-4dd6-9b78-38a83d6ed292": "notion_update_adatabase",
  "049fb738-1b89-4ba6-9083-8639fc4bb5f5": "notion_query_adatabase",
  "1b15030a-ba70-4717-b369-9cb672497220": "notion_retrieve_apage",
  "3a31d285-56dd-44d5-9c6d-2f735c7ba3a1": "notion_update_page_properties",
  "510fcd48-88cf-411c-88f0-fbfcbbcd9782": "notion_retrieve_apage_property_item",
  "6e9a79fb-4709-481b-9f28-c6e8e459c271": "notion_retrieve_auser",
  "3d82af92-d727-49bc-acc9-5bece2e09898": "airtable_get_user_info",
  "f2a8b4d6-9c3e-4f7a-b5d1-8e2c6b9a3f7d": "airtable_records_list",
  "c7e5a9d2-4b8f-3e6d-9a1c-5f8b2d7e4c9a": "airtable_records_update_multiple",
  "b6d4c8a3-7f2e-5b9d-8e4a-3c6f9b5d2a8c": "airtable_records_replace_multiple",
  "a5c3b7d9-6e4f-2c8a-9b5d-7f3e8c4a2b6d": "airtable_records_create",
  "d8f6e4c2-9b7a-1d3e-5c8f-4a2b6d9c7e5a": "airtable_records_delete_multiple",
  "e9d7c5a3-8b6f-4d2e-7c9a-5b3f8d6e4c2a": "airtable_records_get",
  "c7d5e3b9-8a6f-2d4c-5b7e-9f3a8d6c4b2e": "airtable_records_update",
  "f8e6d4b2-7c5a-3e9d-6b8f-4a2c9d7e5b3a": "airtable_records_replace",
  "b6c4a8d2-9e7f-5b3d-4c8e-7f2a9b5d6c3e": "airtable_records_delete",
  "be4ed291-958e-49be-9f84-e458a1745a9f": "airtable_records_sync_csv",
  "637fc379-3502-44fd-925b-e4d40248d827": "airtable_records_upload_attachment",
  "018b13ea-3a01-4608-9d30-3d06f5d35195": "airtable_fields_update",
  "ad675da4-a83c-4151-8431-5c170225ed37": "airtable_fields_create",
  "9398d0f4-0a97-4a89-93ef-fa5af5fd10b0": "airtable_comments_list",
  "d3cce98d-69bd-436a-aaa5-672a80edb0be": "airtable_comments_create",
  "870423ec-dc89-4fcb-ad42-3efd0f023c73": "airtable_comments_update",
  "ca85acd7-d2ca-4b40-af30-2ade01b8ad46": "airtable_comments_delete",
  "7a00cde3-383e-4995-a76a-0f99827d8ee6": "airtable_tables_update",
  "0fe8ca56-21b3-4359-bde8-f927e37b6f40": "airtable_tables_create",
  "c8a19765-1b16-4df6-a65d-b2eb7e3cd1bd": "airtable_bases_schema",
  "cda58275-c1a8-4fc5-96ab-17b26289ed68": "airtable_bases_list",
  "03a28d60-c9a7-418f-b67c-ecbf075cb962": "airtable_bases_create",
  "2d988273-3c02-4dbb-80ce-eda9397186b0": "slack_web_api_admin_apps_approve",
  "ea49d928-212c-47d5-a95a-5bdb554d68ad": "slack_web_api_admin_apps_approved_list",
  "209e5d1f-de3b-4e62-8ccf-fffe630df927": "slack_web_api_admin_apps_requests_list",
  "d40c1ba3-9f0e-4617-9096-6f12298c36d5": "slack_web_api_admin_apps_restrict",
  "4c43b355-520d-4063-a475-193f3e22e7b7": "slack_web_api_admin_apps_restricted_list",
  "897574c1-55d4-467b-8157-53ca58a6d240": "slack_web_api_admin_conversations_archive",
  "db3b8356-269b-4a8e-a4b4-ca07270d3137": "slack_web_api_admin_conversations_convert_to_private",
  "455b1d34-18ce-45f7-af24-91ffcecc4b48": "slack_web_api_admin_conversations_create",
  "9e909ead-4734-4060-afd4-1aafcc24ce5c": "slack_web_api_admin_conversations_delete",
  "0e6bb466-5c0a-4401-bc0b-b3b5ac86aa57": "slack_web_api_admin_conversations_disconnect_shared",
  "41ded63c-7f88-47fd-8083-1d0b4c6e6888": "slack_web_api_admin_conversations_ekm_list_original_connected_channel_info",
  "8ca55dec-27a2-4ff8-af9e-fbd2dca6d3c4": "slack_web_api_admin_conversations_get_conversation_prefs",
  "a963ecec-2927-4b4a-a77d-f8f05b488e3b": "slack_web_api_admin_conversations_get_teams",
  "66936658-6388-47c4-9c4a-fe922f0a6641": "slack_web_api_admin_conversations_invite",
  "4b0ce9cb-0386-40d8-aee2-cf9114693284": "slack_web_api_admin_conversations_rename",
  "c1b2c5d0-e9d0-4dea-8703-57849098abc6": "slack_web_api_admin_conversations_restrict_access_add_group",
  "7305201a-72c4-4df2-a3f1-f604ae94a896": "slack_web_api_admin_conversations_restrict_access_list_groups",
  "f54bc5a3-cb3b-4c66-a077-419f2c1be685": "slack_web_api_admin_conversations_restrict_access_remove_group",
  "d62f3d11-da1e-4744-ba46-f970ad89fc29": "slack_web_api_admin_conversations_search",
  "f67c6bfc-cbac-4701-b892-47bd74bb3df1": "slack_web_api_admin_conversations_set_conversation_prefs",
  "12f199e5-84f9-4963-8cde-6a6b9cd7e008": "slack_web_api_admin_conversations_set_teams",
  "15caaacb-a1e5-4500-a048-3c5e8f7dc6ee": "slack_web_api_admin_conversations_unarchive",
  "1dff1f42-44f5-47ab-b985-b7f2deb28daf": "slack_web_api_admin_emoji_add",
  "93dd10d8-74df-461b-933c-d5f594a86bc1": "slack_web_api_admin_emoji_add_alias",
  "9868d68e-49e8-437a-8b97-eab44b639460": "slack_web_api_admin_emoji_list",
  "9c46890f-4876-451d-b329-b967bdb2beaf": "slack_web_api_admin_emoji_remove",
  "97c9bf07-0910-4f23-a61d-10e0a38a7829": "slack_web_api_admin_emoji_rename",
  "47ab7560-7b1d-48e7-b05d-481830bf7f62": "slack_web_api_admin_invite_requests_approve",
  "8afa24bb-7c1f-41f6-9125-c80498fa73b6": "slack_web_api_admin_invite_requests_approved_list",
  "67593af7-9c0f-4de9-b170-0413cb51c547": "slack_web_api_admin_invite_requests_denied_list",
  "b5edc6c1-41e7-491d-a1cd-7fdb283af1d0": "slack_web_api_admin_invite_requests_deny",
  "ad8fb2e9-957f-4810-842d-75a9c36e7009": "slack_web_api_admin_invite_requests_list",
  "a178015c-8023-4b03-a210-b24bf542303f": "slack_web_api_admin_teams_admins_list",
  "3aa1f92e-4fa6-449a-ba10-47dceb1df8ec": "slack_web_api_admin_teams_create",
  "3c3370fc-9a09-4f9c-bfbb-b02d51e0c010": "slack_web_api_admin_teams_list",
  "76aa4296-e834-48ca-a0d5-f3bbbadd86af": "slack_web_api_admin_teams_owners_list",
  "8ca8f1ec-7358-4e99-a0a7-3dbc8e57a8ec": "slack_web_api_admin_teams_settings_info",
  "a59cf694-b587-44ef-8d8e-0bb0ed3ab4d4": "slack_web_api_admin_teams_settings_set_default_channels",
  "c538ce06-0eb5-4666-b188-71c21da521d8": "slack_web_api_admin_teams_settings_set_description",
  "ac9306a5-fd74-4d05-87e2-7cc59bc04ef9": "slack_web_api_admin_teams_settings_set_discoverability",
  "7d8fa47f-16d4-4aa3-b087-f16726040b47": "slack_web_api_admin_teams_settings_set_icon",
  "8d17e673-1ff8-4fc5-80ec-583a800453f0": "slack_web_api_admin_teams_settings_set_name",
  "e98cc7e6-8805-4946-af9e-57c85e300bd2": "slack_web_api_admin_usergroups_add_channels",
  "b710819c-50b6-463f-97da-ec27f1e3464b": "slack_web_api_admin_usergroups_add_teams",
  "6ad0c86f-06ab-44dc-b619-55a98d029cec": "slack_web_api_admin_usergroups_list_channels",
  "fdd693aa-8c03-43a8-93bd-1ab2d5c5eca9": "slack_web_api_admin_usergroups_remove_channels",
  "e308f6ce-22ab-40ce-8a72-9f6e4ff9e073": "slack_web_api_admin_users_assign",
  "d6538e33-6c1f-41db-87be-17f33399c108": "slack_web_api_admin_users_invite",
  "d704e492-b26a-458d-816c-ddb3f716a2e4": "slack_web_api_admin_users_list",
  "e8b18bd8-7aa2-46ea-af6f-80cd4103bc37": "slack_web_api_admin_users_remove",
  "c1d3bd32-0bd8-4430-91ca-31403782e4ac": "slack_web_api_admin_users_session_invalidate",
  "d28fd650-1745-4115-ad43-48b5590fe6e5": "slack_web_api_admin_users_session_reset",
  "7d7cf8ee-c1e0-4ee0-b18a-49bec22d2142": "slack_web_api_admin_users_set_admin",
  "f6514f00-cda1-4a27-beef-e4426d18a527": "slack_web_api_admin_users_set_expiration",
  "a6e69430-0f87-46f8-b673-597f9816d584": "slack_web_api_admin_users_set_owner",
  "1f311687-72e4-4b55-b258-574388984807": "slack_web_api_admin_users_set_regular",
  "f8f14d03-81ab-4ba0-8ac1-dce1ca2939f1": "slack_web_api_api_test",
  "8658bdba-da20-4b4b-98a1-77ca88d15830": "slack_web_api_apps_event_authorizations_list",
  "8e9b3729-4c4d-4ae6-8306-2614f7d479e6": "slack_web_api_apps_permissions_info",
  "0086e09d-b934-47a4-9cde-ab07be638a39": "slack_web_api_apps_permissions_request",
  "0a88214d-c35a-4a74-8ef3-3e422357d2f4": "slack_web_api_apps_permissions_resources_list",
  "d4edc09d-dfe5-4671-a7e6-1f01645df57c": "slack_web_api_apps_permissions_scopes_list",
  "510a09c9-28ac-42eb-b5bc-39bcdd61a9f4": "slack_web_api_apps_permissions_users_list",
  "a9dbd01c-5b3b-40be-8b4c-0b95c03e2b9d": "slack_web_api_apps_permissions_users_request",
  "6a40f668-8860-4855-b535-5c3fbb22d10b": "slack_web_api_apps_uninstall",
  "7554134d-b3b0-47e1-9d17-fbb8ca979bb3": "slack_web_api_auth_revoke",
  "6eac46a5-3f13-4fc6-96c4-401c6271f43b": "slack_web_api_auth_test",
  "56724622-4a06-471f-9852-e5c5f9305b99": "slack_web_api_bots_info",
  "561130a3-abf3-4b7a-96dc-262c9f33fe6a": "slack_web_api_calls_add",
  "696d63f8-dcd4-41b7-b563-c87efa017f5c": "slack_web_api_calls_end",
  "205ec1ab-f828-41d2-88cc-436677608461": "slack_web_api_calls_info",
  "091679a4-554d-47c0-ace5-ef80b136f4fc": "slack_web_api_calls_participants_add",
  "9f8b5e14-0478-4e2b-bf36-b6673bb95bf7": "slack_web_api_calls_participants_remove",
  "3912525f-7bc2-4cc9-9efd-80af484dfc89": "slack_web_api_calls_update",
  "a47a84b7-29fb-4b40-9f37-0dc396ce7d85": "slack_web_api_chat_delete",
  "44e4e59d-501f-4dc6-a9e8-038777ba8c1c": "slack_web_api_chat_delete_scheduled_message",
  "80c6cace-de35-4091-b069-b26561302258": "slack_web_api_chat_get_permalink",
  "e1246f3a-d308-4cf0-b52f-aeaaff2546f8": "slack_web_api_chat_me_message",
  "60267f29-bccf-47e9-a2ce-7d66cdf3cf50": "slack_web_api_chat_post_ephemeral",
  "03745bdc-8751-40e7-88e0-6af6ca99e2b8": "slack_web_api_chat_post_message",
  "420a4fee-330c-4701-92fa-4792860fef8d": "slack_web_api_chat_schedule_message",
  "d3e036c5-68da-4963-9be2-bf46a61bb312": "slack_web_api_chat_scheduled_messages_list",
  "050054da-36e6-4bba-ba80-d55cb1cc837c": "slack_web_api_chat_unfurl",
  "904773eb-140d-4065-879d-670492a3a935": "slack_web_api_chat_update",
  "a7141f73-b5cf-47a7-ab03-4394626a1fbe": "slack_web_api_conversations_archive",
  "823e5802-3f4a-42d4-a8f8-f7d9b33d4ca3": "slack_web_api_conversations_close",
  "a52b4ef6-4ae5-45e0-adab-57d072b034a0": "slack_web_api_conversations_create",
  "906e8d87-cef2-4e62-a367-cf791b84e06d": "slack_web_api_conversations_history",
  "daa6d78e-1a4c-4a07-b245-2b6ccddbf485": "slack_web_api_conversations_info",
  "8242f6ee-b7a0-4708-9375-7ee27ae48a33": "slack_web_api_conversations_invite",
  "bd1d7048-3166-43c7-8a07-c13d9f405630": "slack_web_api_conversations_join",
  "43727777-792f-4572-bf8c-04e418bd12f8": "slack_web_api_conversations_kick",
  "09057030-607b-4452-b1fa-2c99b0299b55": "slack_web_api_conversations_leave",
  "77c9f68b-d48e-4e41-adc6-9931c4e08e8f": "slack_web_api_conversations_list",
  "18bb58b1-ad32-42c1-accd-4d8e660c9d0e": "slack_web_api_conversations_mark",
  "17ed2dca-dd28-4be1-8d9a-a67d2ca1b5dc": "slack_web_api_conversations_members",
  "6f121113-9f08-4bb5-b8d1-883b19b6de3a": "slack_web_api_conversations_open",
  "9c76a23c-9024-4e71-87cf-5349ddb1937e": "slack_web_api_conversations_rename",
  "9adf4e13-83b8-4b5b-ad0f-4152e14af376": "slack_web_api_conversations_replies",
  "eac63de8-dd23-4450-9341-68f5f3c5deaf": "slack_web_api_conversations_set_purpose",
  "19b38015-b7da-4e41-bd20-79bca9bfbcdf": "slack_web_api_conversations_set_topic",
  "8d0299f6-a43b-430b-8b7b-2fe27c0ded55": "slack_web_api_conversations_unarchive",
  "2380b7eb-9bae-4a06-aaf6-3c4e551f0732": "slack_web_api_dialog_open",
  "d78c0e98-fac3-4114-bdf2-b4be24dc4cde": "slack_web_api_dnd_end_dnd",
  "d26d9c91-1c1c-4791-baf6-38ac35e71b3c": "slack_web_api_dnd_end_snooze",
  "6712e8af-1b72-4b25-b68b-a65041f6f479": "slack_web_api_dnd_info",
  "5f5f119d-7c68-49bd-8e7f-df60764be6d5": "slack_web_api_dnd_set_snooze",
  "d61a2f39-6959-4820-be19-0274c42646f1": "slack_web_api_dnd_team_info",
  "aed76ce1-3ff5-4817-ad1d-1ebb6e1ef9b7": "slack_web_api_emoji_list",
  "b77bb3c0-9543-4fc8-865b-f3f8bedffae5": "slack_web_api_files_comments_delete",
  "c58b80be-d759-4793-85dd-5bf8bad6d4c8": "slack_web_api_files_delete",
  "905ff0a1-17b3-413e-b0cc-bf07f20979b2": "slack_web_api_files_info",
  "35bdb847-51b9-4512-ab59-726eb58c03d4": "slack_web_api_files_list",
  "4a4b11db-7e21-4112-9dcc-c83179e79d3c": "slack_web_api_files_remote_add",
  "eae2c2d5-dc14-4373-9302-3d8cce4473f1": "slack_web_api_files_remote_info",
  "109c4330-c134-4ae7-987e-15ccc03b59b8": "slack_web_api_files_remote_list",
  "6e3cbcc3-503d-46ce-bf82-8a2e1139fbb6": "slack_web_api_files_remote_remove",
  "676cf9c7-b500-4a72-86c0-0efa1dc47ad0": "slack_web_api_files_remote_share",
  "2d2f60c2-9674-46ff-8321-ce2d64208378": "slack_web_api_files_remote_update",
  "fca2ff0b-3462-424d-9fd3-0cda740228bb": "slack_web_api_files_revoke_public_url",
  "e4b3427b-e17f-4522-859a-f5edfaaf3b92": "slack_web_api_files_shared_public_url",
  "08d48527-aaff-411b-9a3e-b358a7a2ad83": "slack_web_api_files_upload",
  "a7f37606-f56f-43da-914c-b948d5c06e28": "slack_web_api_migration_exchange",
  "29098431-9024-4419-b6b9-098b05d4c54f": "slack_web_api_oauth_access",
  "6d82add8-52aa-4065-ae4e-3756a7bc0347": "slack_web_api_oauth_token",
  "67e469ae-2679-45c0-977f-0210f95a6653": "slack_web_api_oauth_v2_access",
  "b4d20f5d-1796-417d-be0d-68e1f681d70e": "slack_web_api_pins_add",
  "784b0fc9-79c3-4767-8720-f01262309813": "slack_web_api_pins_list",
  "bcd76f70-486f-4314-8b84-a01bd018570b": "slack_web_api_pins_remove",
  "e01bd93a-b4d1-40d9-b3f4-a5f72b02b9b6": "slack_web_api_reactions_add",
  "697be0f9-c1b7-4220-b82b-f664051f8d01": "slack_web_api_reactions_get",
  "4a4cd900-bd75-44ed-9189-2eb3d24215fa": "slack_web_api_reactions_list",
  "7fd6c325-5444-404c-894a-9a6e13507719": "slack_web_api_reactions_remove",
  "b6921860-fc34-44ac-928c-d85c5cc9fce3": "slack_web_api_reminders_add",
  "502a2872-3d92-4fb0-a27d-3f972fe501d9": "slack_web_api_reminders_complete",
  "ef31e971-04ae-4302-a31a-4b3e146730bb": "slack_web_api_reminders_delete",
  "d816eb0d-7829-4ebc-9d65-db5dc0244fe5": "slack_web_api_reminders_info",
  "69100e6e-06da-4416-9001-9d3b3c44fcfa": "slack_web_api_reminders_list",
  "da3dc2d1-d49e-4a94-a965-668094fe44a5": "slack_web_api_rtm_connect",
  "dbb35b5e-45db-4773-944e-a30dfb44b2f2": "slack_web_api_search_messages",
  "5dac944e-babb-4815-b983-43222649f648": "slack_web_api_stars_add",
  "1f0db219-7a49-47e3-8963-b30cf39ce9d4": "slack_web_api_stars_list",
  "add237f7-27f1-4ce3-918d-2900324848e8": "slack_web_api_stars_remove",
  "fadf63e2-646c-42a3-893b-81282f259f99": "slack_web_api_team_access_logs",
  "b410797c-f153-48ed-9ef6-753466d6a3dc": "slack_web_api_team_billable_info",
  "450d5fe5-214a-4ec5-b7a3-5c1ce019210a": "slack_web_api_team_info",
  "af19f9ab-ea8d-4ced-91ac-6d8c9e8ebbce": "slack_web_api_team_integration_logs",
  "23e3b115-5ac8-4820-9aaa-792886b79f23": "slack_web_api_team_profile_get",
  "54f0f0d3-5bc2-4a43-ba13-a55e941a5223": "slack_web_api_usergroups_create",
  "0c1d737d-df91-4dc5-ada1-18c7e2c45544": "slack_web_api_usergroups_disable",
  "46de3d09-12c6-4e38-9d08-84b05af8f1db": "slack_web_api_usergroups_enable",
  "674bd393-57df-4a7c-a2b7-63ad9e10e848": "slack_web_api_usergroups_list",
  "39da7cf2-4053-4ee6-8ce8-abb921e572b2": "slack_web_api_usergroups_update",
  "e065f8ee-83d4-4428-9fdc-dfaeba83f78a": "slack_web_api_usergroups_users_list",
  "e99a0d6e-1657-4bf0-836b-0b2a4085dbad": "slack_web_api_usergroups_users_update",
  "2247afd4-7abc-4937-9d41-bae66e25d926": "slack_web_api_users_conversations",
  "6d47a470-ba8d-492f-9018-fcfdf54d4f4c": "slack_web_api_users_delete_photo",
  "1f24f954-0176-4d1b-8e94-bdda5755aab7": "slack_web_api_users_get_presence",
  "4dab75cb-a5f8-4f8a-b403-c6cceabd74d2": "slack_web_api_users_identity",
  "1561a995-bbbd-49f8-a3f0-4d08f99222d3": "slack_web_api_users_info",
  "84cef0df-6686-4fdf-bf6f-1aa39af282b9": "slack_web_api_users_list",
  "1f27abf6-57a7-4550-a10a-589e10a74bdb": "slack_web_api_users_lookup_by_email",
  "807847b7-38a1-4952-8a73-ad4a6aa757ad": "slack_web_api_users_profile_get",
  "078659eb-de74-46a9-b083-8c2c42c4f503": "slack_web_api_users_profile_set",
  "89ea6143-0b5c-4f47-b4d3-e0d1f3bdd8bf": "slack_web_api_users_set_active",
  "3597578b-2ef1-4ece-bea5-c5f341fe3b45": "slack_web_api_users_set_photo",
  "979bf218-0c29-4abe-a182-c55dd13570cf": "slack_web_api_users_set_presence",
  "83768320-21af-4320-b526-f4369b12ea99": "slack_web_api_views_open",
  "5890b5f0-916a-41c4-bec5-667bd83be8c0": "slack_web_api_views_publish",
  "565bcf4f-55e1-4437-bc29-9e9d85df9bd1": "slack_web_api_views_push",
  "354048d2-4135-4fec-994f-d82171cfd698": "slack_web_api_views_update",
  "ca34131e-d991-4eea-b90d-454389a08f79": "slack_web_api_workflows_step_completed",
  "6315e24e-f4b1-4afe-970f-9608b82225f7": "slack_web_api_workflows_step_failed",
  "dd3bc4dd-4bd4-4d94-998d-6469c93bbe3c": "slack_web_api_workflows_update_step",
  "ecb22f58-80d7-462c-abca-7aae7f6b895c": "zoom_accounts",
  "da2c2bac-ca71-4ed6-8a29-44600ba3606d": "zoom_account_create",
  "badca3d8-51b5-4d71-a168-10dbd29a3da8": "zoom_account_disassociate",
  "94ce0ddb-3d8a-4e34-a42c-1101620b83ce": "zoom_account",
  "d6e4c46e-dc11-4692-ba6f-40c05ccdd1a5": "zoom_account_billing",
  "29d787b3-c4bb-4ec5-8f13-c7af349693ae": "zoom_account_billing_update",
  "0416ecf6-043b-412b-b0f8-2f4ab7148f97": "zoom_account_billing_invoices",
  "311dbf21-4c71-4e81-9989-f5c135d48e40": "zoom_get_account_billing_invoice",
  "7a2a6762-de12-495e-b790-238f0d6cb8a6": "zoom_get_account_lock_settings",
  "dd862e4a-380f-4bc2-b6d0-85fd2c4ce185": "zoom_update_account_lock_settings",
  "89a4f542-f5db-4247-8333-e2d395ecceb4": "zoom_account_managed_domain",
  "fc684d91-2d66-4b70-bcf1-0b8f42af39dc": "zoom_account_options_update",
  "563dcb09-af86-4076-8285-4f741eea7b0a": "zoom_update_account_owner",
  "3f340237-81a1-4b8f-8587-7ce8ab7a7185": "zoom_update_phone_settings",
  "7d9b8335-bf83-4134-86c5-a341416cc5a0": "zoom_set_up_account",
  "00bf57b5-bb9d-487d-895c-da7754d93ac4": "zoom_post_phone_siptrunk",
  "67382817-1cf8-476d-b72b-f70026d26e59": "zoom_update_phone_siptrunk",
  "63a3f311-6055-4fe6-bbe8-90b10c10c06b": "zoom_account_plans",
  "9da81c16-5a7c-4f36-8856-8385cc191483": "zoom_account_plan_create",
  "d331dd44-b702-4197-a46a-7bef71bdd5a3": "zoom_account_plan_addon_create",
  "225c331f-935d-43c0-936f-562d70a03a30": "zoom_account_plan_addon_update",
  "0d4d0204-a790-4c40-b9c8-1ec3c45954a8": "zoom_account_plan_addon_cancel",
  "9e14a6e7-05b1-4e73-b709-f3aba65fda4d": "zoom_account_plan_base_update",
  "b6539ff0-0103-4ccb-a8c2-aa3d9149dd66": "zoom_account_plan_base_delete",
  "026716e5-ffa9-4c04-8339-426a02f977bc": "zoom_get_plan_usage",
  "43841fe4-dbe1-4f3f-a9dc-5fc2b02c153a": "zoom_get_account_cloud_recording",
  "1f88ce76-b944-457b-b18a-887f5e36627f": "zoom_account_settings",
  "de8f85e4-c655-48c8-90e5-310777258284": "zoom_account_settings_update",
  "799f4496-c489-41cb-a394-725a04e89792": "zoom_del_vb",
  "550a918d-d2dd-464a-9bff-1e0d52f3129c": "zoom_upload_vb",
  "d7dfdc35-f315-4c13-a741-767cd07c85c3": "zoom_list_internal_callout_countries",
  "e030f1de-afd6-472c-bc33-66f683bda62d": "zoom_add_callout_countries",
  "6870109a-83ae-4c34-be18-9904c0623119": "zoom_delete_internal_call_out_country",
  "ab2b53a7-f2de-40dc-bd27-96ce62711b99": "zoom_list_internal_numbers",
  "e8bea3ea-e31c-4c4b-9da5-47e1942b6fc3": "zoom_add_internal_numbers",
  "73383dcc-89ee-45ae-924d-fec7d576d403": "zoom_delete_internal_number",
  "362ae8ce-9434-40f3-850d-82ff664bd06f": "zoom_delete_all_sip_numbers",
  "218e7694-4e5b-4c8e-a790-53a532f70400": "zoom_assign_sip_trunk_numbers",
  "67d62721-44ff-42ce-8fd2-77df0aa3ce39": "zoom_assign_sipconfig",
  "6dd42ccb-3153-45be-aad1-5a2ceb69ba5d": "zoom_list_siptrunks",
  "1937dbbf-7978-4de6-b5fc-dbffb5c8278a": "zoom_assign_siptrunks",
  "e37b8551-d0cf-4af1-8b2a-b1c7a47a06f0": "zoom_delete_siptrunk",
  "039ba639-ddc5-4557-a531-efb7dcc645c5": "zoom_account_trusted_domain",
  "fdf10e07-9353-4520-860b-ed3c987961ca": "zoom_switch_user_account",
  "12504257-32cc-4add-baf4-48db9c91a41c": "zoom_download_invoice_pdf",
  "dda35518-8167-4b53-8678-61dfe850864c": "zoom_list_archived_files",
  "58e69feb-e1f0-46a6-b817-7cc22db3bf4b": "zoom_delete_user_level_channel",
  "7f799d5f-75ac-49b4-aa93-e5904459e672": "zoom_get_user_level_channel",
  "4d93017d-dfdf-4fea-bacd-42e20c774980": "zoom_update_user_level_channel",
  "30e4765e-991e-429b-8c69-96ee939bdf82": "zoom_leave_channel",
  "920f86cb-6c9b-4805-9683-d0a3e9ee04c1": "zoom_join_channel",
  "0f67c9f9-4ff6-41b0-beb4-2789eb7ab93c": "zoom_remove_auser_level_channel_member",
  "4e92a1c0-11d0-470b-ad3e-c5a7852652b3": "zoom_get_user_contacts",
  "7091bc54-140c-4091-8700-52bf17317035": "zoom_get_user_contact",
  "0408cd34-20ea-4581-b133-aead61b5c087": "zoom_get_channels",
  "a8c0ff33-4d66-4db2-97d1-b42456f02c5b": "zoom_create_channel",
  "ac0153fb-a7e3-4fe4-9149-4b1606f76fe5": "zoom_delete_channel",
  "30660431-17ae-42b0-8b39-ce9c813735e6": "zoom_get_channel",
  "883de544-19dc-4656-af1f-0e5c9ec3f917": "zoom_update_channel",
  "7ac8483c-781f-4689-8819-d2684faab838": "zoom_list_channel_members",
  "17e6d05f-c96a-4490-a417-9831ba14f12e": "zoom_invite_channel_members",
  "943e82ca-9dd8-4914-8497-b7c0f579e236": "zoom_remove_achannel_member",
  "5f653cc7-42bc-41fe-b34e-524ae103a5f9": "zoom_get_chat_messages",
  "e38253fc-83ea-44ae-bbb5-16cd569fdd59": "zoom_senda_chat_message",
  "d76bf65a-3c4c-456d-95f8-fd1a5df83ca8": "zoom_delete_chat_message",
  "f54e6a44-9cee-49b8-9179-7a52cb43ccb7": "zoom_edit_message",
  "60091a4a-83ce-49cc-ad91-e26a6bacf14e": "zoom_search_company_contacts",
  "2fe7c6dc-1d9e-4212-98fa-5f3fb7b4779e": "zoom_groups",
  "733128c1-50c4-4a5d-b74e-43a49929532d": "zoom_group_create",
  "dc72a8a0-6fd2-4c50-82cb-e2a5763b0af8": "zoom_group_delete",
  "c45397c7-1cb3-4642-aa99-ad8d6760353d": "zoom_group",
  "8c54bebe-4490-4c62-a5ca-2745e65e4b43": "zoom_group_update",
  "8f13b49b-0c9e-4531-8cda-540e92f46fbc": "zoom_get_group_lock_settings",
  "6c33f3f1-82ac-42f9-8948-34542c7992ef": "zoom_group_locked_settings",
  "ec492127-2917-4972-a6c7-3ec785ff7c69": "zoom_group_members",
  "101c84ad-507c-4115-9ac2-7ccad3aa42fe": "zoom_group_members_create",
  "47f08b6e-636d-412c-b460-27b3e7ce1a10": "zoom_group_members_delete",
  "1132d490-7bb3-498c-96df-c925edc370b8": "zoom_update_agroup_member",
  "d067a443-b4b0-4ae5-9a46-67e257a08a1a": "zoom_get_group_settings",
  "a64c872a-8865-4c99-b7c7-84a40a91901c": "zoom_update_group_settings",
  "ec9086d7-8457-4677-9df5-6e959a58f4eb": "zoom_del_group_vb",
  "e99ff3ef-d7e5-4ebf-a06e-7c530bcadd41": "zoom_upload_group_vb",
  "9ec78ff0-5c1d-482d-b2cc-fcebef74849a": "zoom_device_list",
  "03e23ff5-7981-4ef2-a501-65d26fe26094": "zoom_device_create",
  "0acc3257-bddd-468f-84a8-a53617982df8": "zoom_device_delete",
  "9a993e24-e70c-463e-b8e3-a01ab9f92cf0": "zoom_device_update",
  "92d3837e-3082-4251-8e5b-856e1c26d7ca": "zoom_sendchatbot",
  "eaf1971c-2dc3-42c3-a5c3-d724125d6703": "zoom_delete_achatbot_message",
  "98c5f9a4-e743-48a2-bffc-50554318d17e": "zoom_edit_chatbot_message",
  "7347eed8-ddac-4123-8ddf-4a877271145f": "zoom_im_chat_sessions",
  "93e31ebf-70c6-42c0-80bc-0457c04c6670": "zoom_im_chat_messages",
  "059f4e77-7334-4a04-96ea-b7c7989fe421": "zoom_im_groups",
  "9aa05e2a-1877-4999-8309-2e1a609a5348": "zoom_im_group_create",
  "711e0cb7-bd75-4401-b7f3-d49f6cf75c7b": "zoom_im_group_delete",
  "024f30a3-a47c-4542-b165-02f908369a25": "zoom_im_group",
  "4eded50c-0539-4d00-96cb-4eaa9509092c": "zoom_im_group_update",
  "f90766cb-6a93-4be2-9e49-035b0939530e": "zoom_im_group_members",
  "7fc4ee3c-a3e1-4c9b-a4ca-e1cb7cab6ef2": "zoom_im_group_members_create",
  "ccb1ec47-f17d-4e8f-8a9c-fefc3bce309b": "zoom_im_group_members_delete",
  "5d832853-d0f1-4268-a072-84f1bafa1aff": "zoom_sendimmessages",
  "34d431b2-0f66-4cf2-b864-fc41ac733a43": "zoom_listimmessages",
  "6e989114-34e9-42aa-819f-3a21afd0e31b": "zoom_in_meeting_recording_control",
  "3f1e3964-f537-4304-ac68-e90ce3638083": "zoom_meeting_delete",
  "c0db67f3-5532-42f8-b667-565ed65d7852": "zoom_meeting",
  "5f71ffb7-bdb0-47dd-b87b-6bd8204b7e77": "zoom_meeting_update",
  "efbeb2dc-45c9-4358-8ce6-19d045a4638d": "zoom_create_batch_polls",
  "7ad97667-2e1d-48f8-89b4-d23b81200bf5": "zoom_meeting_invitation",
  "f42364b3-e8dd-4743-8c58-285722fe2060": "zoom_get_live_stream_details",
  "f2bcfcef-7041-444f-8388-46b79f92e71a": "zoom_meeting_live_stream_update",
  "8fac005f-8388-4a51-bead-d80d68204f0b": "zoom_meeting_live_stream_status_update",
  "dab65e5f-a84e-4314-bc38-b9691d1a117b": "zoom_meeting_polls",
  "b6b446dc-91bc-474d-8787-b77ce67157b1": "zoom_meeting_poll_create",
  "7786ad04-1873-40d3-9612-d35f515a77aa": "zoom_meeting_poll_delete",
  "fbc37865-13b8-493b-b089-d5f2b0ee7653": "zoom_meeting_poll_get",
  "047e8d5f-62bf-4a38-a0ef-fb22dd3639e8": "zoom_meeting_poll_update",
  "f17feb1f-5efc-473e-849e-863bf31fcc83": "zoom_recording_delete",
  "d819aceb-5008-4c87-939a-29008746a670": "zoom_recording_get",
  "5ffdbb5c-4055-4b82-ade5-0a5f54b250bf": "zoom_meeting_recording_registrants",
  "72e82a4e-ecc7-4384-bc29-a386bbdc03eb": "zoom_meeting_recording_registrant_create",
  "61f07669-bca1-4a5f-945b-306ce760e8cf": "zoom_recording_registrants_questions_get",
  "6959bc32-376a-446c-91dc-ab171f9ae435": "zoom_recording_registrant_question_update",
  "99f154c5-0b89-4a4a-81a4-6cfc096dd100": "zoom_meeting_recording_registrant_status",
  "1dc1e61d-8116-401a-9c3a-da2f582c06dd": "zoom_recording_setting_update",
  "981e7684-175c-486f-a785-6f9790569645": "zoom_recording_settings_update",
  "ea4a28e3-daf4-4516-9542-650df3348130": "zoom_recording_status_update",
  "98951cc8-db1f-492a-933a-e626f3753aca": "zoom_recording_delete_one",
  "b593b6d4-6189-4514-b8c6-460521d5514b": "zoom_recording_status_update_one",
  "12e27afa-204c-4c7a-9dd2-7426dd6abda2": "zoom_meeting_registrants",
  "dd62f8a5-484f-408f-92aa-747215cb28ad": "zoom_meeting_registrant_create",
  "046a56ca-b2fd-410b-b4a8-1dadf7442d18": "zoom_meeting_registrants_questions_get",
  "a678412c-be86-4fbb-9f4b-1790457311df": "zoom_meeting_registrant_question_update",
  "21b98f93-5560-497a-9ebb-77960146b43c": "zoom_meeting_registrant_status",
  "46b198ef-3377-408f-98f9-7d04e8ffe6cb": "zoom_meetingregistrantdelete",
  "5bbb9b3d-9e88-47e3-a26a-6c5fba571ac8": "zoom_meeting_status",
  "d4ed5447-47a0-4635-af03-619e2ed0a120": "zoom_dashboard_client_feedback",
  "e6792cbd-8a21-4120-9d73-6c5ab0a60bf8": "zoom_dashboard_client_feedback_detail",
  "07efd72d-a52e-4882-b172-f78effa48e99": "zoom_list_meeting_satisfaction",
  "56d059f9-e68b-40d5-82f4-7e7705f86686": "zoom_dashboard_crc",
  "790fa2d4-5099-4496-8ab4-3df85a85eeb6": "zoom_dashboard_im",
  "4d168f2f-039f-4426-b9b8-691f01d16359": "zoom_dashboard_issue_zoom_room",
  "43b55fc6-e3a3-4dd8-8ccc-e79542c0b3f6": "zoom_dashboard_issue_detail_zoom_room",
  "677350e3-0bad-4244-8a2b-d3d53c958318": "zoom_dashboard_meetings",
  "c471c966-489d-4194-9bb2-074bc552dc0c": "zoom_dashboard_meeting_detail",
  "0320800b-67e3-4d04-9eae-e4c3d2b57a13": "zoom_dashboard_meeting_participants",
  "61d2d8ee-88a8-4c5a-839f-dd3574459c54": "zoom_dashboard_meeting_participants_qos",
  "c05f2685-4e83-421c-a964-362099ffa570": "zoom_participant_feedback",
  "ea6dfde7-ce76-41b1-bf06-b554fcfd4452": "zoom_dashboard_meeting_participant_share",
  "ef613773-a8b9-4b38-b85f-869890cd5390": "zoom_dashboard_meeting_participant_qos",
  "6c47e024-5400-48a6-8053-f7adc1ce02c7": "zoom_dashboard_webinars",
  "d85fb978-835e-4db1-ae07-9a2397aeffa9": "zoom_dashboard_webinar_detail",
  "6b81e8f3-28a1-437a-b620-598627bf5b7d": "zoom_dashboard_webinar_participants",
  "f0ef5f24-982a-4299-87cd-2a054e0e09db": "zoom_dashboard_webinar_participants_qos",
  "8cc3e117-2976-4e47-9252-4c0f10f3c5f2": "zoom_participant_webinar_feedback",
  "5dfc8579-899a-44ec-847c-53fca2b107c0": "zoom_dashboard_webinar_participant_share",
  "1833c35a-374f-4698-851f-f4cd7caf308d": "zoom_dashboard_webinar_participant_qos",
  "cc8f15d2-b8a1-4af6-8cce-5b94ba79673f": "zoom_dashboard_zoom_rooms",
  "cad719aa-9006-4a9d-abef-e2a52f1eef25": "zoom_dashboard_zoom_room_issue",
  "d61c9048-b682-460a-9a35-0e211e99a1d1": "zoom_dashboard_zoom_room",
  "0d37c73f-de2c-4a08-afb8-1624c83c4d4d": "zoom_list_past_meeting_files",
  "24660866-16e6-4bd4-b837-6b5f3cf945c8": "zoom_past_meetings",
  "744d4a8f-a4b4-4fb3-ab58-d667f9b4a28c": "zoom_list_past_meeting_polls",
  "0ba98a83-ca1b-4adc-88e3-b907a5bc2522": "zoom_past_meeting_details",
  "298c2da0-e287-4d5e-a68f-44b661960226": "zoom_past_meeting_participants",
  "92e6f89d-6567-4c5d-b843-1c08a27686b3": "zoom_webinar_absentees",
  "a2166075-3098-453e-8281-df26010c471b": "zoom_list_past_webinar_files",
  "2e1db055-31b0-4631-b96a-fc2d17288049": "zoom_past_webinars",
  "987c85f5-219d-4393-a963-187f6bed427d": "zoom_list_webinar_participants",
  "747b1f59-aeaa-4dd5-aad0-67c5bd81d2e2": "zoom_list_past_webinar_poll_results",
  "f6926aac-c5a7-4dab-b97b-88a311058f60": "zoom_list_past_webinar_qa",
  "27407441-f7fc-4b98-800e-29ba012c709b": "zoom_add_auto_receptionist",
  "74e87381-7d85-47c6-8f30-adbc50e18643": "zoom_update_auto_receptionist",
  "92093889-c272-4421-b9bc-a4492c39b8bc": "zoom_unassign_all_phone_nums_auto_receptionist",
  "30a1513e-3a96-49e0-8c83-ef9f859c5c24": "zoom_assign_phone_numbers_auto_receptionist",
  "57bed949-6a0b-40ac-a49f-11a7cf6db057": "zoom_unassign_aphone_num_auto_receptionist",
  "18396e42-55c6-445e-b8ef-29602294ec26": "zoom_list_blocked_list",
  "61cb0001-8242-4881-939b-6bcbb11d503f": "zoom_add_anumber_to_blocked_list",
  "b9b080b3-a266-40e2-b353-2cb6123cc95f": "zoom_delete_ablocked_list",
  "c6c146f4-7b2f-47ce-b860-5c2d752297c9": "zoom_get_ablocked_list",
  "50fdd1d8-fc89-439f-a0b3-aaf551eee1e1": "zoom_update_blocked_list",
  "859c12ce-bae8-44f0-9b16-a20c0783fa29": "zoom_add_byocnumber",
  "54417a4c-df9a-4da3-83f9-18ce2ecaf37a": "zoom_account_call_logs",
  "0eae8114-4fde-4124-a655-bc7838e0b6ef": "zoom_list_call_queues",
  "8718802f-da0b-4ec6-921c-8f6cebf8f1a5": "zoom_create_call_queue",
  "441c966b-8763-4dc9-8bca-8c82ef837f6d": "zoom_delete_acall_queue",
  "a98aadf3-07e1-4cc3-82c5-ccf3c9749d91": "zoom_get_acall_queue",
  "64ea0ac0-8bb5-4146-abaa-a530f33ed838": "zoom_update_call_queue",
  "66baa2b5-c21e-43a7-8afe-d04057ea539b": "zoom_change_call_queue_manager",
  "84d209c6-3dec-40a0-9266-dda90cebf2e3": "zoom_unassign_all_members",
  "7fd09976-885f-4b2d-9775-a3639de45e74": "zoom_add_members_to_call_queue",
  "e4a7ecd5-88de-4de5-8994-b69d725dcf20": "zoom_unassign_member_from_call_queue",
  "80d59db8-03f2-45ab-bc9f-8e5ed3d401e8": "zoom_unassign_aphone_num_call_queue",
  "1e809099-586a-42b1-b388-95f49acbfe36": "zoom_assign_phone_to_call_queue",
  "d2bce754-37c5-4c1a-8c0b-20525ceadd4d": "zoom_un_assign_phone_num_call_queue",
  "5ef8b09e-3f62-4744-bcf6-20c8004ad401": "zoom_get_call_queue_recordings",
  "c889659b-882a-4ba8-b857-5b2752c27909": "zoom_list_calling_plans",
  "1d5bf878-fd35-4233-a92e-618fe0836b7c": "zoom_list_common_area_phones",
  "73acc3ac-b3b7-4f0f-8619-63ce4592ebf9": "zoom_add_common_area_phone",
  "2396345d-389e-45ec-85f1-e4bfb6406584": "zoom_delete_common_area_phone",
  "6f71e1af-eb80-481b-a783-e6fb76211726": "zoom_get_acommon_area_phone",
  "e9728179-3bd6-46f1-b410-b838c7b10603": "zoom_update_common_area_phone",
  "5dbb1019-04a3-47df-86fb-c8a52338dabe": "zoom_change_main_company_number",
  "bee0cff8-3c1c-478a-b7c6-59a6fe7e821a": "zoom_list_phone_devices",
  "5cd19496-f8a4-4a21-bffb-269a3f6cc840": "zoom_add_phone_device",
  "13916b5e-ceb5-46e7-9464-27169ab59e1d": "zoom_delete_adevice",
  "cdda10a4-f9b3-4781-bcb4-234ba3a354c6": "zoom_get_adevice",
  "8210dbb9-3ee2-4411-adb1-f4ab0c12d6b4": "zoom_update_adevice",
  "cd759185-6561-4439-ae0a-d5a0051879de": "zoom_list_call_logs_metrics",
  "7b81809c-c83e-41d9-87cc-58ffb407da6d": "zoom_get_call_qo_s",
  "eb107d46-6fc8-4720-9ba5-42e1a5dfe7e1": "zoom_get_call_log_metrics_details",
  "d2975798-767d-402e-838b-8ee261250977": "zoom_list_account_phone_numbers",
  "6d0a947f-a6d8-4c37-96e1-4c8e7a2c98a8": "zoom_get_phone_number_details",
  "58d0b9bb-ea1c-417e-af7f-708205721ffd": "zoom_update_phone_number_details",
  "32014f6e-2e38-4fcc-80b8-277012a21a40": "zoom_get_phone_recordings",
  "a595db7d-610a-4307-bbe4-678116a79968": "zoom_get_psoperation_logs",
  "0defa4fb-930b-480c-a6cc-d8c23c71d22e": "zoom_list_setting_templates",
  "4a691978-b399-4070-b58f-b3052ecadde2": "zoom_add_setting_template",
  "6a84c054-4df6-4715-84d0-3e28717edded": "zoom_get_setting_template",
  "32c66331-5f12-4158-ba3a-e3581513f384": "zoom_update_setting_template",
  "e8e1b92c-53f8-44e6-b90c-fc8d8de2fc8d": "zoom_list_shared_line_groups",
  "709dfd16-950b-441e-ad78-23132cc8d6b7": "zoom_create_ashared_line_group",
  "eead0eed-9160-47b9-917a-62e26114e82a": "zoom_delete_ashared_line_group",
  "6384a284-3d8c-45bc-851e-c5cfcace6b27": "zoom_get_ashared_line_group",
  "1ac1fb39-e9a7-4fa7-a106-6b8cc129ffe8": "zoom_update_ashared_line_group",
  "2b382ea0-ae5f-49c2-b979-e4c5cc3f7bde": "zoom_delete_members_of_slg",
  "58273a39-1983-4dcb-b9bd-2bd373cd8662": "zoom_add_members_to_shared_line_group",
  "fb7ca455-67fb-4057-91ae-6f5a36cef2e2": "zoom_delete_amember_slg",
  "44f3755e-7680-4874-8a29-078bc8db4f8b": "zoom_assign_phone_numbers_slg",
  "b4d9607e-3056-4122-84e0-3cf5f63b3d43": "zoom_delete_aphone_number_slg",
  "6e46a726-af4d-484f-b1c1-d9a3cc1d253d": "zoom_list_byocsiptrunk",
  "d05474fb-05c0-4551-bebf-a909f0e4c50e": "zoom_list_phone_sites",
  "3ee11d90-ba6d-4640-b569-ede35d983cf5": "zoom_create_phone_site",
  "181bce04-4471-4861-b560-8988bb22b81a": "zoom_delete_phone_site",
  "f19e9c8f-b59e-4781-bfe6-582603ce91d7": "zoom_get_asite",
  "e1a8e406-b3a9-43fd-a48e-8be4d4760d37": "zoom_update_site_details",
  "6cd83650-12fe-40b5-b238-f14e85d316aa": "zoom_list_phone_users",
  "0ec5674d-0bd9-4a49-b220-3466110fa554": "zoom_phone_user",
  "7a22e02d-c9e2-4be6-a746-43eb4c32e657": "zoom_update_user_profile",
  "0381d878-d9ac-49bf-bbfa-403f52b25fb2": "zoom_phone_user_call_logs",
  "c6b4aeb0-e10e-4f04-8a36-8dd7f77a2b36": "zoom_delete_call_log",
  "db32ab5e-b87a-407e-ba3f-c9b77eba87c4": "zoom_assign_calling_plan",
  "8a1ca86c-53ca-4603-a0bd-7431314a0f12": "zoom_unassign_calling_plan",
  "7da95968-a24d-46ff-a706-6242dea2b6e2": "zoom_assign_phone_number",
  "8949092c-cd86-4ccf-982d-b2607907c066": "zoom_unassign_phone_number",
  "9d6d2b89-b472-4859-a0d8-b5a2e504ef3c": "zoom_phone_user_recordings",
  "1d543183-3b2e-483c-8a8a-ccb2eeb6ef6f": "zoom_phone_user_settings",
  "d6166815-30f5-437c-82f6-0e8557c209ba": "zoom_delete_user_setting",
  "ffa49027-0aeb-45d2-9f0f-1f43ef287cdd": "zoom_update_user_setting",
  "90d396fc-ebfb-47a9-af42-1807f725cf58": "zoom_add_user_setting",
  "9aae8bb0-7f4c-4d76-879a-4e912d5c10c9": "zoom_phone_user_voice_mails",
  "3d69c029-19fd-4dd3-b633-362298d9ae6e": "zoom_delete_voicemail",
  "e84c90a5-eb11-47e1-aa09-358757c1b604": "zoom_report_sign_in_sign_out_activities",
  "ed52787b-e7b9-41ff-8b92-264aff751772": "zoom_report_cloud_recording",
  "9eb1069b-fd30-4b34-b3e4-86ddf39e2947": "zoom_report_daily",
  "2932bc4c-eb16-4429-bd8a-b0e5ec9210cd": "zoom_report_meeting_details",
  "b5e21b0b-0de1-4854-b466-581ee5e69aa9": "zoom_report_meeting_participants",
  "55003b75-d61b-4564-8b47-b4464c8f5cf5": "zoom_report_meeting_polls",
  "8edeb43c-ceb3-42a3-bc19-ed8b1a902e2c": "zoom_report_operation_logs",
  "aa785c23-5f21-44ba-b4f1-e05c1ca98356": "zoom_report_telephone",
  "231fb5e9-440d-4793-80e8-0ab14bc33c72": "zoom_report_users",
  "d7a8c150-5ed0-4847-abf5-90ef3e96cdfe": "zoom_report_meetings",
  "7a8c73f5-b175-4609-8ff4-bfeb27fd4856": "zoom_report_webinar_details",
  "ea45080b-1e76-4b86-ab6c-6635a4999f05": "zoom_report_webinar_participants",
  "89005a01-41b3-440c-ae9c-3ec24066c13d": "zoom_report_webinar_polls",
  "4d41101c-7d99-4e4e-bd19-6d5d5603410c": "zoom_report_webinar_qa",
  "27eef70d-9545-4440-9a6f-3ed9bec277e1": "zoom_roles",
  "49c96920-c659-429c-8730-1be6f487cc52": "zoom_create_role",
  "eabdd02e-5866-4d53-a0ff-3ef64850ad41": "zoom_delete_role",
  "58e0923b-d88b-48f2-a091-eeecee9ece2d": "zoom_get_role_information",
  "147cadd1-f695-4dad-9fd9-cac4b33689b1": "zoom_update_role",
  "5050551a-c50f-44f4-bf26-ff8880814abf": "zoom_role_members",
  "fdb0eba4-98c0-4606-ae02-e519ef0679a7": "zoom_add_role_members",
  "392698c1-77f8-40f6-9ffb-4ecfb39a78e9": "zoom_role_member_delete",
  "eca00daa-e4c8-4ef4-b7db-e37540441fed": "zoom_list_zoom_rooms",
  "43458ec7-da52-4b05-bb04-e7b3002503e1": "zoom_add_aroom",
  "332f43e3-ec57-4b99-a9dd-6a5c2d001072": "zoom_get_zraccount_profile",
  "3d49f21c-2652-4b74-a4e4-e3ae6f93c5f1": "zoom_update_zracc_profile",
  "e5dad23e-fac7-4d04-a72b-0d7660e0fbb3": "zoom_get_zraccount_settings",
  "86dee441-abbb-4798-8c5d-3f3be5869e25": "zoom_update_zoom_room_acc_settings",
  "ea75bf21-aeef-414b-9a82-637e2abf1dbe": "zoom_list_digital_signage_content",
  "4edb3330-0d2e-44ff-903b-741e606fe3c2": "zoom_manage_e911signage",
  "13b296e1-f998-4396-96dc-25c139e83953": "zoom_list_zrlocations",
  "8faf4e5c-8dd2-4577-a6f1-1843d5e528de": "zoom_add_azrlocation",
  "2767b866-91cd-444e-94cf-e3419f21fc2c": "zoom_get_zrlocation_structure",
  "92d7638c-6518-4f79-af64-15344fe74060": "zoom_update_zoom_rooms_location_structure",
  "537849c1-5686-4a13-87e5-8ecbb1c9f95d": "zoom_get_zrlocation_profile",
  "47f022b7-04f4-478b-b413-bc7bebf451b5": "zoom_update_zrlocation_profile",
  "6280ae98-de65-491c-a3b1-46bcbce8d575": "zoom_change_parent_location",
  "07168367-7b0f-41b4-8d1e-62c691bacae3": "zoom_get_zrlocation_settings",
  "3f4d7305-007f-4eae-947d-8f26ef854bca": "zoom_update_zrlocation_settings",
  "223fb05a-21ed-4620-8606-5322c74970dd": "zoom_check_in_rooms",
  "1c46701a-a6a9-4452-8fd4-2e511a8e5110": "zoom_delete_azoom_room",
  "4d3b7cb1-a6d1-451f-8d91-e51a89f4e249": "zoom_get_zrprofile",
  "93dda470-8cef-4b2b-ac21-507b4ab30279": "zoom_update_room_profile",
  "bc38cd79-9a1d-4805-a711-ed5a72d59eb7": "zoom_list_zrdevices",
  "9ed60cb2-11ff-4627-9a2d-7a3ed9a33cd8": "zoom_change_zoom_rooms_app_version",
  "5885d52e-7426-409a-9b61-bb236df58f98": "zoom_change_zrlocation",
  "df9e3772-2b31-4580-bdd2-e6bc638cb5ad": "zoom_get_zrsettings",
  "82e2a22a-75aa-4c74-96ce-f14964a33d34": "zoom_update_zrsettings",
  "81b8f862-bdbd-47f2-9b9a-8b69b6cad1c5": "zoom_list_sip_phones",
  "31869977-099f-44c3-9dc2-8eedf07aa4bf": "zoom_create_sipphone",
  "94538b31-eea0-4cf4-b56f-6b859e159983": "zoom_delete_sipphone",
  "0e6e73e6-1894-44d0-98fb-47d5c2fe890e": "zoom_update_sipphone",
  "2493fd76-9ec4-41da-8ac3-82e1e6b55ab3": "zoom_list_sip_trunk_numbers",
  "d11c19fa-ccb7-437e-be25-a04c17be65d3": "zoom_trackingfield_list",
  "ba6686cf-414d-43fe-a68c-5ad659621308": "zoom_trackingfield_create",
  "a80f473f-3b99-4e67-8fe8-ecc9e050bac3": "zoom_trackingfield_delete",
  "d49e6080-81c6-484e-a5d5-2adf7eb5a2f3": "zoom_trackingfield_get",
  "f11df85d-c0d9-4727-b2ac-a91479633e03": "zoom_trackingfield_update",
  "79925ca8-a17f-4099-a549-91172dd4bdd4": "zoom_tsp",
  "a5587f8c-e111-45e0-a824-a2278cccc134": "zoom_tsp_update",
  "567e5419-d0dd-4d7d-846d-23e5353c711e": "zoom_users",
  "d411fb68-90bf-454c-a0a8-6dbb0c58d3bb": "zoom_user_create",
  "fd2ec2c9-8fe8-44c2-9b9a-f900faea1bfe": "zoom_user_email",
  "fcc23030-ba01-49ff-a2c5-28f263bc2c86": "zoom_user_zak",
  "27ad4fd7-334f-4c0d-bd72-102b6f6dad5e": "zoom_user_vanity_name",
  "dd93f077-4a90-4d0c-acf6-21c4eab05b52": "zoom_user_delete",
  "6fe97037-b202-4aeb-8000-1fb64beff955": "zoom_user",
  "b90a4091-62dd-461d-b2e8-dd4822a2f7b8": "zoom_user_update",
  "95a71177-d5f3-414f-9881-d3d06ccc46d0": "zoom_user_assistants_delete",
  "d37b732e-c5e3-4b0c-8c64-08e941440e62": "zoom_user_assistants",
  "156f43dc-1bf4-4f96-9fae-cac65df8162a": "zoom_user_assistant_create",
  "41884502-f6f1-4bd9-aa29-a52dcb352fe9": "zoom_user_assistant_delete",
  "a23fd82f-d98b-4b32-8da8-dca56d92696e": "zoom_user_email_update",
  "19561718-dae2-4d87-9979-8686611d316e": "zoom_list_meeting_templates",
  "3c78c1f8-7b89-4d71-bda0-9a2c268a0750": "zoom_meetings",
  "9e9c4957-99cc-496b-864f-072b0a41e5eb": "zoom_meeting_create",
  "db360a46-279d-4a0f-8082-6ce76dc114e3": "zoom_user_pacs",
  "9c02ab22-b533-41f9-908b-6e5e948d3a8e": "zoom_user_password",
  "722b2741-eb32-4f5d-9503-3889cb7ebcc7": "zoom_user_permission",
  "8f4c4e97-8b8c-453a-b6ff-6401e913ee81": "zoom_user_picture",
  "2a6a5ecf-b47b-4dc3-a03d-725dd11eaf12": "zoom_update_presence_status",
  "e1cdfc94-e370-45b6-8113-d4d9d9b5b90e": "zoom_recordings_list",
  "c6988496-0661-4311-8544-ea979ece0d96": "zoom_user_schedulers_delete",
  "7680025b-36f7-4ff5-bef0-b42c98ac7f8a": "zoom_user_schedulers",
  "a2ccbfd1-7a81-41ef-b71c-2b0c95b3037e": "zoom_user_scheduler_delete",
  "400564e3-8447-41b4-b87c-8df0afd5beaa": "zoom_user_settings",
  "5e551a1a-5fcc-4ce5-aecc-4126e5f6674f": "zoom_user_settings_update",
  "2969a972-a082-407c-adfb-79314145d75d": "zoom_del_user_vb",
  "97a7e9fd-df09-4ae0-9cf7-d1123b26fff6": "zoom_upload_vbuser",
  "5db1ffdf-a6e2-4a4b-a8d0-3eea299f899e": "zoom_user_status",
  "e64e8cea-c47a-4939-8698-0303521ca5a2": "zoom_user_ssotoken_delete",
  "f8032cc8-742c-4865-8f9a-22b7ea7634f3": "zoom_user_token",
  "3a1221fd-1315-4b8a-8223-11744d41128c": "zoom_user_tsps",
  "4d078036-274c-4849-86a2-d284436bc6cd": "zoom_user_tspcreate",
  "416147b3-f6a7-4428-b4e6-7f996f0eab03": "zoom_tsp_url_update",
  "d06ea9ea-8698-4c67-b5f8-b38ed51372b7": "zoom_user_tspdelete",
  "b42dd5f1-6704-4c42-ad2b-15bc6c17d7d0": "zoom_user_tsp",
  "5fca1b20-40d4-4d3a-a73f-53bdb105e394": "zoom_user_tspupdate",
  "ac906faa-bd67-4de1-8aa6-ee0cba980ce5": "zoom_list_webinar_templates",
  "55471697-ac59-4611-8306-01162dacb10b": "zoom_webinars",
  "cbfe9e07-0b1a-4999-954d-a46c880a9355": "zoom_webinar_create",
  "b809546a-1388-42d3-bb5c-fab6b4c72847": "zoom_webinar_delete",
  "b8f37f80-1ce0-435d-b41c-1aaf2024cd1e": "zoom_webinar",
  "32fe971a-34df-44ea-bc42-b5ae72d5f447": "zoom_webinar_update",
  "75c55e73-d10a-4836-acba-9e45b3951309": "zoom_add_batch_webinar_registrants",
  "dd4ce7c1-6b5f-4409-8208-a504e725a87a": "zoom_webinar_panelists_delete",
  "3ea5f2cb-bdc7-4d25-92c5-9bfaf4898f80": "zoom_webinar_panelists",
  "179e1f18-698e-4d2d-871b-a02498b5bc4e": "zoom_webinar_panelist_create",
  "7985e934-56c7-44a1-befc-35215e878203": "zoom_webinar_panelist_delete",
  "d84e932a-08f0-4282-8c0e-18521739ded2": "zoom_webinar_polls",
  "d4462c5d-dad1-48d0-83ff-3a4004bce024": "zoom_webinar_poll_create",
  "0894cb55-86ab-4b9b-9fba-0ca08ce7da07": "zoom_webinar_poll_delete",
  "e5d69349-f504-4d55-984b-60a20c321768": "zoom_webinar_poll_get",
  "46b3d17c-f8b0-4366-a342-ddf85e112422": "zoom_webinar_poll_update",
  "8e3239b1-a792-4449-9734-e9fa69f2c967": "zoom_webinar_registrants",
  "779e7e49-fad5-48d2-87a1-eb4bd563bc19": "zoom_webinar_registrant_create",
  "f624805b-bae6-41c5-8765-c7ff4cd9ee7b": "zoom_webinar_registrants_questions_get",
  "66107c0e-5fc4-4dc4-8d9f-7d121d2ae1db": "zoom_webinar_registrant_question_update",
  "8113dcfb-4202-4858-8207-0dd5bef87863": "zoom_webinar_registrant_status",
  "93bb2288-4bfb-4247-97cb-1f4593aeb69c": "zoom_delete_webinar_registrant",
  "8244dc34-d210-412f-8e84-cd577d43e752": "zoom_webinar_registrant_get",
  "acca6897-d5b6-4d27-8ed9-d27a0e4cc798": "zoom_webinar_status",
  "f414ae85-5369-40d8-89cf-7f02459dd4ba": "zoom_get_tracking_sources",
  "664ffa48-80ba-4eaf-8798-8953aeec2708": "brave_search_get_web_search_results",
  "713c5cd5-edef-42be-b2d1-e70b6c2d056c": "gmail_users_drafts_list",
  "d87694b5-fa5a-4c51-95f9-de371e522b8b": "gmail_users_drafts_create",
  "1b9b9323-bb98-4838-97aa-5591818b1566": "gmail_users_drafts_send",
  "039d47a4-2246-42d1-ac9f-e0c81be66289": "gmail_users_drafts_delete",
  "e9896feb-0037-4bac-9bbd-b92d7afc8e95": "gmail_users_drafts_get",
  "bc5b022e-5c23-4fdc-8853-06f5947372a7": "gmail_users_drafts_update",
  "76a81f4d-8de7-4d92-a798-ad293e8daeee": "gmail_users_history_list",
  "d74b5bd0-d674-4f9b-a091-eabf44a3bc8c": "gmail_users_labels_list",
  "4e823c42-2b89-41b8-ba9a-e9d4920b9544": "gmail_users_labels_create",
  "18352eb9-a826-499d-b0c2-891198e0ad88": "gmail_users_labels_delete",
  "ff22ca9b-624c-4d1c-9980-47e5eb5c2fce": "gmail_users_labels_get",
  "eb15b5b7-2b75-4cc8-9591-d216f56c5f43": "gmail_users_labels_patch",
  "7348de2c-bf2f-45b9-84fe-902a82f7e96e": "gmail_users_labels_update",
  "cddc2eeb-5c6f-459b-a550-18f0bd432068": "gmail_users_messages_list",
  "11505bcb-2f0a-40c7-b0be-46619ed9f7ef": "gmail_users_messages_insert",
  "bad5232b-43ee-470d-9730-2216d563c36d": "gmail_users_messages_batch_delete",
  "3519c37a-4fed-43d8-83b9-1095fa2dfb18": "gmail_users_messages_batch_modify",
  "ae6b9ac0-b93c-401a-9191-429b002a0b41": "gmail_users_messages_import",
  "b6f57ab4-1f59-47d8-b0e6-e42c2451d96f": "gmail_users_messages_send",
  "e6fd6614-55d5-4ee3-bcb4-5bf57694a5b5": "gmail_users_messages_delete",
  "8d044926-45bc-4c2b-bb9a-26a580290824": "gmail_users_messages_get",
  "287a55a6-9f59-40c6-8b6b-733e3ccb184e": "gmail_users_messages_modify",
  "a12fbd23-ba53-4550-b6f0-4fb191717e5f": "gmail_users_messages_trash",
  "bf186bff-e4a6-41b4-a89c-541055d44b64": "gmail_users_messages_untrash",
  "08b872de-4bd0-4da8-bc11-e594567cd960": "gmail_users_messages_attachments_get",
  "b006fb08-a1ad-4bf3-89cb-25df5579fad2": "gmail_users_get_profile",
  "ad9d54b8-1617-4d2a-9eea-c9b54c1d0939": "gmail_users_settings_get_auto_forwarding",
  "d39bbf90-a79e-42ae-88d4-9f0d9ef6cc13": "gmail_users_settings_update_auto_forwarding",
  "7774aa89-c638-4efc-a5e9-ce6172337d43": "gmail_users_settings_cse_identities_list",
  "9772c847-bc72-4114-a4e0-3b0db02bdfcb": "gmail_users_settings_cse_identities_create",
  "da4862a9-6422-4ea3-a11e-3aa9ca92f1eb": "gmail_users_settings_cse_identities_delete",
  "893191f6-657e-47b5-840e-d3c3d1125b79": "gmail_users_settings_cse_identities_get",
  "5fc5ac2f-1ede-4ea9-affe-3b690dcb1923": "gmail_users_settings_cse_identities_patch",
  "eff5e249-58cb-4dc1-8413-f6ea872e9d66": "gmail_users_settings_cse_keypairs_list",
  "d99db2d5-e55e-482e-b42e-9a7d8584c56a": "gmail_users_settings_cse_keypairs_create",
  "30b4254e-b823-49a4-addc-1e590899440c": "gmail_users_settings_cse_keypairs_get",
  "b0d6fa80-9f25-460a-9100-cea4cd1cf02b": "gmail_users_settings_cse_keypairs_disable",
  "c96007ce-a37a-4067-8d49-108f897469d5": "gmail_users_settings_cse_keypairs_enable",
  "97490061-bdb1-4f21-acb5-612f2f0a6290": "gmail_users_settings_cse_keypairs_obliterate",
  "3e311642-fea9-4252-9270-17cd44004759": "gmail_users_settings_delegates_list",
  "d86edd9d-5102-49ff-8b8b-262a12ef85d3": "gmail_users_settings_delegates_create",
  "7e0e024f-4aa6-4450-8f89-045653e5b181": "gmail_users_settings_delegates_delete",
  "3170ed96-ec7b-43ca-806c-4716d9c43d94": "gmail_users_settings_delegates_get",
  "9f2bace1-e0aa-46c3-b9ac-badef9ba981f": "gmail_users_settings_filters_list",
  "ca28f74e-30fa-4fd5-af70-9be808580ddb": "gmail_users_settings_filters_create",
  "cda9bba8-a1e7-4d51-9d2d-c5b2b63ca436": "gmail_users_settings_filters_delete",
  "9d112070-3a34-4073-9b63-f93ae61bbc59": "gmail_users_settings_filters_get",
  "71678aad-c1c4-4262-97e4-dacf016ee38f": "gmail_users_settings_forwarding_addresses_list",
  "077dd080-2c45-485c-9031-206a4f545c77": "gmail_users_settings_forwarding_addresses_create",
  "4d0ce1cf-1513-44d4-b03f-83848552b62b": "gmail_users_settings_forwarding_addresses_delete",
  "7f5363de-5def-472a-bc98-b01b89736976": "gmail_users_settings_forwarding_addresses_get",
  "bcccc292-e663-491f-a0c6-73902a88359e": "gmail_users_settings_get_imap",
  "9eab50ec-167d-4977-8428-1e6cf2549512": "gmail_users_settings_update_imap",
  "79a4f40a-8c2a-4407-9f0f-04f7a58a1e49": "gmail_users_settings_get_language",
  "3056b3e5-a89a-490d-b122-5c9abf0177a0": "gmail_users_settings_update_language",
  "1de11f18-fb2f-4612-abca-92172406a261": "gmail_users_settings_get_pop",
  "a94e04f8-7ad5-42e5-8a19-ce5bcfbefbb2": "gmail_users_settings_update_pop",
  "700e9ae2-6d82-4662-8aca-85684099bf74": "gmail_users_settings_send_as_list",
  "955832e3-ab21-4d6e-8e9d-18415c631b7a": "gmail_users_settings_send_as_create",
  "177dddb6-3d20-43a8-adff-d1343f085130": "gmail_users_settings_send_as_delete",
  "95ee71d9-f699-4963-bc3b-6dc2a3863a61": "gmail_users_settings_send_as_get",
  "5cd76e75-f2b2-4790-8e2a-ad64eeb98bee": "gmail_users_settings_send_as_patch",
  "e27c6a8c-624b-4051-9798-9642896c3de5": "gmail_users_settings_send_as_update",
  "562de6ea-dcdb-4a3d-b5ce-f8782cda8145": "gmail_users_settings_send_as_smime_info_list",
  "3fe81f34-ec29-4ee1-9ea9-1990887ab92d": "gmail_users_settings_send_as_smime_info_insert",
  "b80c11c2-b5bb-430a-9577-79194931ec70": "gmail_users_settings_send_as_smime_info_delete",
  "4a2d2c5f-c36b-46cb-a029-2ea98c821b1f": "gmail_users_settings_send_as_smime_info_get",
  "da5f494c-5de5-432c-a6fa-5061d355218c": "gmail_users_settings_send_as_smime_info_set_default",
  "c0a40a52-89a9-44e3-bc69-d5be74df7ef6": "gmail_users_settings_send_as_verify",
  "196ba81c-97bb-4b72-9499-0cf6953b8e12": "gmail_users_settings_get_vacation",
  "4a7304d5-59e0-496d-ae6a-77bda6c0f691": "gmail_users_settings_update_vacation",
  "5a6e0240-4dc1-48fc-8f89-87401edb6d4c": "gmail_users_stop",
  "511214f8-0897-464c-803e-2566f4d15197": "gmail_users_threads_list",
  "764ac696-304b-44bc-9b4f-0c8b29029807": "gmail_users_threads_delete",
  "7fc7f6b7-8d35-403e-8b6f-82c026d24e31": "gmail_users_threads_get",
  "713974c5-580e-4afb-9eea-dda4539a0875": "gmail_users_threads_modify",
  "75c575df-76bb-4804-80a0-4455a0182b77": "gmail_users_threads_trash",
  "b6016052-02d8-4a19-b6e0-9187529bf690": "gmail_users_threads_untrash",
  "8b7dc0ea-d946-441f-8b8c-3f33755ae4bf": "gmail_users_watch",
  "bc81d3d4-647d-4eb8-aa00-7b544a62ccc4": "stripe_get_account",
  "54b236a4-8959-4c44-b397-7ef28f0affae": "stripe_post_account_links",
  "9d2c0e32-4e05-424f-b455-560ac1a8126a": "stripe_post_account_sessions",
  "75da9b18-f2a0-46c8-bb8b-f31ae39818cd": "stripe_get_accounts",
  "3fe4a10d-1827-410f-8df8-ca6af12f96f8": "stripe_post_accounts",
  "e489f402-210d-477f-813d-903f496b591a": "stripe_delete_accounts_account",
  "f7a2515f-ed95-4ad6-a4b0-2c692e2827d8": "stripe_get_accounts_account",
  "1c25db01-3a1e-47b4-b6ce-c4aa5a4095b3": "stripe_post_accounts_account",
  "19b05605-cd67-4ec1-a257-a55c52e94c06": "stripe_post_accounts_account_bank_accounts",
  "1feb4981-3070-4c54-8475-81a0c4fbd2a5": "stripe_delete_accounts_account_bank_accounts_id",
  "5582b61b-3c22-44de-aabd-7837fcae2672": "stripe_get_accounts_account_bank_accounts_id",
  "92b5fe26-3f00-455d-a6c0-7fcee8330a14": "stripe_post_accounts_account_bank_accounts_id",
  "136e4902-941c-4f91-976b-292ed6145493": "stripe_get_accounts_account_capabilities",
  "1c7b5690-57ad-4284-970c-3a2ff8c35b98": "stripe_get_accounts_account_capabilities_capability",
  "d32d46c9-4751-4c48-b279-5042df5bd288": "stripe_post_accounts_account_capabilities_capability",
  "6e1dc5fa-07c6-4051-a3a3-7570c4e8788a": "stripe_get_accounts_account_external_accounts",
  "7f151fb4-ca97-46ad-851a-dcca49676d86": "stripe_post_accounts_account_external_accounts",
  "13f40634-4cf5-4180-9f1d-2e4a8b0d1ac3": "stripe_delete_accounts_account_external_accounts_id",
  "84b45838-d178-4a1a-9fb6-97a4b11a0445": "stripe_get_accounts_account_external_accounts_id",
  "8050b200-3381-4853-900f-80641804877f": "stripe_post_accounts_account_external_accounts_id",
  "b870b03d-97f6-46e0-9c68-814f38e366e2": "stripe_post_accounts_account_login_links",
  "897d1d7d-05ad-4315-8a73-2967a74fffc7": "stripe_get_accounts_account_people",
  "3deb84be-73a2-4b64-b2c9-6566d8cdf425": "stripe_post_accounts_account_people",
  "5f8ec176-0554-4408-bc32-b7223028eed6": "stripe_delete_accounts_account_people_person",
  "15b6b0f4-2e5d-4a5a-9dbd-6a55f5f7596a": "stripe_get_accounts_account_people_person",
  "93ce643e-9169-4944-9279-7bef6f7430c6": "stripe_post_accounts_account_people_person",
  "137934c2-3052-42fd-80ad-492af31843bc": "stripe_get_accounts_account_persons",
  "53fc68a0-7327-43b7-95cb-fdffad528243": "stripe_post_accounts_account_persons",
  "5b7f2bac-46f6-47eb-be07-03ee61a0363d": "stripe_delete_accounts_account_persons_person",
  "0ce11d95-4c44-4514-9947-1279e613a093": "stripe_get_accounts_account_persons_person",
  "2571dabc-ac12-41de-864f-3b442a052a89": "stripe_post_accounts_account_persons_person",
  "3e56659d-274a-4422-b379-5e9099894166": "stripe_post_accounts_account_reject",
  "4caf917d-b89c-4408-ac84-984081673bad": "stripe_get_apple_pay_domains",
  "19016a18-4539-46d2-81d0-03ed007049b0": "stripe_post_apple_pay_domains",
  "17995f4b-79af-43a1-aed8-cc4b49b38326": "stripe_delete_apple_pay_domains_domain",
  "176f2530-9d84-403a-b7ee-f812d2d9dc59": "stripe_get_apple_pay_domains_domain",
  "bd087299-29af-4d7c-9ead-b182ef9decea": "stripe_get_application_fees",
  "2a757567-efc4-4c8c-b9f7-f0b104f334b1": "stripe_get_application_fees_fee_refunds_id",
  "625bd4b8-ee15-4fc4-b5c8-c958c52219b7": "stripe_post_application_fees_fee_refunds_id",
  "cf594e8c-655a-4323-aac3-a32327b52e95": "stripe_get_application_fees_id",
  "c23bb53d-5f27-40cd-8b7a-a6c0545e1045": "stripe_post_application_fees_id_refund",
  "85403007-f851-4c91-91d6-487380f5bc92": "stripe_get_application_fees_id_refunds",
  "c9bb72b1-e55d-4d36-80f9-e0d8c3c6076d": "stripe_post_application_fees_id_refunds",
  "9344d33f-763b-4d14-9b19-374fe21cc173": "stripe_get_apps_secrets",
  "9924e02e-3b86-42e4-96ff-d34962df87e0": "stripe_post_apps_secrets",
  "d3a1c2b8-dc06-4142-b998-1df8b91a6c33": "stripe_post_apps_secrets_delete",
  "028db81d-0840-4d70-8e95-e71ccd270ec6": "stripe_get_apps_secrets_find",
  "dd1b8eb1-fb80-4c19-b1c2-a8523a24553e": "stripe_get_balance",
  "bb1e4216-5865-45b7-aa44-24fc70662d8c": "stripe_get_balance_history",
  "468a1a31-9096-4daf-a08c-370791cb9ba5": "stripe_get_balance_history_id",
  "02516369-5b57-485c-b5a7-bb1a653e2296": "stripe_get_balance_transactions",
  "dd065d73-6843-4f47-8f1a-512a40f0d3a3": "stripe_get_balance_transactions_id",
  "2a9e0c91-6e34-44f3-8902-b8ef9d634946": "stripe_get_billing_alerts",
  "a035dbae-3443-42f6-8b91-a141765790a6": "stripe_post_billing_alerts",
  "767f8189-43c9-4dd4-b838-2478e961917f": "stripe_get_billing_alerts_id",
  "08a3f248-03e2-4181-a8a7-a18c0a584bc9": "stripe_post_billing_alerts_id_activate",
  "cbab23cd-5815-4883-a164-f37c689ff76e": "stripe_post_billing_alerts_id_archive",
  "d653dda0-050e-4c88-8f5f-bc12312b71fa": "stripe_post_billing_alerts_id_deactivate",
  "8a9123b0-7c22-4f63-a1b5-eb23de1c2bbb": "stripe_get_billing_credit_balance_summary",
  "5a162963-2218-43a4-8c84-a7335cd3188d": "stripe_get_billing_credit_balance_transactions",
  "81b23992-146a-48c9-8491-451f2b8ee906": "stripe_get_billing_credit_balance_transactions_id",
  "60c07bda-b2d4-41b0-8989-28521ad760a8": "stripe_get_billing_credit_grants",
  "b083f527-b14b-4dde-bfdb-ec1c8bdc4c38": "stripe_post_billing_credit_grants",
  "a154ad1f-8cab-45cb-9e92-a126e28126cd": "stripe_get_billing_credit_grants_id",
  "d5a2b707-b303-46c4-94b4-7e6581a4f275": "stripe_post_billing_credit_grants_id",
  "0cded6ae-c978-49e3-8e80-b94b8f54d9d0": "stripe_post_billing_credit_grants_id_expire",
  "c0c13db7-3bd6-4a8a-bff2-8e572ac14d95": "stripe_post_billing_credit_grants_id_void",
  "9acae2f8-1fd1-4968-ae26-139e467fdf9a": "stripe_post_billing_meter_event_adjustments",
  "ac4ca6eb-df05-41bf-8a6b-1376b6cb03bf": "stripe_post_billing_meter_events",
  "2b9da966-49a7-4027-a70a-e4af94f0806c": "stripe_get_billing_meters",
  "0bfcc07c-2939-4f5f-8871-dd1cdf3f1361": "stripe_post_billing_meters",
  "254e3828-3e07-40b8-b63a-fb36df478128": "stripe_get_billing_meters_id",
  "aa82007e-1adf-4408-ab31-7306f6afee75": "stripe_post_billing_meters_id",
  "1b18c326-f952-43f4-899b-146226fc49e7": "stripe_post_billing_meters_id_deactivate",
  "c3359d25-8141-48d7-bd85-cc2777256c51": "stripe_get_billing_meters_id_event_summaries",
  "88864ded-03fb-446d-a277-26beab5e7cc5": "stripe_post_billing_meters_id_reactivate",
  "62620530-9d72-49af-a808-fbec768765be": "stripe_get_billing_portal_configurations",
  "128a3d61-ab4f-4914-8553-0c72a933d517": "stripe_post_billing_portal_configurations",
  "3966c382-7ade-408e-b22b-b530e861bf08": "stripe_get_billing_portal_configurations_configuration",
  "935cae83-e0af-4912-9519-cbbff56f5259": "stripe_post_billing_portal_configurations_configuration",
  "eba3ffff-05a6-453d-bb1d-4319fda9ae89": "stripe_post_billing_portal_sessions",
  "9f8889cb-8f04-4c21-9031-178d6d02620c": "stripe_get_charges",
  "cdbfe58d-08fa-4a44-8a28-a05b96b99048": "stripe_post_charges",
  "a0a1d37c-d88d-4c1a-a108-d9cb5b306224": "stripe_get_charges_search",
  "88ade440-0784-4843-9cf1-1fa63f8fe015": "stripe_get_charges_charge",
  "196651db-fe8b-4d33-8cdf-002d794f432b": "stripe_post_charges_charge",
  "92fb2e65-ae61-4ae4-a5b6-63ac3ecdcf4e": "stripe_post_charges_charge_capture",
  "34e53402-d7ea-47a9-b06e-9523db2b413e": "stripe_get_charges_charge_dispute",
  "cb9a0420-0a27-4c59-93f0-ccd857886747": "stripe_post_charges_charge_dispute",
  "81523c88-6ca3-4d01-9fed-2f1cad9e2278": "stripe_post_charges_charge_dispute_close",
  "9c4da163-443f-4e60-af72-3c70b383f0f0": "stripe_post_charges_charge_refund",
  "9fd49eb8-f04e-41bc-84b9-02b5ec185f10": "stripe_get_charges_charge_refunds",
  "c2890595-8fab-431c-b9d4-1c666fee8e2e": "stripe_post_charges_charge_refunds",
  "bcabf068-3351-4a68-8625-c27c64ac9695": "stripe_get_charges_charge_refunds_refund",
  "fd1a9ee8-2b11-413b-a5a0-3015ed74884b": "stripe_post_charges_charge_refunds_refund",
  "4b396ec4-6a81-4d83-881e-8afcffaf51a2": "stripe_get_checkout_sessions",
  "41ca3345-fb1a-4d83-9420-6e257bd241b0": "stripe_post_checkout_sessions",
  "e5d4e74d-5309-48b1-89d2-758d31c8f5b5": "stripe_get_checkout_sessions_session",
  "bf173f8f-fe9b-4039-b5af-aa2b8dcb30cc": "stripe_post_checkout_sessions_session",
  "f28d39fc-c89e-4c94-a462-07d8eddc0dcd": "stripe_post_checkout_sessions_session_expire",
  "32fe5f50-917b-446f-a16d-3085203bbbc0": "stripe_get_checkout_sessions_session_line_items",
  "5ebccc40-b9fb-455f-80bc-a6c974df63b9": "stripe_get_climate_orders",
  "af175763-2962-441b-be90-46ab4f5b1c84": "stripe_post_climate_orders",
  "c573c762-b7f0-4f9f-ac11-6f141575408a": "stripe_get_climate_orders_order",
  "ee5e844b-cf49-4735-9eca-38c508899111": "stripe_post_climate_orders_order",
  "c06a6a3c-65b7-48f7-b850-5a4254e81e70": "stripe_post_climate_orders_order_cancel",
  "a5e52e36-3600-4671-b508-fc54002712c4": "stripe_get_climate_products",
  "efd6236c-b6f5-4364-8c95-c79894565fee": "stripe_get_climate_products_product",
  "b5bb82a1-739f-42fa-9200-44da68f7d3c8": "stripe_get_climate_suppliers",
  "fefcdb45-5e7f-4610-ae10-5bef1b9447a9": "stripe_get_climate_suppliers_supplier",
  "62143f56-324e-42e2-a07e-383d085f8eb0": "stripe_get_confirmation_tokens_confirmation_token",
  "eff02ddc-72fb-4e43-a995-ab03087c8783": "stripe_get_country_specs",
  "aed24e6f-997b-454a-88e8-4085f5d2aab9": "stripe_get_country_specs_country",
  "0dc98702-0df6-43df-a11d-8530c9eb6641": "stripe_get_coupons",
  "13af113a-0257-4214-b2d4-65b13c610518": "stripe_post_coupons",
  "913878ce-0602-4b0f-aaeb-e863df1110ee": "stripe_delete_coupons_coupon",
  "956b631d-90b5-45b4-895d-fa28ceeecd5b": "stripe_get_coupons_coupon",
  "0b9df57c-1b1d-455c-9f74-2e0ab5e71e89": "stripe_post_coupons_coupon",
  "51c2348a-5771-43a0-9945-2fbbe715ce01": "stripe_get_credit_notes",
  "2bf8b266-da3a-460d-8b20-fb8d8c63475a": "stripe_post_credit_notes",
  "5131077d-fd54-487d-9736-8806995f3d69": "stripe_get_credit_notes_preview",
  "cbfd461f-9538-48a6-9b03-77e1442b3c80": "stripe_get_credit_notes_preview_lines",
  "247573b7-7a3d-4eef-95b9-8ae7d05738de": "stripe_get_credit_notes_credit_note_lines",
  "190e8a1f-4d93-4f34-a23e-d42c70b63f91": "stripe_get_credit_notes_id",
  "bf255c9a-894d-45cf-ac2d-d9e676b696e1": "stripe_post_credit_notes_id",
  "9cd9cf2b-e0fd-445b-9e8a-f516ba376d6c": "stripe_post_credit_notes_id_void",
  "faa0bde2-0f18-4f0f-8b7e-c830accc464a": "stripe_post_customer_sessions",
  "bad2719a-31cf-4b88-972f-93d16ed56eb5": "stripe_get_customers",
  "a4212956-efe4-4dcd-a25b-7c33069e96f7": "stripe_post_customers",
  "ae7868df-1243-4265-b58c-884ab19d06ea": "stripe_get_customers_search",
  "1fd696ec-3dfc-4a06-b0fd-d420d09be51a": "stripe_delete_customers_customer",
  "1dc718dc-d8b4-42a1-b480-f7ef4956a2bf": "stripe_get_customers_customer",
  "ff920b71-2e9f-424d-bb23-e64a545961f6": "stripe_post_customers_customer",
  "a7f996a0-79cc-483c-9ad5-b6d0b1cce9d0": "stripe_get_customers_customer_balance_transactions",
  "3d931aa2-02f3-4fda-9f39-24b47b7c6411": "stripe_post_customers_customer_balance_transactions",
  "73fb898f-8f42-49d6-8eab-62dc6213f80a": "stripe_get_customers_customer_balance_transactions_transaction",
  "37a9c1d1-2315-4d1b-9a2f-6bec3439f4d7": "stripe_post_customers_customer_balance_transactions_transaction",
  "6180df96-6fd5-4040-8e32-f8ba15e96b8f": "stripe_get_customers_customer_bank_accounts",
  "9425288f-c22b-4683-be88-0874e3aa861c": "stripe_post_customers_customer_bank_accounts",
  "512a5956-a61a-46d9-b92e-8e8a0fe928d1": "stripe_delete_customers_customer_bank_accounts_id",
  "54ccd0ba-a2fa-4f83-b7ac-0b649e16b7fb": "stripe_get_customers_customer_bank_accounts_id",
  "e8a58aa5-980e-41f0-bff2-cd2124872685": "stripe_post_customers_customer_bank_accounts_id",
  "d5e847c8-97db-447e-acbf-731bf10c6309": "stripe_post_customers_customer_bank_accounts_id_verify",
  "dcb18107-36e6-412e-9a32-44211e0c336c": "stripe_get_customers_customer_cards",
  "8ccfce9a-bbcf-468e-8d42-e20c8c6e37e1": "stripe_post_customers_customer_cards",
  "dafc70aa-69fb-4e3d-bfd4-219c4fa4b887": "stripe_delete_customers_customer_cards_id",
  "06fecc09-e823-45ac-a348-254637b4450d": "stripe_get_customers_customer_cards_id",
  "3db28d63-10b3-4a73-af89-d672678d848f": "stripe_post_customers_customer_cards_id",
  "3212798d-53b5-4a4c-b869-d3da236e5d94": "stripe_get_customers_customer_cash_balance",
  "89e2edad-a823-4ee5-af9e-fa6c5b59d3cc": "stripe_post_customers_customer_cash_balance",
  "e2b87a30-1cb6-4035-8720-bd21177ddd9c": "stripe_get_customers_customer_cash_balance_transactions",
  "a75f0333-d0f0-44d0-ad10-4ad18047246c": "stripe_get_customers_customer_cash_balance_transactions_transaction",
  "69aecdf5-b0d6-436c-9f61-582a9db5643e": "stripe_delete_customers_customer_discount",
  "311d049f-681c-4b97-893d-8e93f74bf6fa": "stripe_get_customers_customer_discount",
  "6a9c9696-22dd-4554-8a13-8fa3d95c53c2": "stripe_post_customers_customer_funding_instructions",
  "f7e04b9e-3a83-492c-9d31-eab3749dcfba": "stripe_get_customers_customer_payment_methods",
  "971cb62d-6bab-49df-85aa-3891d921736a": "stripe_get_customers_customer_payment_methods_payment_method",
  "46787a18-34da-499a-b4f2-b0f7de354e63": "stripe_get_customers_customer_sources",
  "be62a12e-229b-4624-83d3-6223949acaef": "stripe_post_customers_customer_sources",
  "1e3baed9-9222-4b9b-916b-82284602c02c": "stripe_delete_customers_customer_sources_id",
  "eb2b4a51-8075-4ea5-b7a2-48cd996c4ed0": "stripe_get_customers_customer_sources_id",
  "60b33cc6-291f-4ce7-8df5-8dd1036723dc": "stripe_post_customers_customer_sources_id",
  "24e1ea8d-058c-4402-aba6-1b830f451785": "stripe_post_customers_customer_sources_id_verify",
  "f3486069-1173-4bc2-b965-35c3fbbd8e5c": "stripe_get_customers_customer_subscriptions",
  "a94ec134-2e6b-4cd8-a774-622fe8b346fd": "stripe_post_customers_customer_subscriptions",
  "6b0f1439-00d2-40b3-975f-cef3ebd3dd5c": "stripe_delete_customers_customer_subscriptions_subscription_exposed_id",
  "29e4caad-dda4-4fbb-8bbc-c99d059f3e22": "stripe_get_customers_customer_subscriptions_subscription_exposed_id",
  "7ca2f9c9-c0ca-4d83-8744-9f372ca9caab": "stripe_post_customers_customer_subscriptions_subscription_exposed_id",
  "f26cda75-7266-4a5c-8e38-1061809e6bc0": "stripe_delete_customers_customer_subscriptions_subscription_exposed_id_discount",
  "7e79706d-784c-4908-88f8-91ccb9b751e2": "stripe_get_customers_customer_subscriptions_subscription_exposed_id_discount",
  "5e6b4f3c-7504-47b8-8c99-5245cf7e0929": "stripe_get_customers_customer_tax_ids",
  "d130a819-f98a-4347-ac48-279952da278a": "stripe_post_customers_customer_tax_ids",
  "9d9977be-a01a-4857-88c0-5cd63b8f1834": "stripe_delete_customers_customer_tax_ids_id",
  "fb0ce2b1-ead8-427d-877b-f4dbd481eb6e": "stripe_get_customers_customer_tax_ids_id",
  "647eb24d-3c97-474a-befa-101bfbf7dbc3": "stripe_get_disputes",
  "862edf62-6832-41bc-be6d-f82adfaa5c93": "stripe_get_disputes_dispute",
  "6fa87df5-c0a2-4fc9-b8a0-8cba006c5abe": "stripe_post_disputes_dispute",
  "09b17cc5-5eb3-4f91-920f-c0527f37eb73": "stripe_post_disputes_dispute_close",
  "8969c07d-09b7-4d3e-a732-2f6e8fc16304": "stripe_get_entitlements_active_entitlements",
  "3c24a14a-da14-45e6-8f13-17949557323a": "stripe_get_entitlements_active_entitlements_id",
  "45d0f3b0-7ec4-4307-869e-aecdd9ab9caf": "stripe_get_entitlements_features",
  "7bea13d2-6a49-4a63-9079-0587dfc7d20b": "stripe_post_entitlements_features",
  "0845a2aa-ee66-4e41-86cd-b3f2fc0747a5": "stripe_get_entitlements_features_id",
  "080fa4b5-0386-4499-9f6b-21f8ae49a456": "stripe_post_entitlements_features_id",
  "a160960e-eaaa-4151-a9a3-20b844db791c": "stripe_post_ephemeral_keys",
  "ac8efd59-8b73-4708-ae3b-0adfbe6ab668": "stripe_delete_ephemeral_keys_key",
  "73e9de23-0c98-4df2-b4fc-88005abed759": "stripe_get_events",
  "e22c5f79-eb4f-4a2c-9506-0b4c545e3f5e": "stripe_get_events_id",
  "e2395110-b774-4c2c-9054-6444558c4554": "stripe_get_exchange_rates",
  "48d8664a-db71-4469-bbcf-b9355216320a": "stripe_get_exchange_rates_rate_id",
  "eb7700fc-eb23-4a75-9ece-d1527cacd80f": "stripe_get_file_links",
  "98d0173f-c851-470d-900a-1c4ce3be5c87": "stripe_post_file_links",
  "baade64c-e663-4b41-ae33-2ba245576f48": "stripe_get_file_links_link",
  "be9da96e-2ed3-4b51-9c4d-adfe143292e5": "stripe_post_file_links_link",
  "311be310-2e74-4d7f-b20d-c77a571e846c": "stripe_get_files",
  "e61bf64a-a457-4db7-b365-22b646da833f": "stripe_post_files",
  "f59e4653-4956-4ec5-9e61-914c1a829db6": "stripe_get_files_file",
  "9cf80237-8c92-4d03-a0f0-1ddd47d9cec9": "stripe_get_financial_connections_accounts",
  "e89b01a1-cad1-439a-8299-829180f1374d": "stripe_get_financial_connections_accounts_account",
  "b443b3d6-3e51-4ffb-9e72-ede8388edea3": "stripe_post_financial_connections_accounts_account_disconnect",
  "1e041af7-25c6-4caa-9134-0293a0ab3305": "stripe_get_financial_connections_accounts_account_owners",
  "cc4938cb-8385-4b9b-a13e-c3adc6870e76": "stripe_post_financial_connections_accounts_account_refresh",
  "72c01af8-0537-4bc5-bc90-6b034904b049": "stripe_post_financial_connections_accounts_account_subscribe",
  "0482e244-5ca2-40f8-b061-86c6a3be3bd9": "stripe_post_financial_connections_accounts_account_unsubscribe",
  "6add7574-a94c-4270-b259-4e45333f4c5a": "stripe_post_financial_connections_sessions",
  "c7fa917c-1ebd-46a8-b83c-0b96303e05ef": "stripe_get_financial_connections_sessions_session",
  "3a5f1492-46f7-46d5-94a1-8758b87b1624": "stripe_get_financial_connections_transactions",
  "3b14af28-0367-49c5-8d7f-d6ea4e6c6724": "stripe_get_financial_connections_transactions_transaction",
  "8d4efebc-36bd-4a05-89ce-e91da2945cbe": "stripe_get_forwarding_requests",
  "e21df603-918c-4085-bdee-88ee7d41bdc3": "stripe_post_forwarding_requests",
  "66f596d0-ad72-49ba-89c7-88b5464e0db8": "stripe_get_forwarding_requests_id",
  "f15cfb78-09cc-4822-a093-c4e371f10df3": "stripe_get_identity_verification_reports",
  "6f02f1ac-2207-4760-b05b-e8b7dad67d54": "stripe_get_identity_verification_reports_report",
  "e30bd071-9ba2-4ba3-a3fb-4ed0d29acef5": "stripe_get_identity_verification_sessions",
  "8e6592e4-28ae-4f83-9041-f0dd41fe6721": "stripe_post_identity_verification_sessions",
  "9e03bad9-5cb1-435d-9b27-aba3810c1ac8": "stripe_get_identity_verification_sessions_session",
  "2b920f92-708c-47f7-b9dd-805503e04cc7": "stripe_post_identity_verification_sessions_session",
  "b132330c-3fa9-42f1-adf6-87f5b2826c17": "stripe_post_identity_verification_sessions_session_cancel",
  "59f4bd11-1e5c-4a2a-bae6-e397aaa9b982": "stripe_post_identity_verification_sessions_session_redact",
  "a9c84adc-49a3-43c6-8623-0c031d88ee75": "stripe_get_invoice_rendering_templates",
  "ddb87535-be91-4db1-8eb8-892a5976fe09": "stripe_get_invoice_rendering_templates_template",
  "ac157c68-40a8-4132-8cb5-69abc849c810": "stripe_post_invoice_rendering_templates_template_archive",
  "cb7e4a19-1b71-44f0-84c1-750796425718": "stripe_post_invoice_rendering_templates_template_unarchive",
  "3119f89a-59bc-4c29-a634-cfdcf3196cbc": "stripe_get_invoiceitems",
  "6afdbf53-b00e-47df-b72b-7ff73ff078d8": "stripe_post_invoiceitems",
  "f8cb7b92-1132-4ed0-ad69-21157f375fee": "stripe_delete_invoiceitems_invoiceitem",
  "5e29c83a-4e67-4259-be26-2210ec0c950c": "stripe_get_invoiceitems_invoiceitem",
  "5f6129da-7b26-4bdf-8554-5f4e6f9991a4": "stripe_post_invoiceitems_invoiceitem",
  "e215b485-3aff-4dfa-83b2-f31e0eb271af": "stripe_get_invoices",
  "6ae139e3-9ae3-481e-94b7-7a99ac6732aa": "stripe_post_invoices",
  "3b268e61-fc21-455d-a278-32605f350696": "stripe_post_invoices_create_preview",
  "69a583d3-24cb-47fb-af61-d7ef7971bd7a": "stripe_get_invoices_search",
  "75d2f1b2-3c49-4d81-ab0d-b7fc049a957c": "stripe_get_invoices_upcoming",
  "f9c817dd-076c-4a38-882d-4747cfa31145": "stripe_get_invoices_upcoming_lines",
  "e7250a2c-8ae9-4227-90d4-d46226a4374d": "stripe_delete_invoices_invoice",
  "366354fe-867b-4e0b-b38f-3bebc266ba6d": "stripe_get_invoices_invoice",
  "a0c52558-f0ce-47af-99d9-530d575e0b5e": "stripe_post_invoices_invoice",
  "9ecf5add-82a5-4a4d-95eb-101cd77dcf83": "stripe_post_invoices_invoice_add_lines",
  "23c30f37-7c17-4b97-bb8f-fe61e1fa8fa4": "stripe_post_invoices_invoice_finalize",
  "34cfd7a5-0602-4b79-b8a9-11cf12723549": "stripe_get_invoices_invoice_lines",
  "91e031ef-cbc9-4cb1-8121-5398b28c348f": "stripe_post_invoices_invoice_lines_line_item_id",
  "21a8ccfe-37d2-48e3-8d45-9cd51de5e693": "stripe_post_invoices_invoice_mark_uncollectible",
  "c5e4f47a-2dce-4b67-8bb3-598640986c78": "stripe_post_invoices_invoice_pay",
  "e40a93bc-ae74-4c01-87ee-a0007f6c7565": "stripe_post_invoices_invoice_remove_lines",
  "572e59a4-c95c-4e14-8ddb-89571409bdad": "stripe_post_invoices_invoice_send",
  "c006a2a1-ff43-48b7-807d-e0f953e3253e": "stripe_post_invoices_invoice_update_lines",
  "fe7a5ddb-b65f-4307-bbbe-8b36b7a25ebe": "stripe_post_invoices_invoice_void",
  "fccca908-cad0-4b0c-aed2-2645070ddc1b": "stripe_get_issuing_authorizations",
  "e6f25324-ef50-4890-b592-5d41ea76d8b6": "stripe_get_issuing_authorizations_authorization",
  "c6eb7679-4acc-4816-9e95-0a8329d97a9d": "stripe_post_issuing_authorizations_authorization",
  "dc3c5b63-ab89-498c-8269-71ce8ae72c17": "stripe_post_issuing_authorizations_authorization_approve",
  "16731797-adab-42b2-9e25-1a2e18810536": "stripe_post_issuing_authorizations_authorization_decline",
  "903ad034-119b-4f23-96e5-328d2682f866": "stripe_get_issuing_cardholders",
  "1c559a0a-4763-488b-92b2-9ed51032208c": "stripe_post_issuing_cardholders",
  "304ebed7-2438-49ab-af32-3d04816fa63f": "stripe_get_issuing_cardholders_cardholder",
  "48d2271c-d866-4488-8b88-d93d8a48e683": "stripe_post_issuing_cardholders_cardholder",
  "b82e9e78-f478-427f-9e3f-db9e6331badc": "stripe_get_issuing_cards",
  "ff89a8fb-fb34-41dc-aa8e-de5503d16b9b": "stripe_post_issuing_cards",
  "d12d4015-d1e6-40bc-9825-7af3cf7ce0e3": "stripe_get_issuing_cards_card",
  "2789cfc0-e8be-446a-9d53-ad5b0754346f": "stripe_post_issuing_cards_card",
  "ec7b5cc6-b40f-4aae-b948-ff5d8e3936e6": "stripe_get_issuing_disputes",
  "1311b263-de65-4f00-b81c-c2102f3a2b82": "stripe_post_issuing_disputes",
  "cb0a99e3-aa29-4e1d-a2f6-911621fe8c04": "stripe_get_issuing_disputes_dispute",
  "27178d44-b691-4f07-bfc6-a82c1e59f868": "stripe_post_issuing_disputes_dispute",
  "5f442daa-b1b9-4568-add9-8a4fd3cd304f": "stripe_post_issuing_disputes_dispute_submit",
  "7e8f0240-a6cb-4a05-b5b8-a9ad5de3e970": "stripe_get_issuing_personalization_designs",
  "0213d185-6b18-4cc5-9526-d83a3ce3219d": "stripe_post_issuing_personalization_designs",
  "bb276c56-86d6-4383-8908-89ff5b997925": "stripe_get_issuing_personalization_designs_personalization_design",
  "627d85bb-33a6-46c5-8121-ffbec124fea5": "stripe_post_issuing_personalization_designs_personalization_design",
  "f8abbf81-df31-464d-be48-bb6f4fc4c6ca": "stripe_get_issuing_physical_bundles",
  "a8aae5de-0747-482a-be59-388fa99ce9f2": "stripe_get_issuing_physical_bundles_physical_bundle",
  "2314aef5-9aa9-4a6f-a107-597759363c69": "stripe_get_issuing_settlements_settlement",
  "210de1d8-255f-4424-ba42-cd4b1449c660": "stripe_post_issuing_settlements_settlement",
  "24aba854-ba82-4e13-b0d2-c71cd49f84eb": "stripe_get_issuing_tokens",
  "8ed8d8f2-14a6-414c-b5f0-6b8d131eb9d8": "stripe_get_issuing_tokens_token",
  "80721f47-7d76-417a-bdde-de34e23b2708": "stripe_post_issuing_tokens_token",
  "7a055242-d511-4ca6-847f-87045def2ab3": "stripe_get_issuing_transactions",
  "f7b1f583-eb66-4635-8349-7ea078b72e96": "stripe_get_issuing_transactions_transaction",
  "7dbfe742-88c7-4a62-bbe8-b54319a25b32": "stripe_post_issuing_transactions_transaction",
  "b713f8dc-b739-4842-9978-e029bb211018": "stripe_post_link_account_sessions",
  "077b7155-b235-4e0b-b0b7-4a3a9f215204": "stripe_get_link_account_sessions_session",
  "04c9ae8d-6005-4af9-b64f-92c213e64aae": "stripe_get_linked_accounts",
  "adbc59ce-b4d4-4685-87af-1341023311df": "stripe_get_linked_accounts_account",
  "df32f1fb-7da3-4117-a14b-6a6962212a3f": "stripe_post_linked_accounts_account_disconnect",
  "dc150350-d43a-4e37-acc5-e9b85420136b": "stripe_get_linked_accounts_account_owners",
  "e1f4dd61-d44b-4db5-8b9c-f2f0a31a0b5b": "stripe_post_linked_accounts_account_refresh",
  "2f755f2e-71c6-49e4-9f41-9988a1026135": "stripe_get_mandates_mandate",
  "2c0e19e3-ab7d-4cbb-a027-ed1e0307ea4f": "stripe_get_payment_intents",
  "58277f02-9222-4733-9397-a000058d2bd8": "stripe_post_payment_intents",
  "405b89ef-ed4e-4a7c-8752-8b3b84d87cbf": "stripe_get_payment_intents_search",
  "539c6608-b4bb-4211-bf20-888c66850753": "stripe_get_payment_intents_intent",
  "bcee356a-85bb-4904-9d1f-b2d758e182cc": "stripe_post_payment_intents_intent",
  "a4435416-dad1-4321-8b16-a3ad5ef11a3a": "stripe_post_payment_intents_intent_apply_customer_balance",
  "d39e034d-2dd2-4110-805c-f18136357baa": "stripe_post_payment_intents_intent_cancel",
  "d46a1490-a0a5-45bc-a06c-42f74988997d": "stripe_post_payment_intents_intent_capture",
  "41f2d5d1-f3f6-4e0f-ab05-1208d7d1cd19": "stripe_post_payment_intents_intent_confirm",
  "52494655-16b7-4a7a-b0f0-70e6f0d907a2": "stripe_post_payment_intents_intent_increment_authorization",
  "d2f78a24-e8db-4fb7-b6de-16dddee7fe40": "stripe_post_payment_intents_intent_verify_microdeposits",
  "f18d313b-cab0-4a2a-8507-1100cdd7f1fe": "stripe_get_payment_links",
  "7cf28fc7-9dc2-4129-9a54-c6d73f3cbb1a": "stripe_post_payment_links",
  "cfa0df5b-082d-4157-9a86-6deeaa1829d1": "stripe_get_payment_links_payment_link",
  "f2250e95-884d-41bc-a0b1-4cbb14b6edb7": "stripe_post_payment_links_payment_link",
  "f95ae275-7989-4510-b37f-5df3b117e1bf": "stripe_get_payment_links_payment_link_line_items",
  "8b92260c-b7de-4220-809c-32354fa52137": "stripe_get_payment_method_configurations",
  "1c408976-48e9-4433-91e4-0cc577b01510": "stripe_post_payment_method_configurations",
  "19840630-0376-492a-9660-e14c4c8112e3": "stripe_get_payment_method_configurations_configuration",
  "35b8655a-96e0-4fd5-b02a-03cf9d1d5ada": "stripe_post_payment_method_configurations_configuration",
  "2e57362b-0886-41d2-9224-b95f7eb10bad": "stripe_get_payment_method_domains",
  "a400495c-9e87-42ef-874e-4742be3eeae3": "stripe_post_payment_method_domains",
  "72d915fc-5105-42c0-961a-85e1bb70b8e1": "stripe_get_payment_method_domains_payment_method_domain",
  "bcf31138-600a-492f-92b7-b6eda0c6e4cf": "stripe_post_payment_method_domains_payment_method_domain",
  "09f1c702-83a1-4fc1-8c7c-5aef206c167c": "stripe_post_payment_method_domains_payment_method_domain_validate",
  "eb872270-12f9-4c40-a4de-350b2dca4a3d": "stripe_get_payment_methods",
  "9b28a3c3-0263-4731-a89f-78e407ff9abb": "stripe_post_payment_methods",
  "9d187635-817a-4e98-9ee9-54784059346a": "stripe_get_payment_methods_payment_method",
  "4e2893eb-84f4-4c60-ab5f-073a9aa0e9b3": "stripe_post_payment_methods_payment_method",
  "412ea829-287b-41f4-a4b9-2460183a428d": "stripe_post_payment_methods_payment_method_attach",
  "0135755f-7e23-4382-9068-5ee867b39c3a": "stripe_post_payment_methods_payment_method_detach",
  "59321deb-464a-4954-babc-390008f81e42": "stripe_get_payouts",
  "5caf873c-43e7-4277-97cf-838f48cdba4d": "stripe_post_payouts",
  "bda7b1b2-e54f-4188-b358-e1580bf1e26c": "stripe_get_payouts_payout",
  "99e7d36d-4f16-4264-b8fd-12e4e3488579": "stripe_post_payouts_payout",
  "42ea5cd0-0209-451e-8738-029a82336c8f": "stripe_post_payouts_payout_cancel",
  "0da06d3b-5e60-490b-ad03-92bf0644c2b4": "stripe_post_payouts_payout_reverse",
  "72ae32a6-b43a-4537-8934-a0bde8a81a8c": "stripe_get_plans",
  "fb2b4c94-9184-4d99-a0e1-031b90f5698a": "stripe_post_plans",
  "0e372f05-0d31-428b-ba0b-25b089624772": "stripe_delete_plans_plan",
  "72c83042-1a52-482e-8af1-0f7e2d64ad59": "stripe_get_plans_plan",
  "a9b80922-450c-4186-9378-b9362d27aeaa": "stripe_post_plans_plan",
  "b7ae87a6-a898-43e6-aafb-323a7bcb9302": "stripe_get_prices",
  "2aca5943-3bf3-416e-aca4-a3524c821983": "stripe_post_prices",
  "204729a0-6e3b-42e1-9beb-2fa2dab5cee4": "stripe_get_prices_search",
  "b9495b34-4ae8-472e-8790-ef345df44cef": "stripe_get_prices_price",
  "b8dd76f1-f3e9-48e1-838e-cdb262e497af": "stripe_post_prices_price",
  "02727a3a-91df-48e3-ae06-28aef5f7237e": "stripe_get_products",
  "3057d709-b0ee-4f4b-93e7-d218613c165f": "stripe_post_products",
  "265a7442-d03f-41f1-b78c-9fb5ddbe7c4f": "stripe_get_products_search",
  "9e54cce4-1f6a-4b21-b636-170376a8e53e": "stripe_delete_products_id",
  "f3633052-1c13-4c8a-ae00-77160ec47a03": "stripe_get_products_id",
  "2bb0d830-4e44-48f0-969c-e758d7b586a5": "stripe_post_products_id",
  "a024aaaa-d249-4d1f-a6a2-69b9d5a99cd8": "stripe_get_products_product_features",
  "88d0e151-bb01-4ff9-8328-6a6c42df1623": "stripe_post_products_product_features",
  "43cca5d5-c67e-4201-a804-bda6ea89bf4d": "stripe_delete_products_product_features_id",
  "d034e866-ef60-472c-b8c0-27676f096815": "stripe_get_products_product_features_id",
  "5e9f44ca-a58c-4f2b-875e-dffd4345b7f6": "stripe_get_promotion_codes",
  "160bfd24-a900-4e6f-be84-26afc667d664": "stripe_post_promotion_codes",
  "b2453edc-c6e8-4793-b8a9-2ed8e8c929a6": "stripe_get_promotion_codes_promotion_code",
  "6b869a0c-df34-447e-9922-438fdc7570cf": "stripe_post_promotion_codes_promotion_code",
  "3594d5a7-eb9a-45cf-aa2d-4d5549b00092": "stripe_get_quotes",
  "007e14d8-8910-495e-93fa-4c5ec8b14043": "stripe_post_quotes",
  "127a0cff-a058-414b-8875-b6fb0151c565": "stripe_get_quotes_quote",
  "7251df73-8f05-492c-b0d7-0d90dac1820c": "stripe_post_quotes_quote",
  "7bc44696-f194-4a05-9bbf-63825a44d152": "stripe_post_quotes_quote_accept",
  "64d03868-427a-41ac-94f5-5ebcfdf99e51": "stripe_post_quotes_quote_cancel",
  "6987c3d7-cdb8-4e57-8feb-b79e4c7eea08": "stripe_get_quotes_quote_computed_upfront_line_items",
  "b37b50d1-6c27-42ab-a640-bc92df99530a": "stripe_post_quotes_quote_finalize",
  "495f8595-83a1-4b9e-930e-b1c1368ce8c6": "stripe_get_quotes_quote_line_items",
  "18e87d2d-eb37-4662-bb76-cc91f4f83a5d": "stripe_get_quotes_quote_pdf",
  "48ebf479-3190-4c1e-a70d-3c0b26ccb330": "stripe_get_radar_early_fraud_warnings",
  "f98c997c-6738-4289-9bde-3c0fa6dc633a": "stripe_get_radar_early_fraud_warnings_early_fraud_warning",
  "3ea8ae52-af3e-48e1-af0e-feae44878f3a": "stripe_get_radar_value_list_items",
  "74853aee-5425-4a25-ab92-b436497c5181": "stripe_post_radar_value_list_items",
  "f868fce7-fecd-4224-b7c4-5a077ea4ddaf": "stripe_delete_radar_value_list_items_item",
  "a001c986-03fc-4dc4-a850-e1cdc64c1cd5": "stripe_get_radar_value_list_items_item",
  "2f748c61-5a86-4887-8e35-621065baa1a8": "stripe_get_radar_value_lists",
  "119ded16-d747-44b9-85ac-b6ec3ea9f89c": "stripe_post_radar_value_lists",
  "c55309f1-3b88-4e99-b62c-b714ffc46f3b": "stripe_delete_radar_value_lists_value_list",
  "6ac7e6d0-0acd-46a3-9c5f-4fc3c3e36005": "stripe_get_radar_value_lists_value_list",
  "38c0f059-2860-4c4c-ad2d-a2231eac5e44": "stripe_post_radar_value_lists_value_list",
  "3a1d6028-f750-4786-a677-325cd505293f": "stripe_get_refunds",
  "1073a4cf-5c24-49bb-9831-31676a6d9b99": "stripe_post_refunds",
  "b693f908-b4e7-4a1c-ba27-89fa87ea2589": "stripe_get_refunds_refund",
  "1d589234-6f86-46ba-916d-86398d2dcbeb": "stripe_post_refunds_refund",
  "bb5e314e-929e-4b0f-a96a-475734ac8822": "stripe_post_refunds_refund_cancel",
  "53b6762c-bd9f-4c2e-ba64-325ca38d9faa": "stripe_get_reporting_report_runs",
  "6e06ee74-f260-4358-bc56-c4db11a98055": "stripe_post_reporting_report_runs",
  "299383f6-93a1-44fd-9e20-b8993d2798d8": "stripe_get_reporting_report_runs_report_run",
  "facc50fe-265a-4269-b4c9-be127c3d5594": "stripe_get_reporting_report_types",
  "d0103b5c-dd96-44c1-89a6-ce7e2824f91a": "stripe_get_reporting_report_types_report_type",
  "1636ef28-6885-4275-9b41-d8edc2c598cf": "stripe_get_reviews",
  "8ea9e268-4f7a-449c-930c-7e543809c3ee": "stripe_get_reviews_review",
  "a71c4689-8024-47ab-8161-75985c12a577": "stripe_post_reviews_review_approve",
  "85df838e-a22f-43f3-8af2-c68054a30fca": "stripe_get_setup_attempts",
  "1f482776-9473-44dd-bf34-d89439083f17": "stripe_get_setup_intents",
  "eb1ff11a-89e1-4884-9af2-6411ed9e099d": "stripe_post_setup_intents",
  "6824ac43-1cb3-4b97-b078-b68a3368bd54": "stripe_get_setup_intents_intent",
  "1248416f-e6bf-4c79-9704-5db341f756c3": "stripe_post_setup_intents_intent",
  "6029cff3-14fb-40da-b9c2-056c5aca52f3": "stripe_post_setup_intents_intent_cancel",
  "60a88ac8-9066-4a44-bcd4-a83e43fc3070": "stripe_post_setup_intents_intent_confirm",
  "6b4c71c8-045d-4456-9ebf-00263f575a4a": "stripe_post_setup_intents_intent_verify_microdeposits",
  "3a4f2f1a-fc75-44d5-870e-de891f1b0930": "stripe_get_shipping_rates",
  "4e8f52d2-287f-4acb-8c83-21d294223c7c": "stripe_post_shipping_rates",
  "82b0ad48-b872-4087-ba39-17bac0a545c6": "stripe_get_shipping_rates_shipping_rate_token",
  "af44d0bd-c655-4ee8-998c-bb72d0662fee": "stripe_post_shipping_rates_shipping_rate_token",
  "db462ac5-1be1-4f4a-bc9b-9c5600acb4cb": "stripe_get_sigma_scheduled_query_runs",
  "578715bd-30f5-4c88-a216-c866d87451dd": "stripe_get_sigma_scheduled_query_runs_scheduled_query_run",
  "e1ecec44-6cc7-4baf-886b-9492475b5f63": "stripe_post_sources",
  "3e1c6190-22ea-46e9-8252-7fff0468cddb": "stripe_get_sources_source",
  "bf5f96d9-62c1-4a0b-ad9c-e7f5d735dfc8": "stripe_post_sources_source",
  "52f80228-d658-403b-85d4-bfdedfaf1e0d": "stripe_get_sources_source_mandate_notifications_mandate_notification",
  "a66a04b4-d98d-47b4-8fe4-ac86694c31af": "stripe_get_sources_source_source_transactions",
  "ddc35f41-c7e3-4777-b0b9-eb17188a2b6f": "stripe_get_sources_source_source_transactions_source_transaction",
  "706a59a3-4fc9-4bc8-87d9-65ecd7d7d491": "stripe_post_sources_source_verify",
  "82a9490e-fc8b-4482-b04e-b5df95244d3b": "stripe_get_subscription_items",
  "6e25f021-8b28-4f91-8556-5cabaf62bcb5": "stripe_post_subscription_items",
  "ac597e3d-9319-4ee0-bbc6-94b8e550f071": "stripe_delete_subscription_items_item",
  "6207ce04-237e-4567-abf3-99a0d945799a": "stripe_get_subscription_items_item",
  "165d0728-c7b3-4511-bd38-3a3ed06bf71e": "stripe_post_subscription_items_item",
  "0c0af609-d8c4-45a4-9d65-e2051cec325f": "stripe_get_subscription_items_subscription_item_usage_record_summaries",
  "ad6a417e-8089-4d23-be12-36b39487c08a": "stripe_post_subscription_items_subscription_item_usage_records",
  "795df04d-bce3-48c9-bea3-bacd97eee8f4": "stripe_get_subscription_schedules",
  "e5ec4876-24a3-4214-bbf9-724332ec45d8": "stripe_post_subscription_schedules",
  "671b421d-70ef-4b98-826d-4f442a16a61e": "stripe_get_subscription_schedules_schedule",
  "58030ef8-b80d-42b2-8059-a4511d795b4c": "stripe_post_subscription_schedules_schedule",
  "331ca4e8-6f2a-4106-83f1-a2718362ec8b": "stripe_post_subscription_schedules_schedule_cancel",
  "76daf682-41d1-4b4d-92e8-cc14630c9e44": "stripe_post_subscription_schedules_schedule_release",
  "037a7ce2-bd7a-44e3-b30a-6dc282c4a326": "stripe_get_subscriptions",
  "840d5bb9-3270-46e0-bdb1-f1fe697d0d6c": "stripe_post_subscriptions",
  "25457282-8fd7-4d8a-99d5-eabe95ed3f47": "stripe_get_subscriptions_search",
  "748b45d5-0fa0-4e03-9989-7544bc7804eb": "stripe_delete_subscriptions_subscription_exposed_id",
  "5f0d8eac-9ecd-44f3-9d2e-d559326f9655": "stripe_get_subscriptions_subscription_exposed_id",
  "a8ce2619-a222-4c5c-a434-cedd439798fe": "stripe_post_subscriptions_subscription_exposed_id",
  "3dac2bfd-2015-49de-85b1-202c3715997c": "stripe_delete_subscriptions_subscription_exposed_id_discount",
  "c88b9a04-aec8-4a75-b888-79ecc38f4505": "stripe_post_subscriptions_subscription_resume",
  "14869cd8-b6c0-4ee6-a799-cad2a04446f5": "stripe_post_tax_calculations",
  "490623ca-8c50-4e52-bacf-1e1da7baf57b": "stripe_get_tax_calculations_calculation",
  "c61a0cbd-9932-4e32-b5d2-2f7ae92e0d1c": "stripe_get_tax_calculations_calculation_line_items",
  "e63dca49-763a-4441-8655-0edc55f06e2b": "stripe_get_tax_registrations",
  "37d0caa9-f3cb-4a02-8310-1a2db5782ed6": "stripe_post_tax_registrations",
  "3968463f-796a-4908-b079-68619557e2b5": "stripe_get_tax_registrations_id",
  "1198ec3a-bb9a-4c3f-abc1-85ae17389f89": "stripe_post_tax_registrations_id",
  "168d1035-2c8d-4a9b-93d3-d82b65eb3657": "stripe_get_tax_settings",
  "6c1cdd2c-5ddb-4fee-b6cf-1e540250b609": "stripe_post_tax_settings",
  "26cbfe0b-05c9-4d24-ac7f-dece5ac237a6": "stripe_post_tax_transactions_create_from_calculation",
  "e4b0fd05-78f8-452e-93fe-01936aa5b011": "stripe_post_tax_transactions_create_reversal",
  "62cdee4f-0f26-4e4b-9d0e-1e3d79b9ffad": "stripe_get_tax_transactions_transaction",
  "2be9a841-903b-4b95-9185-1c869d25411a": "stripe_get_tax_transactions_transaction_line_items",
  "fa5bb00c-ca7b-48c2-8c41-9bae857f0940": "stripe_get_tax_codes",
  "6157a93c-11b4-4b9a-a1a9-35fc3b2932e1": "stripe_get_tax_codes_id",
  "0a311ba4-b28f-47e7-9aac-9a9c6e236a58": "stripe_get_tax_ids",
  "fa5839ff-ac37-43ae-86af-7c8d1383cbb3": "stripe_post_tax_ids",
  "ac900d56-2c30-40c0-b2ac-f7287a96800d": "stripe_delete_tax_ids_id",
  "b7121732-97b3-4af6-83c3-68197cdb2294": "stripe_get_tax_ids_id",
  "07d6353a-7875-4208-8ee3-79fdcdff6f7a": "stripe_get_tax_rates",
  "e918655c-8b20-4159-a154-3b6b78262900": "stripe_post_tax_rates",
  "e2808058-86fe-4b6c-94fe-ec6ab0908991": "stripe_get_tax_rates_tax_rate",
  "a580f08d-d244-4e8a-8385-a497e29743e0": "stripe_post_tax_rates_tax_rate",
  "a742d733-520d-4fa6-a8c1-e770aacaffc9": "stripe_get_terminal_configurations",
  "37472a3d-f602-4b8f-8f43-1925b803ce5d": "stripe_post_terminal_configurations",
  "9356808b-390a-4f56-b377-863212cfb255": "stripe_delete_terminal_configurations_configuration",
  "c8ae70e3-7c08-4287-ad6e-79ef4c2a51a4": "stripe_get_terminal_configurations_configuration",
  "3f4c80e1-c132-453b-abc4-8fdf1ec77b3d": "stripe_post_terminal_configurations_configuration",
  "c13c18cd-2681-4d28-b8b2-aea73bedf29f": "stripe_post_terminal_connection_tokens",
  "367eca90-b44c-4623-97f6-b70f6f7dc557": "stripe_get_terminal_locations",
  "ae2e2a7a-4b3c-4296-9932-d2549ac14e19": "stripe_post_terminal_locations",
  "741870ae-7678-49b4-94a2-5b2eac070c03": "stripe_delete_terminal_locations_location",
  "26d37ce6-059e-4f2e-905b-cbc9bb9a2748": "stripe_get_terminal_locations_location",
  "86f525be-9d4f-4bf0-9275-45a3e92c2cd2": "stripe_post_terminal_locations_location",
  "15483d4d-e37d-4423-b324-9b1342951d3b": "stripe_get_terminal_readers",
  "de75d452-1e8d-4aeb-a332-320ad3613951": "stripe_post_terminal_readers",
  "25bbf73e-8d92-4f1e-8001-df903a10f8e0": "stripe_delete_terminal_readers_reader",
  "7f864e5f-c388-4a01-ace2-80f3b1fc069e": "stripe_get_terminal_readers_reader",
  "bd3208aa-b897-456a-b9f0-699e8d3aef6b": "stripe_post_terminal_readers_reader",
  "efb2ae96-05e0-4133-9885-11f06943ebe3": "stripe_post_terminal_readers_reader_cancel_action",
  "823147fe-86c4-4935-ae2d-e0a4cd1c3c54": "stripe_post_terminal_readers_reader_process_payment_intent",
  "f9f3d101-6103-4d29-b1aa-b135a466354d": "stripe_post_terminal_readers_reader_process_setup_intent",
  "5d1b888a-b4ee-496e-8199-28e6e4724285": "stripe_post_terminal_readers_reader_refund_payment",
  "bcc66ef3-ee81-479d-98e5-c1e7b1c862cc": "stripe_post_terminal_readers_reader_set_reader_display",
  "2a2e4651-6e5c-4c2f-815c-95e7802cc0ae": "stripe_post_test_helpers_confirmation_tokens",
  "e07caac8-5887-446c-b6be-ac020510d84a": "stripe_post_test_helpers_customers_customer_fund_cash_balance",
  "dab6bb11-07bc-4735-8b9f-930aca83c809": "stripe_post_test_helpers_issuing_authorizations",
  "6dc212de-fbaa-4d9e-92f7-da9def795dbc": "stripe_post_test_helpers_issuing_authorizations_authorization_capture",
  "dc4a2bff-7852-41d7-9c4c-c52a47254de5": "stripe_post_test_helpers_issuing_authorizations_authorization_expire",
  "13f2e121-9e85-4a6f-8cc5-46adad66f836": "stripe_post_test_helpers_issuing_authorizations_authorization_finalize_amount",
  "cbcceaf6-e0ee-4ae9-883a-50b428a86ccc": "stripe_post_test_helpers_issuing_authorizations_authorization_increment",
  "c9dd1581-3989-402e-994c-814aa67a5ff2": "stripe_post_test_helpers_issuing_authorizations_authorization_reverse",
  "56705e8d-1c75-4f6b-ac3e-95dac5330e00": "stripe_post_test_helpers_issuing_cards_card_shipping_deliver",
  "295d157d-cddf-4483-bb50-8315ce2c41a4": "stripe_post_test_helpers_issuing_cards_card_shipping_fail",
  "d77cc101-395b-429b-913b-5dc93de6b3ff": "stripe_post_test_helpers_issuing_cards_card_shipping_return",
  "a6b4ea1d-c34d-4856-96bf-ba0770e7ad7a": "stripe_post_test_helpers_issuing_cards_card_shipping_ship",
  "94e20719-8864-4d0e-a8ae-b837b6e1779f": "stripe_post_test_helpers_issuing_cards_card_shipping_submit",
  "6cbe4d45-7bd3-4788-ae96-b96e6098814b": "stripe_post_test_helpers_issuing_personalization_designs_personalization_design_activate",
  "fe365e8a-7e72-4a47-accf-317fa18e4517": "stripe_post_test_helpers_issuing_personalization_designs_personalization_design_deactivate",
  "4d5b43c1-0f94-4fff-b237-943a197cd8ec": "stripe_post_test_helpers_issuing_personalization_designs_personalization_design_reject",
  "ec2fd26e-6572-4ed6-8f89-d0fb9e7633f0": "stripe_post_test_helpers_issuing_settlements",
  "dc0caa9c-4307-4ce9-8db0-724193b8d2da": "stripe_post_test_helpers_issuing_transactions_create_force_capture",
  "0227dbab-27d7-4aa2-a49a-5b422f39e0c0": "stripe_post_test_helpers_issuing_transactions_create_unlinked_refund",
  "69500374-0859-4ea7-af19-685330c3f14c": "stripe_post_test_helpers_issuing_transactions_transaction_refund",
  "97a9d93b-75c6-427d-891c-3ffe0ab0f274": "stripe_post_test_helpers_refunds_refund_expire",
  "3ae297dc-1175-4559-b060-9bb9ec28ec67": "stripe_post_test_helpers_terminal_readers_reader_present_payment_method",
  "5bd04241-3b3a-4bb5-87be-8637d27e05e8": "stripe_get_test_helpers_test_clocks",
  "51ef600d-ae98-4669-879e-f0fb3bcad436": "stripe_post_test_helpers_test_clocks",
  "e2de4cda-11ba-45a6-9d7e-787d8b6cbeb6": "stripe_delete_test_helpers_test_clocks_test_clock",
  "8183904e-df6b-4905-a7bf-4e764275a806": "stripe_get_test_helpers_test_clocks_test_clock",
  "fb3148b1-55b9-4b59-9e6b-6c00bcb59574": "stripe_post_test_helpers_test_clocks_test_clock_advance",
  "dc14e954-0a34-4e14-850e-ab0485362676": "stripe_post_test_helpers_treasury_inbound_transfers_id_fail",
  "8085d72d-06f3-4477-be95-92b684f5e10a": "stripe_post_test_helpers_treasury_inbound_transfers_id_return",
  "d830374e-fa1d-48c2-a648-8b1e91e6fce9": "stripe_post_test_helpers_treasury_inbound_transfers_id_succeed",
  "a23cfb72-07c3-4ac1-929e-b95065cd3031": "stripe_post_test_helpers_treasury_outbound_payments_id",
  "e3d24672-e705-41c9-8392-e9be26d91d23": "stripe_post_test_helpers_treasury_outbound_payments_id_fail",
  "7ee7b99d-c0b5-49a5-8ccd-7f83b274d0d6": "stripe_post_test_helpers_treasury_outbound_payments_id_post",
  "5b2ea304-c73f-4027-8ad2-88539d8c4fd8": "stripe_post_test_helpers_treasury_outbound_payments_id_return",
  "c8bd71da-9756-47de-99ed-8534b27160cf": "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer",
  "f31e6478-40f6-45de-8d76-3f0889285f26": "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer_fail",
  "ccdad07e-0ce7-4030-a12a-c2428362f403": "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer_post",
  "2f881ec5-1b23-41b0-be56-3cba68f6cbab": "stripe_post_test_helpers_treasury_outbound_transfers_outbound_transfer_return",
  "17d31778-3425-4309-899d-e2f5f7917d4a": "stripe_post_test_helpers_treasury_received_credits",
  "0f22a740-0f4e-4592-ab05-25c416ddaa8f": "stripe_post_test_helpers_treasury_received_debits",
  "5d09c036-a130-4e3b-b114-e219cf3e80fd": "stripe_post_tokens",
  "39a9f227-4706-4fc2-a329-36ee73b2f695": "stripe_get_tokens_token",
  "83cc0e03-4e77-4bfe-bb5c-0f72aa8daa4a": "stripe_get_topups",
  "f517b124-4999-43ef-8eb4-94c077961d35": "stripe_post_topups",
  "620b2c1f-6e24-4591-a2e8-e5c0403a0bc5": "stripe_get_topups_topup",
  "93bc9420-013b-4334-96d5-f67053487d98": "stripe_post_topups_topup",
  "47da25f4-07f1-4da8-9f0d-7816cc63a8fc": "stripe_post_topups_topup_cancel",
  "15a489e7-92cc-4835-8257-e11c9232f58b": "stripe_get_transfers",
  "d624a2ea-64cb-4614-b951-3915d79fe2e0": "stripe_post_transfers",
  "ef70b4fd-32ef-4027-8cf9-a6dc359800dc": "stripe_get_transfers_id_reversals",
  "0926b81f-8303-44c6-b6fe-5b36b7d33d00": "stripe_post_transfers_id_reversals",
  "adc4e103-5bb9-4b26-877c-b4c431c62c0d": "stripe_get_transfers_transfer",
  "202eba4a-7d0e-4dc2-b8b2-4ee3a3d3a953": "stripe_post_transfers_transfer",
  "69f364ee-bce9-4927-9981-ba8e3b06945b": "stripe_get_transfers_transfer_reversals_id",
  "d56bea50-c2d6-4f01-83ea-9235e03d325c": "stripe_post_transfers_transfer_reversals_id",
  "93706590-cb42-4040-8187-0de15080143e": "stripe_get_treasury_credit_reversals",
  "b5975996-c9fd-4f40-aa04-b6ff24058f7d": "stripe_post_treasury_credit_reversals",
  "94901627-cae9-4618-abd2-2aaa1298c677": "stripe_get_treasury_credit_reversals_credit_reversal",
  "9301d4b0-5ae8-45d4-9844-6c9e9a1b4e7e": "stripe_get_treasury_debit_reversals",
  "9718f8f1-b5be-41ed-80d1-9a5f14559a96": "stripe_post_treasury_debit_reversals",
  "8db75ac2-dbec-42ff-b196-d412614d6f2f": "stripe_get_treasury_debit_reversals_debit_reversal",
  "a235e12f-b871-4595-9342-11328d65142f": "stripe_get_treasury_financial_accounts",
  "4a2e7200-5c03-4c2b-9191-dd00ae61039a": "stripe_post_treasury_financial_accounts",
  "b7d7cb89-bc21-4fbc-b226-b2b6523796b1": "stripe_get_treasury_financial_accounts_financial_account",
  "136377b8-7376-4a9c-acf6-f8f12c4e718c": "stripe_post_treasury_financial_accounts_financial_account",
  "d3310119-a6f4-4afd-8e11-bb8169e09533": "stripe_get_treasury_financial_accounts_financial_account_features",
  "9ae0736b-4aca-478f-bc9b-f0c7a00c431f": "stripe_post_treasury_financial_accounts_financial_account_features",
  "8945f13a-bc0b-4853-86b6-c8cb09b82c7a": "stripe_get_treasury_inbound_transfers",
  "db742d53-5db4-485e-8e41-81efbc4886f8": "stripe_post_treasury_inbound_transfers",
  "12ca9fa1-58b8-4347-a18d-705462c659d6": "stripe_get_treasury_inbound_transfers_id",
  "940cc541-c159-42a5-aa52-8c7e357d605d": "stripe_post_treasury_inbound_transfers_inbound_transfer_cancel",
  "1681d5aa-cc76-4781-858b-c39dd23898bd": "stripe_get_treasury_outbound_payments",
  "857bda62-c199-4911-b6b4-f3cd402ef277": "stripe_post_treasury_outbound_payments",
  "7b7337f0-3404-4402-90de-6ef2081da6f6": "stripe_get_treasury_outbound_payments_id",
  "48243142-34b9-486e-9f9d-e7e606a198ce": "stripe_post_treasury_outbound_payments_id_cancel",
  "e9dddd0e-f191-45f1-b8e2-caa8bf74b533": "stripe_get_treasury_outbound_transfers",
  "e6769687-6f0e-4482-8ddf-b2cfd6783bfa": "stripe_post_treasury_outbound_transfers",
  "e361b930-4df2-4e24-b4a5-dc95c3814aad": "stripe_get_treasury_outbound_transfers_outbound_transfer",
  "12ef23b6-2819-4aaf-b9e2-687617da9949": "stripe_post_treasury_outbound_transfers_outbound_transfer_cancel",
  "e4be3857-63e7-47a2-ad26-3ddaf3403646": "stripe_get_treasury_received_credits",
  "308ca308-3bee-4575-9dfd-4913bc9852c5": "stripe_get_treasury_received_credits_id",
  "0da95007-d1b9-4dd8-884f-a4c1016a3335": "stripe_get_treasury_received_debits",
  "e08026e6-1dc3-44a9-aca2-72dbdfabc372": "stripe_get_treasury_received_debits_id",
  "6f7fc993-2bdc-47ff-8db0-51638960a6ca": "stripe_get_treasury_transaction_entries",
  "5b485e78-095d-4aef-bf03-5c15c88130d2": "stripe_get_treasury_transaction_entries_id",
  "00063776-2837-4e9d-98e9-f0376f8ee8e8": "stripe_get_treasury_transactions",
  "38685169-aa90-4388-ad31-67d3c2f7d42e": "stripe_get_treasury_transactions_id",
  "d5c747db-b220-4017-985c-14f80826f7db": "stripe_get_webhook_endpoints",
  "3c76f048-0cc9-46a7-8b16-a1d98cab0eef": "stripe_post_webhook_endpoints",
  "a4cd6d3e-d229-4638-be7b-896505610efa": "stripe_delete_webhook_endpoints_webhook_endpoint",
  "a0fa3319-2365-4182-821d-eabdd46d1c05": "stripe_get_webhook_endpoints_webhook_endpoint",
  "0deb580d-ca2e-4a8a-aa38-22ef0fb5fbf3": "stripe_post_webhook_endpoints_webhook_endpoint"
}
    
    @classmethod
    def get_id(cls, tool: str):
        return cls.id_lookup[tool]
    
    @classmethod
    def get_tool(cls, tool_id: str):
        return cls.tool_lookup[tool_id]